@@ -6,6 +6,7 @@
 
 #pragma once
 
+#include <algorithm>
 #include <array>
 #include <cassert>
 #include <complex>
@@ -15,10 +16,11 @@
 #include <dolfinx/graph/AdjacencyList.h>
 #include <numeric>
 #include <set>
+#include <span>
+#include <tuple>
 #include <type_traits>
 #include <utility>
 #include <vector>
-#include <xtl/xspan.hpp>
 
 #define MPICH_IGNORE_CXX_SEEK 1
 #include <mpi.h>
@@ -123,12 +125,6 @@ constexpr int index_owner(int size, std::size_t index, std::size_t N)
   }
 }
 
-/// @brief Return list of neighbors ranks (sources and destinations) for
-/// a neighborhood communicator.
-/// @param[in] comm Communicator with a neighborhood
-/// @return source ranks [0], destination ranks [1]
-std::array<std::vector<int>, 2> neighbors(MPI_Comm comm);
-
 /// @brief Determine incoming graph edges using the PCX consensus
 /// algorithm.
 ///
@@ -153,7 +149,7 @@ std::array<std::vector<int>, 2> neighbors(MPI_Comm comm);
 /// @param[in] edges Edges (ranks) from this rank (the caller).
 /// @return Ranks that have defined edges from them to this rank.
 std::vector<int> compute_graph_edges_pcx(MPI_Comm comm,
-                                         const xtl::span<const int>& edges);
+                                         const std::span<const int>& edges);
 
 /// @brief Determine incoming graph edges using the NBX consensus
 /// algorithm.
@@ -180,7 +176,7 @@ std::vector<int> compute_graph_edges_pcx(MPI_Comm comm,
 /// @param[in] edges Edges (ranks) from this rank (the caller).
 /// @return Ranks that have defined edges from them to this rank.
 std::vector<int> compute_graph_edges_nbx(MPI_Comm comm,
-                                         const xtl::span<const int>& edges);
+                                         const std::span<const int>& edges);
 
 /// @brief Distribute row data to 'post office' ranks.
 ///
@@ -203,7 +199,7 @@ std::vector<int> compute_graph_edges_nbx(MPI_Comm comm,
 /// for which the calling process is the post office
 template <typename T>
 std::pair<std::vector<std::int32_t>, std::vector<T>>
-distribute_to_postoffice(MPI_Comm comm, const xtl::span<const T>& x,
+distribute_to_postoffice(MPI_Comm comm, const std::span<const T>& x,
                          std::array<std::int64_t, 2> shape,
                          std::int64_t rank_offset);
 
@@ -230,8 +226,8 @@ distribute_to_postoffice(MPI_Comm comm, const xtl::span<const T>& x,
 /// @pre `shape1 > 0`
 template <typename T>
 std::vector<T> distribute_from_postoffice(
-    MPI_Comm comm, const xtl::span<const std::int64_t>& indices,
-    const xtl::span<const T>& x, std::array<std::int64_t, 2> shape,
+    MPI_Comm comm, const std::span<const std::int64_t>& indices,
+    const std::span<const T>& x, std::array<std::int64_t, 2> shape,
     std::int64_t rank_offset);
 
 /// @brief Distribute rows of a rectangular data array to ranks where
@@ -259,20 +255,8 @@ std::vector<T> distribute_from_postoffice(
 /// @pre `shape1 > 0`
 template <typename T>
 std::vector<T> distribute_data(MPI_Comm comm,
-                               const xtl::span<const std::int64_t>& indices,
-                               const xtl::span<const T>& x, int shape1);
-
-/// @brief Send in_values[n0] to neighbor process n0 and receive values
-/// from neighbor process n1 in out_values[n1].
-///
-/// @param[in] comm Neighborhood communicator
-/// @param[in] send_data The data to send to each rank.
-/// graph::AdjacencyList<T>::num_nodes should be equal to the number of
-/// neighbourhood out edges.
-/// @return Data received from incoming neighbourhood ranks.
-template <typename T>
-graph::AdjacencyList<T>
-neighbor_all_to_all(MPI_Comm comm, const graph::AdjacencyList<T>& send_data);
+                               const std::span<const std::int64_t>& indices,
+                               const std::span<const T>& x, int shape1);
 
 template <typename T>
 struct dependent_false : std::false_type
@@ -283,41 +267,41 @@ struct dependent_false : std::false_type
 template <typename T>
 constexpr MPI_Datatype mpi_type()
 {
-  if constexpr (std::is_same<T, float>::value)
+  if constexpr (std::is_same_v<T, float>)
     return MPI_FLOAT;
-  else if constexpr (std::is_same<T, double>::value)
+  else if constexpr (std::is_same_v<T, double>)
     return MPI_DOUBLE;
-  else if constexpr (std::is_same<T, std::complex<double>>::value)
+  else if constexpr (std::is_same_v<T, std::complex<double>>)
     return MPI_C_DOUBLE_COMPLEX;
-  else if constexpr (std::is_same<T, std::complex<float>>::value)
+  else if constexpr (std::is_same_v<T, std::complex<float>>)
     return MPI_C_FLOAT_COMPLEX;
-  else if constexpr (std::is_same<T, short int>::value)
+  else if constexpr (std::is_same_v<T, short int>)
     return MPI_SHORT;
-  else if constexpr (std::is_same<T, int>::value)
+  else if constexpr (std::is_same_v<T, int>)
     return MPI_INT;
-  else if constexpr (std::is_same<T, unsigned int>::value)
+  else if constexpr (std::is_same_v<T, unsigned int>)
     return MPI_UNSIGNED;
-  else if constexpr (std::is_same<T, long int>::value)
+  else if constexpr (std::is_same_v<T, long int>)
     return MPI_LONG;
-  else if constexpr (std::is_same<T, unsigned long>::value)
+  else if constexpr (std::is_same_v<T, unsigned long>)
     return MPI_UNSIGNED_LONG;
-  else if constexpr (std::is_same<T, long long>::value)
+  else if constexpr (std::is_same_v<T, long long>)
     return MPI_LONG_LONG;
-  else if constexpr (std::is_same<T, unsigned long long>::value)
+  else if constexpr (std::is_same_v<T, unsigned long long>)
     return MPI_UNSIGNED_LONG_LONG;
-  else if constexpr (std::is_same<T, bool>::value)
+  else if constexpr (std::is_same_v<T, bool>)
     return MPI_C_BOOL;
-  else if constexpr (std::is_same<T, std::int8_t>::value)
+  else if constexpr (std::is_same_v<T, std::int8_t>)
     return MPI_INT8_T;
   else
     // Issue compile time error
-    static_assert(!std::is_same<T, T>::value);
+    static_assert(!std::is_same_v<T, T>);
 }
 
 //---------------------------------------------------------------------------
 template <typename T>
 std::pair<std::vector<std::int32_t>, std::vector<T>>
-distribute_to_postoffice(MPI_Comm comm, const xtl::span<const T>& x,
+distribute_to_postoffice(MPI_Comm comm, const std::span<const T>& x,
                          std::array<std::int64_t, 2> shape,
                          std::int64_t rank_offset)
 {
@@ -394,7 +378,7 @@ distribute_to_postoffice(MPI_Comm comm, const xtl::span<const T>& x,
   // Compute send displacements
   std::vector<std::int32_t> send_disp = {0};
   std::partial_sum(num_items_per_dest.begin(), num_items_per_dest.end(),
-                   std::back_insert_iterator(send_disp));
+                   std::back_inserter(send_disp));
 
   // Pack send buffers
   std::vector<T> send_buffer_data(shape[1] * send_disp.back());
@@ -459,8 +443,8 @@ distribute_to_postoffice(MPI_Comm comm, const xtl::span<const T>& x,
 //---------------------------------------------------------------------------
 template <typename T>
 std::vector<T> distribute_from_postoffice(
-    MPI_Comm comm, const xtl::span<const std::int64_t>& indices,
-    const xtl::span<const T>& x, std::array<std::int64_t, 2> shape,
+    MPI_Comm comm, const std::span<const std::int64_t>& indices,
+    const std::span<const T>& x, std::array<std::int64_t, 2> shape,
     std::int64_t rank_offset)
 {
   common::Timer timer("Distribute row-wise data (scalable)");
@@ -535,10 +519,10 @@ std::vector<T> distribute_from_postoffice(
   // Prepare send/receive displacements
   std::vector<std::int32_t> send_disp = {0};
   std::partial_sum(num_items_per_src.begin(), num_items_per_src.end(),
-                   std::back_insert_iterator(send_disp));
+                   std::back_inserter(send_disp));
   std::vector<std::int32_t> recv_disp = {0};
   std::partial_sum(num_items_recv.begin(), num_items_recv.end(),
-                   std::back_insert_iterator(recv_disp));
+                   std::back_inserter(recv_disp));
 
   // Pack my requested indices (global) in send buffer ready to send to
   // post offices
@@ -662,8 +646,8 @@ std::vector<T> distribute_from_postoffice(
 //---------------------------------------------------------------------------
 template <typename T>
 std::vector<T> distribute_data(MPI_Comm comm,
-                               const xtl::span<const std::int64_t>& indices,
-                               const xtl::span<const T>& x, int shape1)
+                               const std::span<const std::int64_t>& indices,
+                               const std::span<const T>& x, int shape1)
 {
   assert(shape1 > 0);
   assert(x.size() % shape1 == 0);
@@ -677,40 +661,5 @@ std::vector<T> distribute_data(MPI_Comm comm,
                                     rank_offset);
 }
 //---------------------------------------------------------------------------
-template <typename T>
-graph::AdjacencyList<T>
-neighbor_all_to_all(MPI_Comm comm, const graph::AdjacencyList<T>& send_data)
-{
-  // Get neighbor processes
-  int indegree(-1), outdegree(-2), weighted(-1);
-  MPI_Dist_graph_neighbors_count(comm, &indegree, &outdegree, &weighted);
-
-  // Allocate memory (add '1' to handle empty case as OpenMPI fails for
-  // null pointers
-  std::vector<int> send_sizes(outdegree, 0);
-  std::vector<int> recv_sizes(indegree);
-  std::adjacent_difference(std::next(send_data.offsets().begin()),
-                           send_data.offsets().end(), send_sizes.begin());
-  // Get receive sizes
-  send_sizes.reserve(1);
-  recv_sizes.reserve(1);
-  MPI_Neighbor_alltoall(send_sizes.data(), 1, MPI_INT, recv_sizes.data(), 1,
-                        MPI_INT, comm);
-
-  // Work out recv offsets
-  std::vector<int> recv_offsets(indegree + 1);
-  recv_offsets[0] = 0;
-  std::partial_sum(recv_sizes.begin(), recv_sizes.end(),
-                   std::next(recv_offsets.begin(), 1));
-
-  std::vector<T> recv_data(recv_offsets[recv_offsets.size() - 1]);
-  MPI_Neighbor_alltoallv(
-      send_data.array().data(), send_sizes.data(), send_data.offsets().data(),
-      dolfinx::MPI::mpi_type<T>(), recv_data.data(), recv_sizes.data(),
-      recv_offsets.data(), dolfinx::MPI::mpi_type<T>(), comm);
-
-  return graph::AdjacencyList<T>(std::move(recv_data), std::move(recv_offsets));
-}
-//---------------------------------------------------------------------------
 
 } // namespace dolfinx::MPI
