@@ -9,8 +9,6 @@
 #include <cmath>
 #include <dolfinx/common/math.h>
 #include <dolfinx/mesh/cell_types.h>
-#include <xtensor/xnoalias.hpp>
-#include <xtensor/xview.hpp>
 
 using namespace dolfinx;
 using namespace dolfinx::fem;
@@ -46,16 +44,12 @@ CoordinateElement::tabulate_shape(std::size_t nd, std::size_t num_points) const
   return _element->tabulate_shape(nd, num_points);
 }
 //-----------------------------------------------------------------------------
-xt::xtensor<double, 4>
-CoordinateElement::tabulate(int n, const xt::xtensor<double, 2>& X) const
+void CoordinateElement::tabulate(int nd, std::span<const double> X,
+                                 std::array<std::size_t, 2> shape,
+                                 std::span<double> basis) const
 {
-  return _element->tabulate(n, X);
-}
-//--------------------------------------------------------------------------------
-void CoordinateElement::tabulate(int n, const xt::xtensor<double, 2>& X,
-                                 xt::xtensor<double, 4>& basis) const
-{
-  _element->tabulate(n, X, basis);
+  assert(_element);
+  _element->tabulate(nd, X, shape, basis);
 }
 //--------------------------------------------------------------------------------
 ElementDofLayout CoordinateElement::create_dof_layout() const
@@ -65,110 +59,92 @@ ElementDofLayout CoordinateElement::create_dof_layout() const
                           _element->entity_closure_dofs(), {}, {});
 }
 //-----------------------------------------------------------------------------
-void CoordinateElement::push_forward(
-    xt::xtensor<double, 2>& x, const xt::xtensor<double, 2>& cell_geometry,
-    const xt::xtensor<double, 2>& phi)
-{
-  assert(phi.shape(1) == cell_geometry.shape(0));
-
-  // Compute physical coordinates
-  // x = phi * cell_geometry;
-  x.fill(0);
-  math::dot(phi, cell_geometry, x);
-}
-//-----------------------------------------------------------------------------
-std::array<double, 3>
-CoordinateElement::x0(const xt::xtensor<double, 2>& cell_geometry)
-{
-  std::array<double, 3> x0 = {0, 0, 0};
-  for (std::size_t i = 0; i < cell_geometry.shape(1); ++i)
-    x0[i] += cell_geometry(0, i);
-  return x0;
-}
-//-----------------------------------------------------------------------------
-void CoordinateElement::pull_back_affine(xt::xtensor<double, 2>& X,
-                                         const xt::xtensor<double, 2>& K,
-                                         const std::array<double, 3>& x0,
-                                         const xt::xtensor<double, 2>& x)
-{
-  assert(X.shape(0) == x.shape(0));
-  assert(X.shape(1) == K.shape(0));
-  assert(x.shape(1) == K.shape(1));
-
-  // Calculate X for each point
-  X.fill(0.0);
-  for (std::size_t p = 0; p < x.shape(0); ++p)
-    for (std::size_t i = 0; i < K.shape(0); ++i)
-      for (std::size_t j = 0; j < K.shape(1); ++j)
-        X(p, i) += K(i, j) * (x(p, j) - x0[j]);
-}
-//-----------------------------------------------------------------------------
-void CoordinateElement::pull_back_nonaffine(
-    xt::xtensor<double, 2>& X, const xt::xtensor<double, 2>& x,
-    const xt::xtensor<double, 2>& cell_geometry, double tol, int maxit) const
+void CoordinateElement::pull_back_nonaffine(mdspan2_t X, cmdspan2_t x,
+                                            cmdspan2_t cell_geometry,
+                                            double tol, int maxit) const
 {
   // Number of points
-  std::size_t num_points = x.shape(0);
+  std::size_t num_points = x.extent(0);
   if (num_points == 0)
     return;
 
   const std::size_t tdim = mesh::cell_dim(this->cell_shape());
-  const std::size_t gdim = x.shape(1);
-  const std::size_t num_xnodes = cell_geometry.shape(0);
-  assert(cell_geometry.shape(1) == gdim);
-  assert(X.shape(0) == num_points);
-  assert(X.shape(1) == tdim);
+  const std::size_t gdim = x.extent(1);
+  const std::size_t num_xnodes = cell_geometry.extent(0);
+  assert(cell_geometry.extent(1) == gdim);
+  assert(X.extent(0) == num_points);
+  assert(X.extent(1) == tdim);
+
+  std::vector<double> dphi_b(tdim * num_xnodes);
+  mdspan2_t dphi(dphi_b.data(), tdim, num_xnodes);
+
+  std::vector<double> Xk_b(tdim);
+  mdspan2_t Xk(Xk_b.data(), 1, tdim);
 
-  xt::xtensor<double, 2> dphi({tdim, num_xnodes});
-  xt::xtensor<double, 2> Xk({1, tdim});
   std::array<double, 3> xk = {0, 0, 0};
-  xt::xtensor<double, 1> dX = xt::empty<double>({tdim});
-  xt::xtensor<double, 2> J({gdim, tdim});
-  xt::xtensor<double, 2> K({tdim, gdim});
-  xt::xtensor<double, 4> basis(_element->tabulate_shape(1, 1));
+
+  std::vector<double> dX(tdim);
+
+  std::vector<double> J_b(gdim * tdim);
+  mdspan2_t J(J_b.data(), gdim, tdim);
+
+  std::vector<double> K_b(tdim * gdim);
+  mdspan2_t K(K_b.data(), tdim, gdim);
+
+  namespace stdex = std::experimental;
+  using mdspan4_t = stdex::mdspan<double, stdex::dextents<std::size_t, 4>>;
+
+  const std::array<std::size_t, 4> bsize = _element->tabulate_shape(1, 1);
+  std::vector<double> basis_b(
+      std::reduce(bsize.begin(), bsize.end(), 1, std::multiplies{}));
+  mdspan4_t basis(basis_b.data(), bsize);
+  std::vector<double> phi(basis.extent(2));
+
   for (std::size_t p = 0; p < num_points; ++p)
   {
-    std::fill(Xk.begin(), Xk.end(), 0.0);
+    std::fill(Xk_b.begin(), Xk_b.end(), 0.0);
     int k;
     for (k = 0; k < maxit; ++k)
     {
-      _element->tabulate(1, Xk, basis);
+      _element->tabulate(1, Xk_b, {1, tdim}, basis_b);
 
       // x = cell_geometry * phi
-      auto phi = xt::view(basis, 0, 0, xt::all(), 0);
       std::fill(xk.begin(), xk.end(), 0.0);
-      for (std::size_t i = 0; i < cell_geometry.shape(0); ++i)
-        for (std::size_t j = 0; j < cell_geometry.shape(1); ++j)
-          xk[j] += cell_geometry(i, j) * phi[i];
+      for (std::size_t i = 0; i < cell_geometry.extent(0); ++i)
+        for (std::size_t j = 0; j < cell_geometry.extent(1); ++j)
+          xk[j] += cell_geometry(i, j) * basis(0, 0, i, 0);
 
       // Compute Jacobian, its inverse and determinant
-      std::fill(J.begin(), J.end(), 0.0);
-      dphi = xt::view(basis, xt::range(1, tdim + 1), 0, xt::all(), 0);
+      std::fill(J_b.begin(), J_b.end(), 0.0);
+      for (std::size_t i = 0; i < tdim; ++i)
+        for (std::size_t j = 0; j < basis.extent(2); ++j)
+          dphi(i, j) = basis(i + 1, 0, j, 0);
+
       compute_jacobian(dphi, cell_geometry, J);
       compute_jacobian_inverse(J, K);
 
       // Compute dX = K * (x_p - x_k)
       std::fill(dX.begin(), dX.end(), 0);
-      auto x_p = xt::row(x, p);
-      for (std::size_t i = 0; i < K.shape(0); ++i)
-        for (std::size_t j = 0; j < K.shape(1); ++j)
-          dX[i] += K(i, j) * (x_p[j] - xk[j]);
+      for (std::size_t i = 0; i < K.extent(0); ++i)
+        for (std::size_t j = 0; j < K.extent(1); ++j)
+          dX[i] += K(i, j) * (x(p, j) - xk[j]);
 
       // Compute Xk += dX
-      std::transform(dX.cbegin(), dX.cend(), Xk.cbegin(), Xk.begin(),
+      std::transform(dX.begin(), dX.end(), Xk_b.begin(), Xk_b.begin(),
                      [](double a, double b) { return a + b; });
 
       // Compute norm(dX)
-      if (auto dX_squared = std::transform_reduce(
-              dX.cbegin(), dX.cend(), 0.0, std::plus<double>(),
-              [](const auto v) { return v * v; });
+      if (auto dX_squared
+          = std::transform_reduce(dX.cbegin(), dX.cend(), 0.0, std::plus{},
+                                  [](auto v) { return v * v; });
           std::sqrt(dX_squared) < tol)
       {
         break;
       }
     }
-    std::copy(Xk.cbegin(), std::next(Xk.cbegin(), tdim),
-              std::next(X.begin(), p * tdim));
+
+    std::copy(Xk_b.cbegin(), std::next(Xk_b.cbegin(), tdim),
+              X.data_handle() + p * tdim);
     if (k == maxit)
     {
       throw std::runtime_error(
@@ -177,14 +153,14 @@ void CoordinateElement::pull_back_nonaffine(
   }
 }
 //-----------------------------------------------------------------------------
-void CoordinateElement::permute_dofs(const xtl::span<std::int32_t>& dofs,
+void CoordinateElement::permute_dofs(const std::span<std::int32_t>& dofs,
                                      std::uint32_t cell_perm) const
 {
   assert(_element);
   _element->permute_dofs(dofs, cell_perm);
 }
 //-----------------------------------------------------------------------------
-void CoordinateElement::unpermute_dofs(const xtl::span<std::int32_t>& dofs,
+void CoordinateElement::unpermute_dofs(const std::span<std::int32_t>& dofs,
                                        std::uint32_t cell_perm) const
 {
   assert(_element);
