@@ -7,13 +7,13 @@
 #pragma once
 
 #include "Function.h"
+#include <array>
 #include <dolfinx/common/utils.h>
 #include <dolfinx/mesh/Mesh.h>
 #include <functional>
+#include <span>
 #include <utility>
 #include <vector>
-#include <xtensor/xtensor.hpp>
-#include <xtl/xspan.hpp>
 
 namespace dolfinx::fem
 {
@@ -32,32 +32,47 @@ class Constant;
 template <typename T>
 class Expression
 {
+  template <typename X, typename = void>
+  struct scalar_value_type
+  {
+    typedef X value_type;
+  };
+  template <typename X>
+  struct scalar_value_type<X, std::void_t<typename X::value_type>>
+  {
+    typedef typename X::value_type value_type;
+  };
+  using scalar_value_type_t = typename scalar_value_type<T>::value_type;
+
 public:
-  /// Create an Expression
+  /// @brief Create an Expression
   ///
-  /// Users should prefer the create_expression factory functions
+  /// @note Users should prefer the create_expression factory functions
   ///
   /// @param[in] coefficients Coefficients in the Expression
   /// @param[in] constants Constants in the Expression
   /// @param[in] mesh
-  /// @param[in] X points on reference cell, number of points rows and
-  /// tdim cols
+  /// @param[in] X points on reference cell, shape=(number of points,
+  /// tdim) and storage is row-major.
+  /// @param[in] Xshape Shape of `X`.
   /// @param[in] fn function for tabulating expression
   /// @param[in] value_shape shape of expression evaluated at single point
   /// @param[in] argument_function_space Function space for Argument
   Expression(
       const std::vector<std::shared_ptr<const Function<T>>>& coefficients,
       const std::vector<std::shared_ptr<const Constant<T>>>& constants,
-      const xt::xtensor<double, 2>& X,
-      const std::function<void(T*, const T*, const T*, const double*,
-                               const int*, const uint8_t*)>
+      std::span<const double> X, std::array<std::size_t, 2> Xshape,
+      const std::function<void(T*, const T*, const T*,
+                               const scalar_value_type_t*, const int*,
+                               const uint8_t*)>
           fn,
       const std::vector<int>& value_shape,
       const std::shared_ptr<const mesh::Mesh>& mesh = nullptr,
       const std::shared_ptr<const FunctionSpace> argument_function_space
       = nullptr)
       : _coefficients(coefficients), _constants(constants), _mesh(mesh),
-        _x_ref(X), _fn(fn), _value_shape(value_shape),
+        _x_ref(std::vector<double>(X.begin(), X.end()), Xshape), _fn(fn),
+        _value_shape(value_shape),
         _argument_function_space(argument_function_space)
   {
     // Extract mesh from argument's function space
@@ -66,8 +81,10 @@ public:
     if (argument_function_space and _mesh != argument_function_space->mesh())
       throw std::runtime_error("Incompatible mesh");
     if (!_mesh)
+    {
       throw std::runtime_error(
           "No mesh could be associated with the Expression.");
+    }
   }
 
   /// Move constructor
@@ -78,15 +95,14 @@ public:
 
   /// Get argument function space
   /// @return The argument function space, nullptr if there is no argument.
-  std::shared_ptr<const fem::FunctionSpace> argument_function_space() const
+  std::shared_ptr<const FunctionSpace> argument_function_space() const
   {
     return _argument_function_space;
   };
 
   /// Get coefficients
   /// @return Vector of attached coefficients
-  const std::vector<std::shared_ptr<const fem::Function<T>>>&
-  coefficients() const
+  const std::vector<std::shared_ptr<const Function<T>>>& coefficients() const
   {
     return _coefficients;
   }
@@ -95,7 +111,7 @@ public:
   /// @return Vector of attached constants with their names. Names are
   ///   used to set constants in user's c++ code. Index in the vector is
   ///   the position of the constant in the original (nonsimplified) form.
-  const std::vector<std::shared_ptr<const fem::Constant<T>>>& constants() const
+  const std::vector<std::shared_ptr<const Constant<T>>>& constants() const
   {
     return _constants;
   }
@@ -121,7 +137,7 @@ public:
   /// is responsible for correct sizing which should be (num_cells,
   /// num_points * value_size * num_all_argument_dofs columns).
   template <typename U>
-  void eval(const xtl::span<const std::int32_t>& cells, U& values) const
+  void eval(const std::span<const std::int32_t>& cells, U& values) const
   {
     // Extract data from Expression
     assert(_mesh);
@@ -135,18 +151,18 @@ public:
     const graph::AdjacencyList<std::int32_t>& x_dofmap
         = _mesh->geometry().dofmap();
     const std::size_t num_dofs_g = _mesh->geometry().cmap().dim();
-    xtl::span<const double> x_g = _mesh->geometry().x();
+    std::span<const double> x_g = _mesh->geometry().x();
 
     // Create data structures used in evaluation
-    std::vector<double> coordinate_dofs(3 * num_dofs_g);
+    std::vector<scalar_value_type_t> coordinate_dofs(3 * num_dofs_g);
 
     int num_argument_dofs = 1;
-    xtl::span<const std::uint32_t> cell_info;
-    std::function<void(const xtl::span<T>&,
-                       const xtl::span<const std::uint32_t>&, std::int32_t,
+    std::span<const std::uint32_t> cell_info;
+    std::function<void(const std::span<T>&,
+                       const std::span<const std::uint32_t>&, std::int32_t,
                        int)>
         dof_transform_to_transpose
-        = [](const xtl::span<T>&, const xtl::span<const std::uint32_t>&,
+        = [](const std::span<T>&, const std::span<const std::uint32_t>&,
              std::int32_t, int)
     {
       // Do nothing
@@ -162,16 +178,16 @@ public:
       if (element->needs_dof_transformations())
       {
         _mesh->topology_mutable().create_entity_permutations();
-        cell_info = xtl::span(_mesh->topology().get_cell_permutation_info());
+        cell_info = std::span(_mesh->topology().get_cell_permutation_info());
         dof_transform_to_transpose
             = element
                   ->template get_dof_transformation_to_transpose_function<T>();
       }
     }
 
-    const int size0 = _x_ref.shape(0) * value_size();
+    const int size0 = _x_ref.second[0] * value_size();
     std::vector<T> values_local(size0 * num_argument_dofs, 0);
-    const xtl::span<T> _values_local(values_local);
+    const std::span<T> _values_local(values_local);
 
     // Iterate over cells and 'assemble' into values
     for (std::size_t c = 0; c < cells.size(); ++c)
@@ -199,8 +215,8 @@ public:
 
   /// Get function for tabulate_expression.
   /// @return fn Function to tabulate expression.
-  const std::function<void(T*, const T*, const T*, const double*, const int*,
-                           const uint8_t*)>&
+  const std::function<void(T*, const T*, const T*, const scalar_value_type_t*,
+                           const int*, const uint8_t*)>&
   get_tabulate_expression() const
   {
     return _fn;
@@ -214,34 +230,37 @@ public:
   /// @return value_size
   int value_size() const
   {
-    return std::accumulate(_value_shape.begin(), _value_shape.end(), 1,
-                           std::multiplies<int>());
+    return std::reduce(_value_shape.begin(), _value_shape.end(), 1,
+                       std::multiplies{});
   }
 
   /// Get value shape
   /// @return value shape
   const std::vector<int>& value_shape() const { return _value_shape; }
 
-  /// Get evaluation points on reference cell
+  /// @brief Evaluation points on the reference cell
   /// @return Evaluation points
-  const xt::xtensor<double, 2>& X() const { return _x_ref; }
+  std::pair<std::vector<double>, std::array<std::size_t, 2>> X() const
+  {
+    return _x_ref;
+  }
 
   /// Scalar type (T)
   using scalar_type = T;
 
 private:
   // Function space for Argument
-  std::shared_ptr<const fem::FunctionSpace> _argument_function_space;
+  std::shared_ptr<const FunctionSpace> _argument_function_space;
 
   // Coefficients associated with the Expression
-  std::vector<std::shared_ptr<const fem::Function<T>>> _coefficients;
+  std::vector<std::shared_ptr<const Function<T>>> _coefficients;
 
   // Constants associated with the Expression
-  std::vector<std::shared_ptr<const fem::Constant<T>>> _constants;
+  std::vector<std::shared_ptr<const Constant<T>>> _constants;
 
   // Function to evaluate the Expression
-  std::function<void(T*, const T*, const T*, const double*, const int*,
-                     const uint8_t*)>
+  std::function<void(T*, const T*, const T*, const scalar_value_type_t*,
+                     const int*, const uint8_t*)>
       _fn;
 
   // The mesh
@@ -251,6 +270,6 @@ private:
   std::vector<int> _value_shape;
 
   // Evaluation points on reference cell. Synonymous with X in public interface.
-  xt::xtensor<double, 2> _x_ref;
+  std::pair<std::vector<double>, std::array<std::size_t, 2>> _x_ref;
 };
 } // namespace dolfinx::fem
