@@ -6,13 +6,14 @@
 
 #pragma once
 
+#include <array>
 #include <basix/finite-element.h>
 #include <dolfinx/mesh/cell_types.h>
 #include <functional>
 #include <memory>
+#include <span>
+#include <utility>
 #include <vector>
-#include <xtensor/xtensor.hpp>
-#include <xtl/xspan.hpp>
 
 struct ufcx_finite_element;
 
@@ -84,9 +85,9 @@ public:
 
   /// The value size, e.g. 1 for a scalar function, 2 for a 2D vector, 9
   /// for a second-order tensor in 3D.
-  /// @note The return value of this function is equal to
+  /// @note The return value of this function is equivalent to
   /// `std::accumulate(value_shape().begin(), value_shape().end(), 1,
-  /// std::multiplies<int>())`.
+  /// std::multiplies{})`.
   /// @return The value size
   int value_size() const;
 
@@ -97,67 +98,40 @@ public:
 
   /// Shape of the value space. The rank is the size of the
   /// `value_shape`.
-  xtl::span<const int> value_shape() const noexcept;
+  std::span<const std::size_t> value_shape() const noexcept;
 
   /// The finite element family
   /// @return The string of the finite element family
   std::string family() const noexcept;
 
-  /// Evaluate all derivatives of the basis functions up to given order
-  /// at given points in reference cell
-  /// @param[in,out] values Four dimensional xtensor that will be filled
-  /// with the tabulated values. Should be of shape {num_derivatives,
-  /// num_points, num_dofs, reference_value_size}
-  /// @param[in] X Two dimensional xtensor of shape [num_points,
-  /// geometric dimension] containing the points at the reference
-  /// element
+  /// @brief Evaluate derivatives of the basis functions up to given order
+  /// at points in the reference cell.
+  /// @param[in,out] values Array that will be filled with the tabulated
+  /// basis values. Must have shape `(num_derivatives, num_points,
+  /// num_dofs, reference_value_size)` (row-major storage)
+  /// @param[in] X The reference coordinates at which to evaluate the
+  /// basis functions. Shape is `(num_points, topological dimension)`
+  /// (row-major storage)
+  /// @param[in] shape The shape of `X`
   /// @param[in] order The number of derivatives (up to and including
   /// this order) to tabulate for
-  void tabulate(xt::xtensor<double, 4>& values, const xt::xtensor<double, 2>& X,
-                int order) const;
+  void tabulate(std::span<double> values, std::span<const double> X,
+                std::array<std::size_t, 2> shape, int order) const;
 
-  /// Return a function that performs the appropriate
-  /// push-forward (pull-back) for the element type
-  ///
-  /// @tparam O The type that hold the computed pushed-forward
-  /// (pulled-back)  data (ndim==1)
-  /// @tparam P The type that hold the data to be pulled back (pushed
-  /// forwarded) (ndim==1)
-  /// @tparam Q The type that holds the Jacobian (inverse Jacobian)
-  /// matrix (ndim==2)
-  /// @tparam R The type that holds the inverse Jacobian (Jacobian)
-  /// matrix (ndim==2)
-  ///
-  /// @return A function that for a push-forward takes arguments
-  /// - `u` [out] The data on the physical cell after the
-  /// push-forward flattened with row-major layout, shape=(num_points,
-  /// value_size)
-  /// - `U` [in] The data on the reference cell physical field to push
-  /// forward, flattened with row-major layout, shape=(num_points,
-  /// ref_value_size)
-  /// - `J` [in] The Jacobian matrix of the map ,shape=(gdim, tdim)
-  /// - `detJ` [in] det(J)
-  /// - `K` [in] The inverse of the Jacobian matrix, shape=(tdim, gdim)
-  ///
-  /// For a pull-back the passed arguments should be:
-  /// - `U` [out] The data on the reference cell after the pull-back,
-  /// flattened with row-major layout, shape=(num_points, ref
-  /// value_size)
-  /// - `u` [in] The data on the physical cell that should be pulled
-  /// back , flattened with row-major layout, shape=(num_points,
-  /// value_size)
-  /// - `K` [in] The inverse oif the Jacobian matrix of the map,
-  /// shape=(tdim, gdim)
-  /// - `detJ_inv` [in] 1/det(J)
-  /// - `J` [in] The Jacobian matrix, shape=(gdim, tdim)
-  template <typename O, typename P, typename Q, typename R>
-  std::function<void(O&, const P&, const Q&, double, const R&)> map_fn() const
-  {
-    assert(_element);
-    return _element->map_fn<O, P, Q, R>();
-  }
+  /// Evaluate all derivatives of the basis functions up to given order
+  /// at given points in reference cell
+  /// @param[in] X The reference coordinates at which to evaluate the
+  /// basis functions. Shape is `(num_points, topological dimension)`
+  /// (row-major storage)
+  /// @param[in] shape The shape of `X`
+  /// @param[in] order The number of derivatives (up to and including
+  /// this order) to tabulate for
+  /// @return Basis function values and array shape (row-major storage)
+  std::pair<std::vector<double>, std::array<std::size_t, 4>>
+  tabulate(std::span<const double> X, std::array<std::size_t, 2> shape,
+           int order) const;
 
-  /// Get the number of sub elements (for a mixed or blocked element)
+  /// @brief Number of sub elements (for a mixed or blocked element)
   /// @return The number of sub elements
   int num_sub_elements() const noexcept;
 
@@ -175,6 +149,9 @@ public:
   std::shared_ptr<const FiniteElement>
   extract_sub_element(const std::vector<int>& component) const;
 
+  /// Return underlying basix element (if it exists)
+  const basix::FiniteElement& basix_element() const;
+
   /// Get the map type used by the element
   basix::maps::type map_type() const;
 
@@ -191,13 +168,18 @@ public:
   /// @return True if the map is the identity
   bool map_ident() const noexcept;
 
-  /// Points on the reference cell at which an expression need to be
-  /// evaluated in order to interpolate the expression in the finite
-  /// element space. For Lagrange elements the points will just be the
+  /// @brief Points on the reference cell at which an expression need to
+  /// be evaluated in order to interpolate the expression in the finite
+  /// element space.
+  ///
+  /// For Lagrange elements the points will just be the
   /// nodal positions. For other elements the points will typically be
   /// the quadrature points used to evaluate moment degrees of freedom.
-  /// @return Points on the reference cell. Shape is (num_points, tdim).
-  const xt::xtensor<double, 2>& interpolation_points() const;
+  /// @return Interpolation point coordinates on the reference cell,
+  /// returning the (0) coordinates data (row-major) storage and (1) the
+  /// shape `(num_points, tdim)`.
+  std::pair<std::vector<double>, std::array<std::size_t, 2>>
+  interpolation_points() const;
 
   /// Interpolation operator (matrix) `Pi` that maps a function
   /// evaluated at the points provided by
@@ -205,28 +187,32 @@ public:
   /// freedom, i.e. dofs = Pi f_x. See the Basix documentation for
   /// basix::FiniteElement::interpolation_matrix for how the data in
   /// `f_x` should be ordered.
-  /// @return The interpolation operator `Pi`. Shape is (num_dofs,
-  /// num_points*value_size)
-  const xt::xtensor<double, 2>& interpolation_operator() const;
+  /// @return The interpolation operator `Pi`, returning the data for
+  /// `Pi` (row-major storage) and the shape `(num_dofs, num_points *
+  /// value_size)`
+  std::pair<std::vector<double>, std::array<std::size_t, 2>>
+  interpolation_operator() const;
 
-  /// Create a matrix that maps degrees of freedom from one element to
-  /// this element (interpolation).
+  /// @brief Create a matrix that maps degrees of freedom from one
+  /// element to this element (interpolation).
   ///
   /// @param[in] from The element to interpolate from
   /// @return Matrix operator that maps the `from` degrees-of-freedom to
-  /// the degrees-of-freedom of this element. Shape is (num_dofs of this
-  /// element, num_dofs of `from`).
-  /// @note The two elements must use the same mapping between the
+  /// the degrees-of-freedom of this element. The (0) matrix data
+  /// (row-major storage) and (1) the shape (num_dofs of `this` element,
+  /// num_dofs of `from`) are returned.
+  ///
+  /// @pre The two elements must use the same mapping between the
   /// reference and physical cells
   /// @note Does not support mixed elements
-  xt::xtensor<double, 2>
+  std::pair<std::vector<double>, std::array<std::size_t, 2>>
   create_interpolation_operator(const FiniteElement& from) const;
 
-  /// Check if DOF transformations are needed for this element.
+  /// @brief Check if DOF transformations are needed for this element.
   ///
   /// DOF transformations will be needed for elements which might not be
   /// continuous when two neighbouring cells disagree on the orientation
-  /// of a shared subentity, and when this cannot be corrected for by
+  /// of a shared sub-entity, and when this cannot be corrected for by
   /// permuting the DOF numbering in the dofmap.
   ///
   /// For example, Raviart-Thomas elements will need DOF
@@ -237,7 +223,7 @@ public:
   /// @return True if DOF transformations are required
   bool needs_dof_transformations() const noexcept;
 
-  /// Check if DOF permutations are needed for this element.
+  /// @brief Check if DOF permutations are needed for this element.
   ///
   /// DOF permutations will be needed for elements which might not be
   /// continuous when two neighbouring cells disagree on the orientation
@@ -245,8 +231,8 @@ public:
   /// permuting the DOF numbering in the dofmap.
   ///
   /// For example, higher order Lagrange elements will need DOF
-  /// permutations, as the arrangement of DOFs on a shared subentity may
-  /// be different from the point of view of neighbouring cells, and
+  /// permutations, as the arrangement of DOFs on a shared sub-entity
+  /// may be different from the point of view of neighbouring cells, and
   /// this can be corrected for by permuting the DOF numbers on each
   /// cell.
   ///
@@ -271,7 +257,7 @@ public:
   /// @param[in] scalar_element Indicates whether the scalar
   /// transformations should be returned for a vector element
   template <typename T>
-  std::function<void(const xtl::span<T>&, const xtl::span<const std::uint32_t>&,
+  std::function<void(const std::span<T>&, const std::span<const std::uint32_t>&,
                      std::int32_t, int)>
   get_dof_transformation_function(bool inverse = false, bool transpose = false,
                                   bool scalar_element = false) const
@@ -279,7 +265,7 @@ public:
     if (!needs_dof_transformations())
     {
       // If no permutation needed, return function that does nothing
-      return [](const xtl::span<T>&, const xtl::span<const std::uint32_t>&,
+      return [](const std::span<T>&, const std::span<const std::uint32_t>&,
                 std::int32_t, int)
       {
         // Do nothing
@@ -291,8 +277,8 @@ public:
       if (_bs == 1)
       {
         // Mixed element
-        std::vector<std::function<void(const xtl::span<T>&,
-                                       const xtl::span<const std::uint32_t>&,
+        std::vector<std::function<void(const std::span<T>&,
+                                       const std::span<const std::uint32_t>&,
                                        std::int32_t, int)>>
             sub_element_functions;
         std::vector<int> dims;
@@ -305,8 +291,8 @@ public:
         }
 
         return [dims, sub_element_functions](
-                   const xtl::span<T>& data,
-                   const xtl::span<const std::uint32_t>& cell_info,
+                   const std::span<T>& data,
+                   const std::span<const std::uint32_t>& cell_info,
                    std::int32_t cell, int block_size)
         {
           std::size_t offset = 0;
@@ -322,15 +308,15 @@ public:
       else if (!scalar_element)
       {
         // Vector element
-        const std::function<void(const xtl::span<T>&,
-                                 const xtl::span<const std::uint32_t>&,
+        const std::function<void(const std::span<T>&,
+                                 const std::span<const std::uint32_t>&,
                                  std::int32_t, int)>
             sub_function = _sub_elements[0]->get_dof_transformation_function<T>(
                 inverse, transpose);
         const int ebs = _bs;
         return
-            [ebs, sub_function](const xtl::span<T>& data,
-                                const xtl::span<const std::uint32_t>& cell_info,
+            [ebs, sub_function](const std::span<T>& data,
+                                const std::span<const std::uint32_t>& cell_info,
                                 std::int32_t cell, int data_block_size)
         { sub_function(data, cell_info, cell, ebs * data_block_size); };
       }
@@ -339,8 +325,8 @@ public:
     {
       if (inverse)
       {
-        return [this](const xtl::span<T>& data,
-                      const xtl::span<const std::uint32_t>& cell_info,
+        return [this](const std::span<T>& data,
+                      const std::span<const std::uint32_t>& cell_info,
                       std::int32_t cell, int block_size)
         {
           apply_inverse_transpose_dof_transformation(data, cell_info[cell],
@@ -349,8 +335,8 @@ public:
       }
       else
       {
-        return [this](const xtl::span<T>& data,
-                      const xtl::span<const std::uint32_t>& cell_info,
+        return [this](const std::span<T>& data,
+                      const std::span<const std::uint32_t>& cell_info,
                       std::int32_t cell, int block_size) {
           apply_transpose_dof_transformation(data, cell_info[cell], block_size);
         };
@@ -360,16 +346,16 @@ public:
     {
       if (inverse)
       {
-        return [this](const xtl::span<T>& data,
-                      const xtl::span<const std::uint32_t>& cell_info,
+        return [this](const std::span<T>& data,
+                      const std::span<const std::uint32_t>& cell_info,
                       std::int32_t cell, int block_size) {
           apply_inverse_dof_transformation(data, cell_info[cell], block_size);
         };
       }
       else
       {
-        return [this](const xtl::span<T>& data,
-                      const xtl::span<const std::uint32_t>& cell_info,
+        return [this](const std::span<T>& data,
+                      const std::span<const std::uint32_t>& cell_info,
                       std::int32_t cell, int block_size)
         { apply_dof_transformation(data, cell_info[cell], block_size); };
       }
@@ -395,7 +381,7 @@ public:
   /// @param[in] scalar_element Indicated whether the scalar
   /// transformations should be returned for a vector element
   template <typename T>
-  std::function<void(const xtl::span<T>&, const xtl::span<const std::uint32_t>&,
+  std::function<void(const std::span<T>&, const std::span<const std::uint32_t>&,
                      std::int32_t, int)>
   get_dof_transformation_to_transpose_function(bool inverse = false,
                                                bool transpose = false,
@@ -405,7 +391,7 @@ public:
     if (!needs_dof_transformations())
     {
       // If no permutation needed, return function that does nothing
-      return [](const xtl::span<T>&, const xtl::span<const std::uint32_t>&,
+      return [](const std::span<T>&, const std::span<const std::uint32_t>&,
                 std::int32_t, int)
       {
         // Do nothing
@@ -416,8 +402,8 @@ public:
       if (_bs == 1)
       {
         // Mixed element
-        std::vector<std::function<void(const xtl::span<T>&,
-                                       const xtl::span<const std::uint32_t>&,
+        std::vector<std::function<void(const std::span<T>&,
+                                       const std::span<const std::uint32_t>&,
                                        std::int32_t, int)>>
             sub_element_functions;
         for (std::size_t i = 0; i < _sub_elements.size(); ++i)
@@ -428,8 +414,8 @@ public:
         }
 
         return [this, sub_element_functions](
-                   const xtl::span<T>& data,
-                   const xtl::span<const std::uint32_t>& cell_info,
+                   const std::span<T>& data,
+                   const std::span<const std::uint32_t>& cell_info,
                    std::int32_t cell, int block_size)
         {
           std::size_t offset = 0;
@@ -444,14 +430,14 @@ public:
       else if (!scalar_element)
       {
         // Vector element
-        const std::function<void(const xtl::span<T>&,
-                                 const xtl::span<const std::uint32_t>&,
+        const std::function<void(const std::span<T>&,
+                                 const std::span<const std::uint32_t>&,
                                  std::int32_t, int)>
             sub_function = _sub_elements[0]->get_dof_transformation_function<T>(
                 inverse, transpose);
         return [this,
-                sub_function](const xtl::span<T>& data,
-                              const xtl::span<const std::uint32_t>& cell_info,
+                sub_function](const std::span<T>& data,
+                              const std::span<const std::uint32_t>& cell_info,
                               std::int32_t cell, int data_block_size)
         {
           const int ebs = block_size();
@@ -469,8 +455,8 @@ public:
     {
       if (inverse)
       {
-        return [this](const xtl::span<T>& data,
-                      const xtl::span<const std::uint32_t>& cell_info,
+        return [this](const std::span<T>& data,
+                      const std::span<const std::uint32_t>& cell_info,
                       std::int32_t cell, int block_size)
         {
           apply_inverse_transpose_dof_transformation_to_transpose(
@@ -479,8 +465,8 @@ public:
       }
       else
       {
-        return [this](const xtl::span<T>& data,
-                      const xtl::span<const std::uint32_t>& cell_info,
+        return [this](const std::span<T>& data,
+                      const std::span<const std::uint32_t>& cell_info,
                       std::int32_t cell, int block_size)
         {
           apply_transpose_dof_transformation_to_transpose(data, cell_info[cell],
@@ -492,8 +478,8 @@ public:
     {
       if (inverse)
       {
-        return [this](const xtl::span<T>& data,
-                      const xtl::span<const std::uint32_t>& cell_info,
+        return [this](const std::span<T>& data,
+                      const std::span<const std::uint32_t>& cell_info,
                       std::int32_t cell, int block_size)
         {
           apply_inverse_dof_transformation_to_transpose(data, cell_info[cell],
@@ -502,8 +488,8 @@ public:
       }
       else
       {
-        return [this](const xtl::span<T>& data,
-                      const xtl::span<const std::uint32_t>& cell_info,
+        return [this](const std::span<T>& data,
+                      const std::span<const std::uint32_t>& cell_info,
                       std::int32_t cell, int block_size) {
           apply_dof_transformation_to_transpose(data, cell_info[cell],
                                                 block_size);
@@ -519,7 +505,7 @@ public:
   /// @param[in] cell_permutation Permutation data for the cell
   /// @param[in] block_size The block_size of the input data
   template <typename T>
-  void apply_dof_transformation(const xtl::span<T>& data,
+  void apply_dof_transformation(const std::span<T>& data,
                                 std::uint32_t cell_permutation,
                                 int block_size) const
   {
@@ -537,7 +523,7 @@ public:
   /// @param[in] block_size The block_size of the input data
   template <typename T>
   void
-  apply_inverse_transpose_dof_transformation(const xtl::span<T>& data,
+  apply_inverse_transpose_dof_transformation(const std::span<T>& data,
                                              std::uint32_t cell_permutation,
                                              int block_size) const
   {
@@ -554,7 +540,7 @@ public:
   /// @param[in] cell_permutation Permutation data for the cell
   /// @param[in] block_size The block_size of the input data
   template <typename T>
-  void apply_transpose_dof_transformation(const xtl::span<T>& data,
+  void apply_transpose_dof_transformation(const std::span<T>& data,
                                           std::uint32_t cell_permutation,
                                           int block_size) const
   {
@@ -571,7 +557,7 @@ public:
   /// @param[in] cell_permutation Permutation data for the cell
   /// @param[in] block_size The block_size of the input data
   template <typename T>
-  void apply_inverse_dof_transformation(const xtl::span<T>& data,
+  void apply_inverse_dof_transformation(const std::span<T>& data,
                                         std::uint32_t cell_permutation,
                                         int block_size) const
   {
@@ -587,7 +573,7 @@ public:
   /// @param[in] cell_permutation Permutation data for the cell
   /// @param[in] block_size The block_size of the input data
   template <typename T>
-  void apply_dof_transformation_to_transpose(const xtl::span<T>& data,
+  void apply_dof_transformation_to_transpose(const std::span<T>& data,
                                              std::uint32_t cell_permutation,
                                              int block_size) const
   {
@@ -604,7 +590,7 @@ public:
   /// @param[in] block_size The block_size of the input data
   template <typename T>
   void
-  apply_inverse_dof_transformation_to_transpose(const xtl::span<T>& data,
+  apply_inverse_dof_transformation_to_transpose(const std::span<T>& data,
                                                 std::uint32_t cell_permutation,
                                                 int block_size) const
   {
@@ -621,7 +607,7 @@ public:
   /// @param[in] block_size The block_size of the input data
   template <typename T>
   void apply_transpose_dof_transformation_to_transpose(
-      const xtl::span<T>& data, std::uint32_t cell_permutation,
+      const std::span<T>& data, std::uint32_t cell_permutation,
       int block_size) const
   {
     assert(_element);
@@ -637,7 +623,7 @@ public:
   /// @param[in] block_size The block_size of the input data
   template <typename T>
   void apply_inverse_transpose_dof_transformation_to_transpose(
-      const xtl::span<T>& data, std::uint32_t cell_permutation,
+      const std::span<T>& data, std::uint32_t cell_permutation,
       int block_size) const
   {
     assert(_element);
@@ -649,14 +635,14 @@ public:
   ///
   /// @param[in,out] doflist The numbers of the DOFs, a span of length num_dofs
   /// @param[in] cell_permutation Permutation data for the cell
-  void permute_dofs(const xtl::span<std::int32_t>& doflist,
+  void permute_dofs(const std::span<std::int32_t>& doflist,
                     std::uint32_t cell_permutation) const;
 
   /// Unpermute the DOFs of the element
   ///
   /// @param[in,out] doflist The numbers of the DOFs, a span of length num_dofs
   /// @param[in] cell_permutation Permutation data for the cell
-  void unpermute_dofs(const xtl::span<std::int32_t>& doflist,
+  void unpermute_dofs(const std::span<std::int32_t>& doflist,
                       std::uint32_t cell_permutation) const;
 
   /// Return a function that applies DOF permutation to some data
@@ -670,7 +656,7 @@ public:
   /// should be returned
   /// @param[in] scalar_element Indicated whether the scalar
   /// transformations should be returned for a vector element
-  std::function<void(const xtl::span<std::int32_t>&, std::uint32_t)>
+  std::function<void(const std::span<std::int32_t>&, std::uint32_t)>
   get_dof_permutation_function(bool inverse = false,
                                bool scalar_element = false) const;
 
@@ -685,7 +671,7 @@ private:
   std::vector<std::shared_ptr<const FiniteElement>> _sub_elements;
 
   // Dimension of each value space
-  std::vector<int> _value_shape;
+  std::vector<std::size_t> _value_shape;
 
   // Block size for VectorElements and TensorElements. This gives the
   // number of DOFs co-located at each dof 'point'.
