@@ -11,8 +11,8 @@
 #include "assemble_vector_impl.h"
 #include <cstdint>
 #include <memory>
+#include <span>
 #include <vector>
-#include <xtl/xspan.hpp>
 
 namespace dolfinx::fem
 {
@@ -24,15 +24,15 @@ class FunctionSpace;
 
 // -- Helper functions -----------------------------------------------------
 
-/// @brief Create a map xtl::span from a map of std::vector
+/// @brief Create a map std::span from a map of std::vector
 template <typename T>
 std::map<std::pair<dolfinx::fem::IntegralType, int>,
-         std::pair<xtl::span<const T>, int>>
+         std::pair<std::span<const T>, int>>
 make_coefficients_span(const std::map<std::pair<IntegralType, int>,
                                       std::pair<std::vector<T>, int>>& coeffs)
 {
   using Key = typename std::remove_reference_t<decltype(coeffs)>::key_type;
-  std::map<Key, std::pair<xtl::span<const T>, int>> c;
+  std::map<Key, std::pair<std::span<const T>, int>> c;
   std::transform(coeffs.cbegin(), coeffs.cend(), std::inserter(c, c.end()),
                  [](auto& e) -> typename decltype(c)::value_type {
                    return {e.first, {e.second.first, e.second.second}};
@@ -53,9 +53,9 @@ make_coefficients_span(const std::map<std::pair<IntegralType, int>,
 /// process
 template <typename T>
 T assemble_scalar(
-    const Form<T>& M, const xtl::span<const T>& constants,
+    const Form<T>& M, const std::span<const T>& constants,
     const std::map<std::pair<IntegralType, int>,
-                   std::pair<xtl::span<const T>, int>>& coefficients)
+                   std::pair<std::span<const T>, int>>& coefficients)
 {
   return impl::assemble_scalar(M, constants, coefficients);
 }
@@ -71,7 +71,7 @@ T assemble_scalar(const Form<T>& M)
   const std::vector<T> constants = pack_constants(M);
   auto coefficients = allocate_coefficient_storage(M);
   pack_coefficients(M, coefficients);
-  return assemble_scalar(M, tcb::make_span(constants),
+  return assemble_scalar(M, std::span(constants),
                          make_coefficients_span(coefficients));
 }
 
@@ -87,9 +87,9 @@ T assemble_scalar(const Form<T>& M)
 /// @param[in] coefficients The coefficients that appear in `L`
 template <typename T>
 void assemble_vector(
-    xtl::span<T> b, const Form<T>& L, const xtl::span<const T>& constants,
+    std::span<T> b, const Form<T>& L, const std::span<const T>& constants,
     const std::map<std::pair<IntegralType, int>,
-                   std::pair<xtl::span<const T>, int>>& coefficients)
+                   std::pair<std::span<const T>, int>>& coefficients)
 {
   impl::assemble_vector(b, L, constants, coefficients);
 }
@@ -99,12 +99,12 @@ void assemble_vector(
 /// before assembly.
 /// @param[in] L The linear forms to assemble into b
 template <typename T>
-void assemble_vector(xtl::span<T> b, const Form<T>& L)
+void assemble_vector(std::span<T> b, const Form<T>& L)
 {
   auto coefficients = allocate_coefficient_storage(L);
   pack_coefficients(L, coefficients);
   const std::vector<T> constants = pack_constants(L);
-  assemble_vector(b, L, tcb::make_span(constants),
+  assemble_vector(b, L, std::span(constants),
                   make_coefficients_span(coefficients));
 }
 
@@ -128,12 +128,12 @@ void assemble_vector(xtl::span<T> b, const Form<T>& L)
 /// is responsible for calling VecGhostUpdateBegin/End.
 template <typename T>
 void apply_lifting(
-    xtl::span<T> b, const std::vector<std::shared_ptr<const Form<T>>>& a,
-    const std::vector<xtl::span<const T>>& constants,
+    std::span<T> b, const std::vector<std::shared_ptr<const Form<T>>>& a,
+    const std::vector<std::span<const T>>& constants,
     const std::vector<std::map<std::pair<IntegralType, int>,
-                               std::pair<xtl::span<const T>, int>>>& coeffs,
+                               std::pair<std::span<const T>, int>>>& coeffs,
     const std::vector<std::vector<std::shared_ptr<const DirichletBC<T>>>>& bcs1,
-    const std::vector<xtl::span<const T>>& x0, double scale)
+    const std::vector<std::span<const T>>& x0, double scale)
 {
   impl::apply_lifting(b, a, constants, coeffs, bcs1, x0, scale);
 }
@@ -152,9 +152,9 @@ void apply_lifting(
 /// is responsible for calling VecGhostUpdateBegin/End.
 template <typename T>
 void apply_lifting(
-    xtl::span<T> b, const std::vector<std::shared_ptr<const Form<T>>>& a,
+    std::span<T> b, const std::vector<std::shared_ptr<const Form<T>>>& a,
     const std::vector<std::vector<std::shared_ptr<const DirichletBC<T>>>>& bcs1,
-    const std::vector<xtl::span<const T>>& x0, double scale)
+    const std::vector<std::span<const T>>& x0, double scale)
 {
   std::vector<
       std::map<std::pair<IntegralType, int>, std::pair<std::vector<T>, int>>>
@@ -177,10 +177,10 @@ void apply_lifting(
     }
   }
 
-  std::vector<xtl::span<const T>> _constants(constants.begin(),
+  std::vector<std::span<const T>> _constants(constants.begin(),
                                              constants.end());
   std::vector<std::map<std::pair<IntegralType, int>,
-                       std::pair<xtl::span<const T>, int>>>
+                       std::pair<std::span<const T>, int>>>
       _coeffs;
   std::transform(coeffs.cbegin(), coeffs.cend(), std::back_inserter(_coeffs),
                  [](auto& c) { return make_coefficients_span(c); });
@@ -198,9 +198,9 @@ void apply_lifting(
 ///  dofs the row and column are zeroed. The diagonal  entry is not set.
 template <typename T, typename U>
 void assemble_matrix(
-    U mat_add, const Form<T>& a, const xtl::span<const T>& constants,
+    U mat_add, const Form<T>& a, const std::span<const T>& constants,
     const std::map<std::pair<IntegralType, int>,
-                   std::pair<xtl::span<const T>, int>>& coefficients,
+                   std::pair<std::span<const T>, int>>& coefficients,
     const std::vector<std::shared_ptr<const DirichletBC<T>>>& bcs)
 {
   // Index maps for dof ranges
@@ -253,7 +253,7 @@ void assemble_matrix(
   pack_coefficients(a, coefficients);
 
   // Assemble
-  assemble_matrix(mat_add, a, tcb::make_span(constants),
+  assemble_matrix(mat_add, a, std::span(constants),
                   make_coefficients_span(coefficients), bcs);
 }
 
@@ -271,11 +271,11 @@ void assemble_matrix(
 /// local index.
 template <typename T, typename U>
 void assemble_matrix(
-    U mat_add, const Form<T>& a, const xtl::span<const T>& constants,
+    U mat_add, const Form<T>& a, const std::span<const T>& constants,
     const std::map<std::pair<IntegralType, int>,
-                   std::pair<xtl::span<const T>, int>>& coefficients,
-    const xtl::span<const std::int8_t>& dof_marker0,
-    const xtl::span<const std::int8_t>& dof_marker1)
+                   std::pair<std::span<const T>, int>>& coefficients,
+    const std::span<const std::int8_t>& dof_marker0,
+    const std::span<const std::int8_t>& dof_marker1)
 
 {
   impl::assemble_matrix(mat_add, a, constants, coefficients, dof_marker0,
@@ -294,8 +294,8 @@ void assemble_matrix(
 ///   local index.
 template <typename T, typename U>
 void assemble_matrix(U mat_add, const Form<T>& a,
-                     const xtl::span<const std::int8_t>& dof_marker0,
-                     const xtl::span<const std::int8_t>& dof_marker1)
+                     const std::span<const std::int8_t>& dof_marker0,
+                     const std::span<const std::int8_t>& dof_marker1)
 
 {
   // Prepare constants and coefficients
@@ -304,7 +304,7 @@ void assemble_matrix(U mat_add, const Form<T>& a,
   pack_coefficients(a, coefficients);
 
   // Assemble
-  assemble_matrix(mat_add, a, tcb::make_span(constants),
+  assemble_matrix(mat_add, a, std::span(constants),
                   make_coefficients_span(coefficients), dof_marker0,
                   dof_marker1);
 }
@@ -320,12 +320,12 @@ void assemble_matrix(U mat_add, const Form<T>& a,
 /// @param[in] diagonal The value to add to the diagonal for the
 ///   specified rows
 template <typename T, typename U>
-void set_diagonal(U set_fn, const xtl::span<const std::int32_t>& rows,
+void set_diagonal(U set_fn, const std::span<const std::int32_t>& rows,
                   T diagonal = 1.0)
 {
   for (std::size_t i = 0; i < rows.size(); ++i)
   {
-    xtl::span diag_span(&diagonal, 1);
+    std::span diag_span(&diagonal, 1);
     set_fn(rows.subspan(i, 1), rows.subspan(i, 1), diag_span);
   }
 }
@@ -372,9 +372,9 @@ void set_diagonal(U set_fn, const fem::FunctionSpace& V,
 /// 'scale'. The vectors b and x0 must have the same local size. The bcs
 /// should be on (sub-)spaces of the form L that b represents.
 template <typename T>
-void set_bc(xtl::span<T> b,
+void set_bc(std::span<T> b,
             const std::vector<std::shared_ptr<const DirichletBC<T>>>& bcs,
-            const xtl::span<const T>& x0, double scale = 1.0)
+            const std::span<const T>& x0, double scale = 1.0)
 {
   if (b.size() > x0.size())
     throw std::runtime_error("Size mismatch between b and x0 vectors.");
@@ -389,7 +389,7 @@ void set_bc(xtl::span<T> b,
 /// 'scale'. The bcs should be on (sub-)spaces of the form L that b
 /// represents.
 template <typename T>
-void set_bc(xtl::span<T> b,
+void set_bc(std::span<T> b,
             const std::vector<std::shared_ptr<const DirichletBC<T>>>& bcs,
             double scale = 1.0)
 {
