@@ -6,10 +6,8 @@
 
 #pragma once
 
-#include <dolfinx/graph/AdjacencyList.h>
-#include <dolfinx/graph/scotch.h>
+#include <cstdint>
 #include <functional>
-#include <memory>
 #include <mpi.h>
 #include <tuple>
 
@@ -23,10 +21,15 @@ namespace dolfinx::mesh
 class Topology;
 }
 
+namespace dolfinx::graph
+{
+template <typename T>
+class AdjacencyList;
+}
+
 namespace dolfinx::fem
 {
 class ElementDofLayout;
-class CoordinateElement;
 
 /// Build dofmap data for an element on a mesh topology
 /// @param[in] comm MPI communicator
@@ -36,8 +39,7 @@ class CoordinateElement;
 /// @param[in] reorder_fn Graph reordering function that is applied to
 /// the dofmap
 /// @return The index map and local to global DOF data for the DOF map
-std::tuple<std::shared_ptr<common::IndexMap>, int,
-           graph::AdjacencyList<std::int32_t>>
+std::tuple<common::IndexMap, int, graph::AdjacencyList<std::int32_t>>
 build_dofmap_data(MPI_Comm comm, const mesh::Topology& topology,
                   const ElementDofLayout& element_dof_layout,
                   const std::function<std::vector<int>(
