@@ -8,35 +8,41 @@
 #include "FiniteElement.h"
 #include "FunctionSpace.h"
 #include <dolfinx/mesh/Mesh.h>
-#include <xtensor/xbuilder.hpp>
 
 using namespace dolfinx;
 
 //-----------------------------------------------------------------------------
-xt::xtensor<double, 2>
-fem::interpolation_coords(const fem::FiniteElement& element,
-                          const mesh::Mesh& mesh,
-                          const xtl::span<const std::int32_t>& cells)
+std::vector<double>
+fem::interpolation_coords(const FiniteElement& element, const mesh::Mesh& mesh,
+                          std::span<const std::int32_t> cells)
 {
   // Get mesh geometry data and the element coordinate map
   const std::size_t gdim = mesh.geometry().dim();
   const graph::AdjacencyList<std::int32_t>& x_dofmap = mesh.geometry().dofmap();
 
-  xtl::span<const double> x_g = mesh.geometry().x();
-  const fem::CoordinateElement& cmap = mesh.geometry().cmap();
+  std::span<const double> x_g = mesh.geometry().x();
+  const CoordinateElement& cmap = mesh.geometry().cmap();
   const std::size_t num_dofs_g = cmap.dim();
 
   // Get the interpolation points on the reference cells
-  const xt::xtensor<double, 2>& X = element.interpolation_points();
-  const xt::xtensor<double, 2> phi
-      = xt::view(cmap.tabulate(0, X), 0, xt::all(), xt::all(), 0);
+  const auto [X, Xshape] = element.interpolation_points();
+
+  // Evaluate coordinate element basis at reference points
+  namespace stdex = std::experimental;
+  using cmdspan4_t
+      = stdex::mdspan<const double, stdex::dextents<std::size_t, 4>>;
+  std::array<std::size_t, 4> phi_shape = cmap.tabulate_shape(0, Xshape[0]);
+  std::vector<double> phi_b(
+      std::reduce(phi_shape.begin(), phi_shape.end(), 1, std::multiplies{}));
+  cmdspan4_t phi_full(phi_b.data(), phi_shape);
+  cmap.tabulate(0, X, Xshape, phi_b);
+  auto phi = stdex::submdspan(phi_full, 0, stdex::full_extent,
+                              stdex::full_extent, 0);
 
   // Push reference coordinates (X) forward to the physical coordinates
   // (x) for each cell
-  xt::xtensor<double, 2> coordinate_dofs
-      = xt::zeros<double>({num_dofs_g, gdim});
-  std::array<std::size_t, 2> shape = {3, cells.size() * X.shape(0)};
-  xt::xtensor<double, 2> x = xt::zeros<double>(shape);
+  std::vector<double> coordinate_dofs(num_dofs_g * gdim, 0);
+  std::vector<double> x(3 * (cells.size() * Xshape[0]), 0);
   for (std::size_t c = 0; c < cells.size(); ++c)
   {
     // Get geometry data for current cell
@@ -48,14 +54,14 @@ fem::interpolation_coords(const fem::FiniteElement& element,
     }
 
     // Push forward coordinates (X -> x)
-    for (std::size_t p = 0; p < X.shape(0); ++p)
+    for (std::size_t p = 0; p < Xshape[0]; ++p)
     {
       for (std::size_t j = 0; j < gdim; ++j)
       {
         double acc = 0;
         for (std::size_t k = 0; k < num_dofs_g; ++k)
-          acc += phi(p, k) * coordinate_dofs(k, j);
-        x(j, c * X.shape(0) + p) = acc;
+          acc += phi(p, k) * coordinate_dofs[k * gdim + j];
+        x[j * (cells.size() * Xshape[0]) + c * Xshape[0] + p] = acc;
       }
     }
   }
