@@ -11,9 +11,9 @@
 #include <memory>
 #include <petscmat.h>
 #include <petscvec.h>
+#include <span>
 #include <utility>
 #include <vector>
-#include <xtl/xspan.hpp>
 
 namespace dolfinx::common
 {
@@ -47,14 +47,14 @@ Mat create_matrix(const Form<PetscScalar>& a,
 /// bilinear forms. The caller is responsible for destroying the Mat
 /// object.
 Mat create_matrix_block(
-    const std::vector<std::vector<const fem::Form<PetscScalar>*>>& a,
+    const std::vector<std::vector<const Form<PetscScalar>*>>& a,
     const std::string& type = std::string());
 
 /// Create nested (MatNest) matrix
 ///
 /// The caller is responsible for destroying the Mat object
 Mat create_matrix_nest(
-    const std::vector<std::vector<const fem::Form<PetscScalar>*>>& a,
+    const std::vector<std::vector<const Form<PetscScalar>*>>& a,
     const std::vector<std::vector<std::string>>& types);
 
 /// Initialise monolithic vector. Vector is not zeroed.
@@ -84,9 +84,9 @@ Vec create_vector_nest(
 /// @param[in] coeffs The coefficients that appear in `L`
 void assemble_vector(
     Vec b, const Form<PetscScalar>& L,
-    const xtl::span<const PetscScalar>& constants,
+    const std::span<const PetscScalar>& constants,
     const std::map<std::pair<IntegralType, int>,
-                   std::pair<xtl::span<const PetscScalar>, int>>& coeffs);
+                   std::pair<std::span<const PetscScalar>, int>>& coeffs);
 
 /// Assemble linear form into an already allocated PETSc vector. Ghost
 /// contributions are not accumulated (not sent to owner). Caller is
@@ -119,9 +119,9 @@ void assemble_vector(Vec b, const Form<PetscScalar>& L);
 /// is responsible for calling VecGhostUpdateBegin/End.
 void apply_lifting(
     Vec b, const std::vector<std::shared_ptr<const Form<PetscScalar>>>& a,
-    const std::vector<xtl::span<const PetscScalar>>& constants,
+    const std::vector<std::span<const PetscScalar>>& constants,
     const std::vector<std::map<std::pair<IntegralType, int>,
-                               std::pair<xtl::span<const PetscScalar>, int>>>&
+                               std::pair<std::span<const PetscScalar>, int>>>&
         coeffs,
     const std::vector<
         std::vector<std::shared_ptr<const DirichletBC<PetscScalar>>>>& bcs1,
