@@ -17,8 +17,7 @@ using namespace dolfinx::fem;
 //-----------------------------------------------------------------------------
 void sparsitybuild::cells(
     la::SparsityPattern& pattern, const mesh::Topology& topology,
-    const std::array<const std::reference_wrapper<const fem::DofMap>, 2>&
-        dofmaps)
+    const std::array<const std::reference_wrapper<const DofMap>, 2>& dofmaps)
 {
   const int D = topology.dim();
   auto cells = topology.connectivity(D, 0);
@@ -30,10 +29,20 @@ void sparsitybuild::cells(
   }
 }
 //-----------------------------------------------------------------------------
+void sparsitybuild::cells(
+    la::SparsityPattern& pattern, const std::span<const std::int32_t>& cells,
+    const std::array<const std::reference_wrapper<const DofMap>, 2>& dofmaps)
+{
+  for (std::int32_t c : cells)
+  {
+    pattern.insert(dofmaps[0].get().cell_dofs(c),
+                   dofmaps[1].get().cell_dofs(c));
+  }
+}
+//-----------------------------------------------------------------------------
 void sparsitybuild::interior_facets(
     la::SparsityPattern& pattern, const mesh::Topology& topology,
-    const std::array<const std::reference_wrapper<const fem::DofMap>, 2>&
-        dofmaps)
+    const std::array<const std::reference_wrapper<const DofMap>, 2>& dofmaps)
 {
   const int D = topology.dim();
   if (!topology.connectivity(D - 1, 0))
@@ -54,6 +63,7 @@ void sparsitybuild::interior_facets(
   {
     // Get cells incident with facet
     auto cells = connectivity->links(f);
+
     // Proceed to next facet if only ony connection
     if (cells.size() == 1)
       continue;
@@ -76,10 +86,32 @@ void sparsitybuild::interior_facets(
   }
 }
 //-----------------------------------------------------------------------------
+void sparsitybuild::interior_facets(
+    la::SparsityPattern& pattern, const std::span<const std::int32_t>& facets,
+    const std::array<const std::reference_wrapper<const DofMap>, 2>& dofmaps)
+{
+  std::array<std::vector<std::int32_t>, 2> macro_dofs;
+  for (std::size_t index = 0; index < facets.size(); index += 4)
+  {
+    const int cell_0 = facets[index];
+    const int cell_1 = facets[index + 2];
+    for (std::size_t i = 0; i < 2; ++i)
+    {
+      auto cell_dofs_0 = dofmaps[i].get().cell_dofs(cell_0);
+      auto cell_dofs_1 = dofmaps[i].get().cell_dofs(cell_1);
+      macro_dofs[i].resize(cell_dofs_0.size() + cell_dofs_1.size());
+      std::copy(cell_dofs_0.begin(), cell_dofs_0.end(), macro_dofs[i].begin());
+      std::copy(cell_dofs_1.begin(), cell_dofs_1.end(),
+                std::next(macro_dofs[i].begin(), cell_dofs_0.size()));
+    }
+
+    pattern.insert(macro_dofs[0], macro_dofs[1]);
+  }
+}
+//-----------------------------------------------------------------------------
 void sparsitybuild::exterior_facets(
     la::SparsityPattern& pattern, const mesh::Topology& topology,
-    const std::array<const std::reference_wrapper<const fem::DofMap>, 2>&
-        dofmaps)
+    const std::array<const std::reference_wrapper<const DofMap>, 2>& dofmaps)
 {
   const int D = topology.dim();
   if (!topology.connectivity(D - 1, 0))
@@ -106,3 +138,15 @@ void sparsitybuild::exterior_facets(
   }
 }
 //-----------------------------------------------------------------------------
+void sparsitybuild::exterior_facets(
+    la::SparsityPattern& pattern, const std::span<const std::int32_t>& facets,
+    const std::array<const std::reference_wrapper<const DofMap>, 2>& dofmaps)
+{
+  for (std::size_t index = 0; index < facets.size(); index += 2)
+  {
+    std::int32_t cell = facets[index];
+    pattern.insert(dofmaps[0].get().cell_dofs(cell),
+                   dofmaps[1].get().cell_dofs(cell));
+  }
+}
+//-----------------------------------------------------------------------------
