@@ -13,19 +13,20 @@
 #include "Expression.h"
 #include "Form.h"
 #include "Function.h"
+#include "sparsitybuild.h"
+#include <array>
 #include <dolfinx/la/SparsityPattern.h>
 #include <dolfinx/mesh/cell_types.h>
 #include <functional>
 #include <memory>
 #include <set>
+#include <span>
 #include <stdexcept>
 #include <string>
+#include <type_traits>
 #include <ufcx.h>
 #include <utility>
 #include <vector>
-#include <xtensor/xadapt.hpp>
-#include <xtensor/xtensor.hpp>
-#include <xtl/xspan.hpp>
 
 /// @file utils.h
 /// @brief Functions supporting finite element method operations
@@ -50,6 +51,28 @@ namespace dolfinx::fem
 {
 class FunctionSpace;
 
+namespace impl
+{
+/// @private These structs are used to get the float/value type from a
+/// template argument, including support for complex types
+template <typename T, typename = void>
+struct scalar_value_type
+{
+  /// @internal
+  typedef T value_type;
+};
+/// @private
+template <typename T>
+struct scalar_value_type<T, std::void_t<typename T::value_type>>
+{
+  typedef typename T::value_type value_type;
+};
+/// @private Convenience typedef
+template <typename T>
+using scalar_value_type_t = typename scalar_value_type<T>::value_type;
+
+} // namespace impl
+
 /// @brief Extract test (0) and trial (1) function spaces pairs for each
 /// bilinear form for a rectangular array of forms
 ///
@@ -115,7 +138,52 @@ la::SparsityPattern create_sparsity_pattern(const Form<T>& a)
     mesh->topology_mutable().create_connectivity(tdim - 1, tdim);
   }
 
-  return create_sparsity_pattern(mesh->topology(), dofmaps, types);
+  common::Timer t0("Build sparsity");
+
+  // Get common::IndexMaps for each dimension
+  const std::array index_maps{dofmaps[0].get().index_map,
+                              dofmaps[1].get().index_map};
+  const std::array bs
+      = {dofmaps[0].get().index_map_bs(), dofmaps[1].get().index_map_bs()};
+
+  // Create and build sparsity pattern
+  la::SparsityPattern pattern(mesh->comm(), index_maps, bs);
+  for (auto type : types)
+  {
+    std::vector<int> ids = a.integral_ids(type);
+    switch (type)
+    {
+    case IntegralType::cell:
+      for (int id : ids)
+      {
+        const std::vector<std::int32_t>& cells = a.cell_domains(id);
+        sparsitybuild::cells(pattern, cells, {{dofmaps[0], dofmaps[1]}});
+      }
+      break;
+    case IntegralType::interior_facet:
+      for (int id : ids)
+      {
+        const std::vector<std::int32_t>& facets = a.interior_facet_domains(id);
+        sparsitybuild::interior_facets(pattern, facets,
+                                       {{dofmaps[0], dofmaps[1]}});
+      }
+      break;
+    case IntegralType::exterior_facet:
+      for (int id : ids)
+      {
+        const std::vector<std::int32_t>& facets = a.exterior_facet_domains(id);
+        sparsitybuild::exterior_facets(pattern, facets,
+                                       {{dofmaps[0], dofmaps[1]}});
+      }
+      break;
+    default:
+      throw std::runtime_error("Unsupported integral type");
+    }
+  }
+
+  t0.stop();
+
+  return pattern;
 }
 
 /// Create an ElementDofLayout from a ufcx_dofmap
@@ -196,8 +264,10 @@ Form<T> create_form(
 
   // Get list of integral IDs, and load tabulate tensor into memory for
   // each
-  using kern = std::function<void(T*, const T*, const T*, const double*,
-                                  const int*, const std::uint8_t*)>;
+  using kern = std::function<void(
+      T*, const T*, const T*,
+      const typename impl::scalar_value_type<T>::value_type*, const int*,
+      const std::uint8_t*)>;
   std::map<IntegralType, std::pair<std::vector<std::pair<int, kern>>,
                                    const mesh::MeshTags<int>*>>
       integral_data;
@@ -214,21 +284,23 @@ Form<T> create_form(
     assert(integral);
 
     kern k = nullptr;
-    if constexpr (std::is_same<T, float>::value)
+    if constexpr (std::is_same_v<T, float>)
       k = integral->tabulate_tensor_float32;
-    else if constexpr (std::is_same<T, std::complex<float>>::value)
+    else if constexpr (std::is_same_v<T, std::complex<float>>)
     {
-      k = reinterpret_cast<void (*)(T*, const T*, const T*, const double*,
-                                    const int*, const unsigned char*)>(
-          integral->tabulate_tensor_complex64);
+      k = reinterpret_cast<void (*)(
+          T*, const T*, const T*,
+          const typename impl::scalar_value_type<T>::value_type*, const int*,
+          const unsigned char*)>(integral->tabulate_tensor_complex64);
     }
-    else if constexpr (std::is_same<T, double>::value)
+    else if constexpr (std::is_same_v<T, double>)
       k = integral->tabulate_tensor_float64;
-    else if constexpr (std::is_same<T, std::complex<double>>::value)
+    else if constexpr (std::is_same_v<T, std::complex<double>>)
     {
-      k = reinterpret_cast<void (*)(T*, const T*, const T*, const double*,
-                                    const int*, const unsigned char*)>(
-          integral->tabulate_tensor_complex128);
+      k = reinterpret_cast<void (*)(
+          T*, const T*, const T*,
+          const typename impl::scalar_value_type<T>::value_type*, const int*,
+          const unsigned char*)>(integral->tabulate_tensor_complex128);
     }
     assert(k);
 
@@ -270,21 +342,23 @@ Form<T> create_form(
     assert(integral);
 
     kern k = nullptr;
-    if constexpr (std::is_same<T, float>::value)
+    if constexpr (std::is_same_v<T, float>)
       k = integral->tabulate_tensor_float32;
-    else if constexpr (std::is_same<T, std::complex<float>>::value)
+    else if constexpr (std::is_same_v<T, std::complex<float>>)
     {
-      k = reinterpret_cast<void (*)(T*, const T*, const T*, const double*,
-                                    const int*, const unsigned char*)>(
-          integral->tabulate_tensor_complex64);
+      k = reinterpret_cast<void (*)(
+          T*, const T*, const T*,
+          const typename impl::scalar_value_type<T>::value_type*, const int*,
+          const unsigned char*)>(integral->tabulate_tensor_complex64);
     }
-    else if constexpr (std::is_same<T, double>::value)
+    else if constexpr (std::is_same_v<T, double>)
       k = integral->tabulate_tensor_float64;
-    else if constexpr (std::is_same<T, std::complex<double>>::value)
+    else if constexpr (std::is_same_v<T, std::complex<double>>)
     {
-      k = reinterpret_cast<void (*)(T*, const T*, const T*, const double*,
-                                    const int*, const unsigned char*)>(
-          integral->tabulate_tensor_complex128);
+      k = reinterpret_cast<void (*)(
+          T*, const T*, const T*,
+          const typename impl::scalar_value_type<T>::value_type*, const int*,
+          const unsigned char*)>(integral->tabulate_tensor_complex128);
     }
     assert(k);
 
@@ -312,21 +386,23 @@ Form<T> create_form(
     assert(integral);
 
     kern k = nullptr;
-    if constexpr (std::is_same<T, float>::value)
+    if constexpr (std::is_same_v<T, float>)
       k = integral->tabulate_tensor_float32;
-    else if constexpr (std::is_same<T, std::complex<float>>::value)
+    else if constexpr (std::is_same_v<T, std::complex<float>>)
     {
-      k = reinterpret_cast<void (*)(T*, const T*, const T*, const double*,
-                                    const int*, const unsigned char*)>(
-          integral->tabulate_tensor_complex64);
+      k = reinterpret_cast<void (*)(
+          T*, const T*, const T*,
+          const typename impl::scalar_value_type<T>::value_type*, const int*,
+          const unsigned char*)>(integral->tabulate_tensor_complex64);
     }
-    else if constexpr (std::is_same<T, double>::value)
+    else if constexpr (std::is_same_v<T, double>)
       k = integral->tabulate_tensor_float64;
-    else if constexpr (std::is_same<T, std::complex<double>>::value)
+    else if constexpr (std::is_same_v<T, std::complex<double>>)
     {
-      k = reinterpret_cast<void (*)(T*, const T*, const T*, const double*,
-                                    const int*, const unsigned char*)>(
-          integral->tabulate_tensor_complex128);
+      k = reinterpret_cast<void (*)(
+          T*, const T*, const T*,
+          const typename impl::scalar_value_type<T>::value_type*, const int*,
+          const unsigned char*)>(integral->tabulate_tensor_complex128);
     }
     assert(k);
 
@@ -459,7 +535,7 @@ namespace impl
 {
 /// @private
 template <typename T>
-xtl::span<const std::uint32_t> get_cell_orientation_info(
+std::span<const std::uint32_t> get_cell_orientation_info(
     const std::vector<std::shared_ptr<const Function<T>>>& coefficients)
 {
   bool needs_dof_transformations = false;
@@ -474,12 +550,12 @@ xtl::span<const std::uint32_t> get_cell_orientation_info(
     }
   }
 
-  xtl::span<const std::uint32_t> cell_info;
+  std::span<const std::uint32_t> cell_info;
   if (needs_dof_transformations)
   {
     auto mesh = coefficients.front()->function_space()->mesh();
     mesh->topology_mutable().create_entity_permutations();
-    cell_info = xtl::span(mesh->topology().get_cell_permutation_info());
+    cell_info = std::span(mesh->topology().get_cell_permutation_info());
   }
 
   return cell_info;
@@ -487,9 +563,9 @@ xtl::span<const std::uint32_t> get_cell_orientation_info(
 
 // Pack a single coefficient for a single cell
 template <typename T, int _bs, typename Functor>
-static inline void pack(const xtl::span<T>& coeffs, std::int32_t cell, int bs,
-                        const xtl::span<const T>& v,
-                        const xtl::span<const std::uint32_t>& cell_info,
+static inline void pack(const std::span<T>& coeffs, std::int32_t cell, int bs,
+                        const std::span<const T>& v,
+                        const std::span<const std::uint32_t>& cell_info,
                         const DofMap& dofmap, Functor transform)
 {
   auto dofs = dofmap.cell_dofs(cell);
@@ -523,19 +599,21 @@ static inline void pack(const xtl::span<T>& coeffs, std::int32_t cell, int bs,
 /// @param[in] cell_info Array of bytes describing which transformation
 /// has to be applied on the cell to map it to the reference element
 /// @param[in] entities The set of active entities
+/// @param[in] estride The stride for each entity in active entities.
 /// @param[in] fetch_cells Function that fetches the cell index for an
 /// entity in active_entities (signature:
 /// `std::function<std::int32_t(E::value_type)>`)
 /// @param[in] offset The offset for c
-template <typename T, typename E, typename Functor>
-void pack_coefficient_entity(const xtl::span<T>& c, int cstride,
+template <typename T, typename Functor>
+void pack_coefficient_entity(const std::span<T>& c, int cstride,
                              const Function<T>& u,
-                             const xtl::span<const std::uint32_t>& cell_info,
-                             const E& entities, Functor fetch_cells,
+                             const std::span<const std::uint32_t>& cell_info,
+                             const std::span<const std::int32_t>& entities,
+                             std::size_t estride, Functor fetch_cells,
                              std::int32_t offset)
 {
   // Read data from coefficient "u"
-  const xtl::span<const T>& v = u.x()->array();
+  const std::span<const T>& v = u.x()->array();
   const DofMap& dofmap = *u.function_space()->dofmap();
   std::shared_ptr<const FiniteElement> element = u.function_space()->element();
   int space_dim = element->space_dimension();
@@ -546,34 +624,38 @@ void pack_coefficient_entity(const xtl::span<T>& c, int cstride,
   switch (bs)
   {
   case 1:
-    for (std::size_t e = 0; e < entities.size(); ++e)
+    for (std::size_t e = 0; e < entities.size(); e += estride)
     {
-      std::int32_t cell = fetch_cells(entities[e]);
-      auto cell_coeff = c.subspan(e * cstride + offset, space_dim);
+      auto entity = entities.subspan(e, estride);
+      std::int32_t cell = fetch_cells(entity);
+      auto cell_coeff = c.subspan(e / estride * cstride + offset, space_dim);
       pack<T, 1>(cell_coeff, cell, bs, v, cell_info, dofmap, transformation);
     }
     break;
   case 2:
-    for (std::size_t e = 0; e < entities.size(); ++e)
+    for (std::size_t e = 0; e < entities.size(); e += estride)
     {
-      std::int32_t cell = fetch_cells(entities[e]);
-      auto cell_coeff = c.subspan(e * cstride + offset, space_dim);
+      auto entity = entities.subspan(e, estride);
+      std::int32_t cell = fetch_cells(entity);
+      auto cell_coeff = c.subspan(e / estride * cstride + offset, space_dim);
       pack<T, 2>(cell_coeff, cell, bs, v, cell_info, dofmap, transformation);
     }
     break;
   case 3:
-    for (std::size_t e = 0; e < entities.size(); ++e)
+    for (std::size_t e = 0; e < entities.size(); e += estride)
     {
-      std::int32_t cell = fetch_cells(entities[e]);
-      auto cell_coeff = c.subspan(e * cstride + offset, space_dim);
+      auto entity = entities.subspan(e, estride);
+      std::int32_t cell = fetch_cells(entity);
+      auto cell_coeff = c.subspan(e / estride * cstride + offset, space_dim);
       pack<T, 3>(cell_coeff, cell, bs, v, cell_info, dofmap, transformation);
     }
     break;
   default:
-    for (std::size_t e = 0; e < entities.size(); ++e)
+    for (std::size_t e = 0; e < entities.size(); e += estride)
     {
-      std::int32_t cell = fetch_cells(entities[e]);
-      auto cell_coeff = c.subspan(e * cstride + offset, space_dim);
+      auto entity = entities.subspan(e, estride);
+      std::int32_t cell = fetch_cells(entity);
+      auto cell_coeff = c.subspan(e / estride * cstride + offset, space_dim);
       pack<T, -1>(cell_coeff, cell, bs, v, cell_info, dofmap, transformation);
     }
     break;
@@ -609,10 +691,10 @@ allocate_coefficient_storage(const Form<T>& form, IntegralType integral_type,
       num_entities = form.cell_domains(id).size();
       break;
     case IntegralType::exterior_facet:
-      num_entities = form.exterior_facet_domains(id).size();
+      num_entities = form.exterior_facet_domains(id).size() / 2;
       break;
     case IntegralType::interior_facet:
-      num_entities = form.interior_facet_domains(id).size() * 2;
+      num_entities = form.interior_facet_domains(id).size() / 2;
       break;
     default:
       throw std::runtime_error(
@@ -654,7 +736,7 @@ allocate_coefficient_storage(const Form<T>& form)
 /// @param[in] cstride The coefficient stride
 template <typename T>
 void pack_coefficients(const Form<T>& form, IntegralType integral_type, int id,
-                       const xtl::span<T>& c, int cstride)
+                       const std::span<T>& c, int cstride)
 {
   // Get form coefficient offsets and dofmaps
   const std::vector<std::shared_ptr<const Function<T>>>& coefficients
@@ -663,37 +745,36 @@ void pack_coefficients(const Form<T>& form, IntegralType integral_type, int id,
 
   if (!coefficients.empty())
   {
-    xtl::span<const std::uint32_t> cell_info
+    std::span<const std::uint32_t> cell_info
         = impl::get_cell_orientation_info(coefficients);
 
     switch (integral_type)
     {
     case IntegralType::cell:
     {
-      auto fetch_cell = [](auto entity) { return entity; };
+      auto fetch_cell = [](auto entity) { return entity.front(); };
       const std::vector<std::int32_t>& cells = form.cell_domains(id);
       // Iterate over coefficients
       for (std::size_t coeff = 0; coeff < coefficients.size(); ++coeff)
       {
         impl::pack_coefficient_entity(c, cstride, *coefficients[coeff],
-                                      cell_info, cells, fetch_cell,
+                                      cell_info, cells, 1, fetch_cell,
                                       offsets[coeff]);
       }
       break;
     }
     case IntegralType::exterior_facet:
     {
-      const std::vector<std::pair<std::int32_t, int>>& facets
-          = form.exterior_facet_domains(id);
+      const std::vector<std::int32_t>& facets = form.exterior_facet_domains(id);
 
       // Create lambda function fetching cell index from exterior facet entity
-      auto fetch_cell = [](auto& entity) { return entity.first; };
+      auto fetch_cell = [](auto& entity) { return entity.front(); };
 
       // Iterate over coefficients
       for (std::size_t coeff = 0; coeff < coefficients.size(); ++coeff)
       {
         impl::pack_coefficient_entity(c, cstride, *coefficients[coeff],
-                                      cell_info, facets, fetch_cell,
+                                      cell_info, facets, 2, fetch_cell,
                                       offsets[coeff]);
       }
 
@@ -701,23 +782,21 @@ void pack_coefficients(const Form<T>& form, IntegralType integral_type, int id,
     }
     case IntegralType::interior_facet:
     {
-      const std::vector<std::tuple<std::int32_t, int, std::int32_t, int>>&
-          facets
-          = form.interior_facet_domains(id);
+      const std::vector<std::int32_t>& facets = form.interior_facet_domains(id);
       // Lambda functions to fetch cell index from interior facet entity
-      auto fetch_cell0 = [](auto& entity) { return std::get<0>(entity); };
-      auto fetch_cell1 = [](auto& entity) { return std::get<2>(entity); };
+      auto fetch_cell0 = [](auto& entity) { return entity[0]; };
+      auto fetch_cell1 = [](auto& entity) { return entity[2]; };
 
       // Iterate over coefficients
       for (std::size_t coeff = 0; coeff < coefficients.size(); ++coeff)
       {
         // Pack coefficient ['+']
         impl::pack_coefficient_entity(c, 2 * cstride, *coefficients[coeff],
-                                      cell_info, facets, fetch_cell0,
+                                      cell_info, facets, 4, fetch_cell0,
                                       2 * offsets[coeff]);
         // Pack coefficient ['-']
         impl::pack_coefficient_entity(c, 2 * cstride, *coefficients[coeff],
-                                      cell_info, facets, fetch_cell1,
+                                      cell_info, facets, 4, fetch_cell1,
                                       offsets[coeff] + offsets[coeff + 1]);
       }
       break;
@@ -731,12 +810,12 @@ void pack_coefficients(const Form<T>& form, IntegralType integral_type, int id,
 
 /// @brief Create Expression from UFC
 template <typename T>
-fem::Expression<T> create_expression(
+Expression<T> create_expression(
     const ufcx_expression& expression,
-    const std::vector<std::shared_ptr<const fem::Function<T>>>& coefficients,
-    const std::vector<std::shared_ptr<const fem::Constant<T>>>& constants,
+    const std::vector<std::shared_ptr<const Function<T>>>& coefficients,
+    const std::vector<std::shared_ptr<const Constant<T>>>& constants,
     const std::shared_ptr<const mesh::Mesh>& mesh = nullptr,
-    const std::shared_ptr<const fem::FunctionSpace>& argument_function_space
+    const std::shared_ptr<const FunctionSpace>& argument_function_space
     = nullptr)
 {
   if (expression.rank > 0 and !argument_function_space)
@@ -745,37 +824,38 @@ fem::Expression<T> create_expression(
         "Expression has Argument but no Argument function space was provided.");
   }
 
-  const int size = expression.num_points * expression.topological_dimension;
-  const xt::xtensor<double, 2> points = xt::adapt(
-      expression.points, size, xt::no_ownership(),
-      std::vector<std::size_t>(
-          {static_cast<std::size_t>(expression.num_points),
-           static_cast<std::size_t>(expression.topological_dimension)}));
+  const std::size_t size
+      = expression.num_points * expression.topological_dimension;
+  std::span<const double> X(expression.points, size);
+  std::array<std::size_t, 2> Xshape
+      = {static_cast<std::size_t>(expression.num_points),
+         static_cast<std::size_t>(expression.topological_dimension)};
 
   std::vector<int> value_shape;
   for (int i = 0; i < expression.num_components; ++i)
     value_shape.push_back(expression.value_shape[i]);
 
-  std::function<void(T*, const T*, const T*, const double*, const int*,
-                     const std::uint8_t*)>
+  std::function<void(T*, const T*, const T*,
+                     const typename impl::scalar_value_type<T>::value_type*,
+                     const int*, const std::uint8_t*)>
       tabulate_tensor = nullptr;
-  if constexpr (std::is_same<T, float>::value)
+  if constexpr (std::is_same_v<T, float>)
     tabulate_tensor = expression.tabulate_tensor_float32;
-  else if constexpr (std::is_same<T, std::complex<float>>::value)
+  else if constexpr (std::is_same_v<T, std::complex<float>>)
   {
-    tabulate_tensor
-        = reinterpret_cast<void (*)(T*, const T*, const T*, const double*,
-                                    const int*, const unsigned char*)>(
-            expression.tabulate_tensor_complex64);
+    tabulate_tensor = reinterpret_cast<void (*)(
+        T*, const T*, const T*,
+        const typename impl::scalar_value_type<T>::value_type*, const int*,
+        const unsigned char*)>(expression.tabulate_tensor_complex64);
   }
-  else if constexpr (std::is_same<T, double>::value)
+  else if constexpr (std::is_same_v<T, double>)
     tabulate_tensor = expression.tabulate_tensor_float64;
-  else if constexpr (std::is_same<T, std::complex<double>>::value)
+  else if constexpr (std::is_same_v<T, std::complex<double>>)
   {
-    tabulate_tensor
-        = reinterpret_cast<void (*)(T*, const T*, const T*, const double*,
-                                    const int*, const unsigned char*)>(
-            expression.tabulate_tensor_complex128);
+    tabulate_tensor = reinterpret_cast<void (*)(
+        T*, const T*, const T*,
+        const typename impl::scalar_value_type<T>::value_type*, const int*,
+        const unsigned char*)>(expression.tabulate_tensor_complex128);
   }
   else
   {
@@ -783,25 +863,24 @@ fem::Expression<T> create_expression(
   }
   assert(tabulate_tensor);
 
-  return fem::Expression(coefficients, constants, points, tabulate_tensor,
-                         value_shape, mesh, argument_function_space);
+  return Expression(coefficients, constants, X, Xshape, tabulate_tensor,
+                    value_shape, mesh, argument_function_space);
 }
 
 /// @brief Create Expression from UFC input (with named coefficients and
 /// constants)
 template <typename T>
-fem::Expression<T> create_expression(
+Expression<T> create_expression(
     const ufcx_expression& expression,
-    const std::map<std::string, std::shared_ptr<const fem::Function<T>>>&
+    const std::map<std::string, std::shared_ptr<const Function<T>>>&
         coefficients,
-    const std::map<std::string, std::shared_ptr<const fem::Constant<T>>>&
-        constants,
+    const std::map<std::string, std::shared_ptr<const Constant<T>>>& constants,
     const std::shared_ptr<const mesh::Mesh>& mesh = nullptr,
-    const std::shared_ptr<const fem::FunctionSpace>& argument_function_space
+    const std::shared_ptr<const FunctionSpace>& argument_function_space
     = nullptr)
 {
   // Place coefficients in appropriate order
-  std::vector<std::shared_ptr<const fem::Function<T>>> coeff_map;
+  std::vector<std::shared_ptr<const Function<T>>> coeff_map;
   std::vector<std::string> coefficient_names;
   for (int i = 0; i < expression.num_coefficients; ++i)
     coefficient_names.push_back(expression.coefficient_names[i]);
@@ -818,7 +897,7 @@ fem::Expression<T> create_expression(
   }
 
   // Place constants in appropriate order
-  std::vector<std::shared_ptr<const fem::Constant<T>>> const_map;
+  std::vector<std::shared_ptr<const Constant<T>>> const_map;
   std::vector<std::string> constant_names;
   for (int i = 0; i < expression.num_constants; ++i)
     constant_names.push_back(expression.constant_names[i]);
@@ -861,7 +940,7 @@ void pack_coefficients(const Form<T>& form,
 template <typename T>
 std::pair<std::vector<T>, int>
 pack_coefficients(const Expression<T>& u,
-                  const xtl::span<const std::int32_t>& cells)
+                  const std::span<const std::int32_t>& cells)
 {
   // Get form coefficient offsets and dofmaps
   const std::vector<std::shared_ptr<const Function<T>>>& coefficients
@@ -873,14 +952,14 @@ pack_coefficients(const Expression<T>& u,
   std::vector<T> c(cells.size() * offsets.back());
   if (!coefficients.empty())
   {
-    xtl::span<const std::uint32_t> cell_info
+    std::span<const std::uint32_t> cell_info
         = impl::get_cell_orientation_info(coefficients);
 
     // Iterate over coefficients
     for (std::size_t coeff = 0; coeff < coefficients.size(); ++coeff)
       impl::pack_coefficient_entity(
-          xtl::span(c), cstride, *coefficients[coeff], cell_info, cells,
-          [](std::int32_t entity) { return entity; }, offsets[coeff]);
+          std::span(c), cstride, *coefficients[coeff], cell_info, cells, 1,
+          [](auto entity) { return entity[0]; }, offsets[coeff]);
   }
   return {std::move(c), cstride};
 }
