@@ -6,7 +6,6 @@
 
 #include "XDMFFile.h"
 #include "cells.h"
-#include "pugixml.hpp"
 #include "xdmf_function.h"
 #include "xdmf_mesh.h"
 #include "xdmf_meshtags.h"
@@ -22,6 +21,7 @@
 #include <dolfinx/mesh/MeshTags.h>
 #include <dolfinx/mesh/utils.h>
 #include <filesystem>
+#include <pugixml.hpp>
 
 using namespace dolfinx;
 using namespace dolfinx::io;
@@ -231,22 +231,24 @@ mesh::Mesh XDMFFile::read_mesh(const fem::CoordinateElement& element,
                                const std::string xpath) const
 {
   // Read mesh data
-  const xt::xtensor<std::int64_t, 2> cells
-      = XDMFFile::read_topology_data(name, xpath);
-  const xt::xtensor<double, 2> x = XDMFFile::read_geometry_data(name, xpath);
+  auto [cells, cshape] = XDMFFile::read_topology_data(name, xpath);
+  auto [x, xshape] = XDMFFile::read_geometry_data(name, xpath);
 
   // Create mesh
-  auto [data, offset] = graph::create_adjacency_data(cells);
-  graph::AdjacencyList<std::int64_t> cells_adj(std::move(data),
+  std::vector<std::int32_t> offset(cshape[0] + 1, 0);
+  for (std::size_t i = 0; i < cshape[0]; ++i)
+    offset[i + 1] = offset[i] + cshape[1];
+
+  graph::AdjacencyList<std::int64_t> cells_adj(std::move(cells),
                                                std::move(offset));
 
   mesh::Mesh mesh
-      = mesh::create_mesh(_comm.comm(), cells_adj, element, x, mode);
+      = mesh::create_mesh(_comm.comm(), cells_adj, element, x, xshape, mode);
   mesh.name = name;
   return mesh;
 }
 //-----------------------------------------------------------------------------
-xt::xtensor<std::int64_t, 2>
+std::pair<std::vector<std::int64_t>, std::array<std::size_t, 2>>
 XDMFFile::read_topology_data(const std::string name,
                              const std::string xpath) const
 {
@@ -263,7 +265,7 @@ XDMFFile::read_topology_data(const std::string name,
   return xdmf_mesh::read_topology_data(_comm.comm(), _h5_id, grid_node);
 }
 //-----------------------------------------------------------------------------
-xt::xtensor<double, 2>
+std::pair<std::vector<double>, std::array<std::size_t, 2>>
 XDMFFile::read_geometry_data(const std::string name,
                              const std::string xpath) const
 {
@@ -330,7 +332,7 @@ XDMFFile::read_meshtags(const std::shared_ptr<const mesh::Mesh>& mesh,
   if (!grid_node)
     throw std::runtime_error("<Grid> with name '" + name + "' not found.");
 
-  const xt::xtensor<std::int64_t, 2> entities = read_topology_data(name, xpath);
+  const auto [entities, eshape] = read_topology_data(name, xpath);
 
   pugi::xml_node values_data_node
       = grid_node.child("Attribute").child("DataItem");
@@ -342,13 +344,12 @@ XDMFFile::read_meshtags(const std::shared_ptr<const mesh::Mesh>& mesh,
   mesh::CellType cell_type = mesh::to_type(cell_type_str.first);
 
   // Permute entities from VTK to DOLFINx ordering
-  xt::xtensor<std::int64_t, 2> entities1 = io::cells::compute_permutation(
-      entities, io::cells::perm_vtk(cell_type, entities.shape(1)));
+  std::vector<std::int64_t> entities1 = io::cells::apply_permutation(
+      entities, eshape, io::cells::perm_vtk(cell_type, eshape[1]));
 
   std::pair<std::vector<std::int32_t>, std::vector<std::int32_t>>
       entities_values = xdmf_utils::distribute_entity_data(
-          *mesh, mesh::cell_dim(cell_type),
-          xtl::span(entities1.data(), entities1.size()), values);
+          *mesh, mesh::cell_dim(cell_type), entities1, values);
 
   LOG(INFO) << "XDMF create meshtags";
   const std::size_t num_vertices_per_entity = mesh::cell_num_entities(
@@ -360,7 +361,7 @@ XDMFFile::read_meshtags(const std::shared_ptr<const mesh::Mesh>& mesh,
                                       num_vertices_per_entity);
   mesh::MeshTags meshtags = mesh::create_meshtags(
       mesh, mesh::cell_dim(cell_type), entities_adj,
-      xtl::span<const std::int32_t>(entities_values.second));
+      std::span<const std::int32_t>(entities_values.second));
   meshtags.name = name;
 
   return meshtags;
