@@ -110,17 +110,17 @@ public:
   /// @param[in] name Name of the mesh (Grid)
   /// @param[in] xpath XPath where Mesh Grid data is located
   /// @return (Cell type, degree), and cells topology (global node indexing)
-  xt::xtensor<std::int64_t, 2> read_topology_data(const std::string name,
-                                                  const std::string xpath
-                                                  = "/Xdmf/Domain") const;
+  std::pair<std::vector<std::int64_t>, std::array<std::size_t, 2>>
+  read_topology_data(const std::string name,
+                     const std::string xpath = "/Xdmf/Domain") const;
 
   /// Read Geometry data for Mesh
   /// @param[in] name Name of the mesh (Grid)
   /// @param[in] xpath XPath where Mesh Grid data is located
   /// @return points on each process
-  xt::xtensor<double, 2> read_geometry_data(const std::string name,
-                                            const std::string xpath
-                                            = "/Xdmf/Domain") const;
+  std::pair<std::vector<double>, std::array<std::size_t, 2>>
+  read_geometry_data(const std::string name,
+                     const std::string xpath = "/Xdmf/Domain") const;
 
   /// Read information about cell type
   /// @param[in] grid_name Name of Grid for which cell type is needed
