@@ -5,10 +5,11 @@ namespace dolfinx
 /*! \namespace dolfinx::io
     \brief Support for file IO
 
-    Support IO to files for checkpointing and visualisation.
+    IO to files for checkpointing and visualisation.
 */
 }
 
 // DOLFINx io interface
 
+#include <dolfinx/io/ADIOS2Writers.h>
 #include <dolfinx/io/VTKFile.h>
