@@ -6,10 +6,12 @@
 
 #include "xdmf_mesh.h"
 #include "cells.h"
-#include "pugixml.hpp"
 #include "xdmf_read.h"
 #include "xdmf_utils.h"
+#include <dolfinx/common/IndexMap.h>
 #include <dolfinx/fem/ElementDofLayout.h>
+#include <pugixml.hpp>
+#include <vector>
 
 using namespace dolfinx;
 using namespace dolfinx::io;
@@ -20,7 +22,7 @@ void xdmf_mesh::add_topology_data(MPI_Comm comm, pugi::xml_node& xml_node,
                                   const std::string path_prefix,
                                   const mesh::Topology& topology,
                                   const mesh::Geometry& geometry, int dim,
-                                  const xtl::span<const std::int32_t>& entities)
+                                  const std::span<const std::int32_t>& entities)
 {
   LOG(INFO) << "Adding topology data to node \"" << xml_node.path('/') << "\"";
 
@@ -169,7 +171,7 @@ void xdmf_mesh::add_geometry_data(MPI_Comm comm, pugi::xml_node& xml_node,
   // Increase 1D to 2D because XDMF has no "X" geometry, use "XY"
   const int width = (gdim == 1) ? 2 : gdim;
 
-  xtl::span<const double> _x = geometry.x();
+  std::span<const double> _x = geometry.x();
 
   int num_values = num_points_local * width;
   std::vector<double> x(num_values, 0.0);
@@ -223,15 +225,15 @@ void xdmf_mesh::add_mesh(MPI_Comm comm, pugi::xml_node& xml_node,
 
   add_topology_data(comm, grid_node, h5_id, path_prefix, mesh.topology(),
                     mesh.geometry(), tdim,
-                    xtl::span<std::int32_t>(cells.data(), num_cells));
+                    std::span<std::int32_t>(cells.data(), num_cells));
 
   // Add geometry node and attributes (including writing data)
   add_geometry_data(comm, grid_node, h5_id, path_prefix, mesh.geometry());
 }
 //----------------------------------------------------------------------------
-xt::xtensor<double, 2> xdmf_mesh::read_geometry_data(MPI_Comm comm,
-                                                     const hid_t h5_id,
-                                                     const pugi::xml_node& node)
+std::pair<std::vector<double>, std::array<std::size_t, 2>>
+xdmf_mesh::read_geometry_data(MPI_Comm comm, const hid_t h5_id,
+                              const pugi::xml_node& node)
 {
   // Get geometry node
   pugi::xml_node geometry_node = node.child("Geometry");
@@ -267,17 +269,10 @@ xt::xtensor<double, 2> xdmf_mesh::read_geometry_data(MPI_Comm comm,
   const std::size_t num_local_nodes = geometry_data.size() / gdim;
   std::array<std::size_t, 2> shape = {num_local_nodes, gdim};
 
-  // The below should work, but misbehaves with the Intel icpx compiler
-  // return xt::adapt(geometry_data.data(), geometry_data.size(),
-  //                  xt::no_ownership(), shape);
-
-  // Explicitly copy to an xtensor
-  xt::xtensor<double, 2> x = xt::empty<double>(shape);
-  std::copy(geometry_data.begin(), geometry_data.end(), x.begin());
-  return x;
+  return {std::move(geometry_data), shape};
 }
 //----------------------------------------------------------------------------
-xt::xtensor<std::int64_t, 2>
+std::pair<std::vector<std::int64_t>, std::array<std::size_t, 2>>
 xdmf_mesh::read_topology_data(MPI_Comm comm, const hid_t h5_id,
                               const pugi::xml_node& node)
 {
@@ -303,18 +298,11 @@ xdmf_mesh::read_topology_data(MPI_Comm comm, const hid_t h5_id,
       = xdmf_read::get_dataset<std::int64_t>(comm, topology_data_node, h5_id);
   const std::size_t num_local_cells = topology_data.size() / npoint_per_cell;
 
-  std::array<std::size_t, 2> shape = {num_local_cells, npoint_per_cell};
-
-  // The below should work, but misbehaves with the Intel icpx compiler
-  // auto cells_vtk = xt::adapt(topology_data.data(), topology_data.size(),
-  //                            xt::no_ownership(), shape);
-
-  // Explicitly copy to an xtensor
-  xt::xtensor<std::int64_t, 2> cells_vtk = xt::empty<std::int64_t>(shape);
-  std::copy(topology_data.begin(), topology_data.end(), cells_vtk.begin());
-
   //  Permute cells from VTK to DOLFINx ordering
-  return io::cells::compute_permutation(
-      cells_vtk, io::cells::perm_vtk(cell_type, cells_vtk.shape(1)));
+  std::array<std::size_t, 2> shape = {num_local_cells, npoint_per_cell};
+  std::vector<std::int64_t> cells = io::cells::apply_permutation(
+      topology_data, shape, io::cells::perm_vtk(cell_type, shape[1]));
+
+  return {std::move(cells), shape};
 }
 //----------------------------------------------------------------------------
