@@ -65,24 +65,10 @@ SparsityPattern::SparsityPattern(
     ghost_owners1.insert(ghost_owners1.end(), owners.begin(), owners.end());
 
   // Create new IndexMaps
-  {
-    std::vector<int> src_ranks = ghost_owners0;
-    std::sort(src_ranks.begin(), src_ranks.end());
-    src_ranks.erase(std::unique(src_ranks.begin(), src_ranks.end()),
-                    src_ranks.end());
-    auto dest_ranks = dolfinx::MPI::compute_graph_edges_nbx(comm, src_ranks);
-    _index_maps[0] = std::make_shared<common::IndexMap>(
-        comm, local_offset0.back(), dest_ranks, ghosts0, ghost_owners0);
-  }
-  {
-    std::vector<int> src_ranks = ghost_owners1;
-    std::sort(src_ranks.begin(), src_ranks.end());
-    src_ranks.erase(std::unique(src_ranks.begin(), src_ranks.end()),
-                    src_ranks.end());
-    auto dest_ranks = dolfinx::MPI::compute_graph_edges_nbx(comm, src_ranks);
-    _index_maps[1] = std::make_shared<common::IndexMap>(
-        comm, local_offset1.back(), dest_ranks, ghosts1, ghost_owners1);
-  }
+  _index_maps[0] = std::make_shared<common::IndexMap>(
+      comm, local_offset0.back(), ghosts0, ghost_owners0);
+  _index_maps[1] = std::make_shared<common::IndexMap>(
+      comm, local_offset1.back(), ghosts1, ghost_owners1);
 
   _row_cache.resize(_index_maps[0]->size_local()
                     + _index_maps[0]->num_ghosts());
@@ -155,8 +141,8 @@ SparsityPattern::SparsityPattern(
   }
 }
 //-----------------------------------------------------------------------------
-void SparsityPattern::insert(const xtl::span<const std::int32_t>& rows,
-                             const xtl::span<const std::int32_t>& cols)
+void SparsityPattern::insert(const std::span<const std::int32_t>& rows,
+                             const std::span<const std::int32_t>& cols)
 {
   if (_graph)
   {
@@ -180,7 +166,7 @@ void SparsityPattern::insert(const xtl::span<const std::int32_t>& rows,
   }
 }
 //-----------------------------------------------------------------------------
-void SparsityPattern::insert_diagonal(const xtl::span<const std::int32_t>& rows)
+void SparsityPattern::insert_diagonal(const std::span<const std::int32_t>& rows)
 {
   if (_graph)
   {
@@ -232,13 +218,7 @@ common::IndexMap SparsityPattern::column_index_map() const
 
   std::array range = _index_maps[1]->local_range();
   const std::int32_t local_size = range[1] - range[0];
-  std::vector<int> src_ranks = _col_ghost_owners;
-  std::sort(src_ranks.begin(), src_ranks.end());
-  src_ranks.erase(std::unique(src_ranks.begin(), src_ranks.end()),
-                  src_ranks.end());
-  auto dest_ranks
-      = dolfinx::MPI::compute_graph_edges_nbx(_comm.comm(), src_ranks);
-  return common::IndexMap(_comm.comm(), local_size, dest_ranks, _col_ghosts,
+  return common::IndexMap(_comm.comm(), local_size, _col_ghosts,
                           _col_ghost_owners);
 }
 //-----------------------------------------------------------------------------
@@ -255,48 +235,30 @@ void SparsityPattern::assemble()
   const std::int32_t local_size0 = _index_maps[0]->size_local();
   const std::array local_range0 = _index_maps[0]->local_range();
   const std::vector<std::int64_t>& ghosts0 = _index_maps[0]->ghosts();
-  const std::vector<int> ghost_owners0 = _index_maps[0]->ghost_owners();
+
+  const std::vector<int>& owners0 = _index_maps[0]->owners();
+  const std::vector<int>& src0 = _index_maps[0]->src();
 
   assert(_index_maps[1]);
   const std::int32_t local_size1 = _index_maps[1]->size_local();
   const std::array local_range1 = _index_maps[1]->local_range();
 
   _col_ghosts = _index_maps[1]->ghosts();
-  {
-    std::vector<int> neighbors = dolfinx::MPI::neighbors(
-        _index_maps[1]->comm(common::IndexMap::Direction::forward))[0];
-    _col_ghost_owners = _index_maps[1]->ghost_owners();
-    std::transform(_col_ghost_owners.cbegin(), _col_ghost_owners.cend(),
-                   _col_ghost_owners.begin(),
-                   [&neighbors](auto r) { return neighbors[r]; });
-  }
-
-  // Global to local map for ghost column indices
-  std::map<std::int64_t, std::int32_t> global_to_local;
-  std::int32_t local_i = local_size1;
-  for (std::int64_t global_i : _col_ghosts)
-    global_to_local.insert({global_i, local_i++});
-
-  // Get number of destination ranks for ghost rows
-  int outdegree(-1);
-  {
-    MPI_Comm comm = _index_maps[0]->comm(common::IndexMap::Direction::reverse);
-    int indegree(-1), weighted(-1);
-    MPI_Dist_graph_neighbors_count(comm, &indegree, &outdegree, &weighted);
-  }
+  _col_ghost_owners = _index_maps[1]->owners();
 
   // Compute size of data to send to each process
-  std::vector<std::int32_t> data_per_proc(outdegree, 0);
-  for (std::size_t i = 0; i < ghost_owners0.size(); ++i)
+  std::vector<int> send_sizes(src0.size(), 0);
+  for (std::size_t i = 0; i < owners0.size(); ++i)
   {
-    // Add to src size
-    assert(ghost_owners0[i] < (int)data_per_proc.size());
-    data_per_proc[ghost_owners0[i]] += 3 * _row_cache[i + local_size0].size();
+    auto it = std::lower_bound(src0.begin(), src0.end(), owners0[i]);
+    assert(it != src0.end() and *it == owners0[i]);
+    const int neighbour_rank = std::distance(src0.begin(), it);
+    send_sizes[neighbour_rank] += 3 * _row_cache[i + local_size0].size();
   }
 
   // Compute send displacements
-  std::vector<int> send_disp(outdegree + 1, 0);
-  std::partial_sum(data_per_proc.begin(), data_per_proc.end(),
+  std::vector<int> send_disp(send_sizes.size() + 1, 0);
+  std::partial_sum(send_sizes.begin(), send_sizes.end(),
                    std::next(send_disp.begin(), 1));
 
   // For each ghost row, pack and send (global row, global col,
@@ -304,9 +266,12 @@ void SparsityPattern::assemble()
   std::vector<int> insert_pos(send_disp);
   std::vector<std::int64_t> ghost_data(send_disp.back());
   const int rank = dolfinx::MPI::rank(_comm.comm());
-  for (std::size_t i = 0; i < ghost_owners0.size(); ++i)
+  for (std::size_t i = 0; i < owners0.size(); ++i)
   {
-    const int neighbour_rank = ghost_owners0[i];
+    auto it = std::lower_bound(src0.begin(), src0.end(), owners0[i]);
+    assert(it != src0.end() and *it == owners0[i]);
+    const int neighbour_rank = std::distance(src0.begin(), it);
+
     for (std::int32_t col_local : _row_cache[i + local_size0])
     {
       // Get index in send buffer
@@ -329,20 +294,46 @@ void SparsityPattern::assemble()
     }
   }
 
-  // Create and communicate adjacency list to neighborhood
-  const graph::AdjacencyList<std::int64_t> ghost_data_out(std::move(ghost_data),
-                                                          std::move(send_disp));
-  MPI_Comm comm = _index_maps[0]->comm(common::IndexMap::Direction::reverse);
-  const graph::AdjacencyList<std::int64_t> ghost_data_in
-      = dolfinx::MPI::neighbor_all_to_all(comm, ghost_data_out);
+  // Exchange data between processes
+  std::vector<std::int64_t> ghost_data_in;
+  {
+    MPI_Comm comm;
+    const std::vector<int>& dest0 = _index_maps[0]->dest();
+    MPI_Dist_graph_create_adjacent(
+        _index_maps[0]->comm(), dest0.size(), dest0.data(), MPI_UNWEIGHTED,
+        src0.size(), src0.data(), MPI_UNWEIGHTED, MPI_INFO_NULL, false, &comm);
+
+    std::vector<int> recv_sizes(dest0.size());
+    send_sizes.reserve(1);
+    recv_sizes.reserve(1);
+    MPI_Neighbor_alltoall(send_sizes.data(), 1, MPI_INT, recv_sizes.data(), 1,
+                          MPI_INT, comm);
+
+    // Build recv displacements
+    std::vector<int> recv_disp = {0};
+    std::partial_sum(recv_sizes.begin(), recv_sizes.end(),
+                     std::back_inserter(recv_disp));
+
+    ghost_data_in.resize(recv_disp.back());
+    MPI_Neighbor_alltoallv(ghost_data.data(), send_sizes.data(),
+                           send_disp.data(), MPI_INT64_T, ghost_data_in.data(),
+                           recv_sizes.data(), recv_disp.data(), MPI_INT64_T,
+                           comm);
+    MPI_Comm_free(&comm);
+  }
+
+  // Global to local map for ghost column indices
+  std::map<std::int64_t, std::int32_t> global_to_local;
+  std::int32_t local_i = local_size1;
+  for (std::int64_t global_i : _col_ghosts)
+    global_to_local.insert({global_i, local_i++});
 
   // Add data received from the neighborhood
-  const std::vector<std::int64_t>& in_ghost_data = ghost_data_in.array();
-  for (std::size_t i = 0; i < in_ghost_data.size(); i += 3)
+  for (std::size_t i = 0; i < ghost_data_in.size(); i += 3)
   {
-    const std::int32_t row_local = in_ghost_data[i] - local_range0[0];
-    const std::int64_t col = in_ghost_data[i + 1];
-    const int owner = in_ghost_data[i + 2];
+    const std::int32_t row_local = ghost_data_in[i] - local_range0[0];
+    const std::int64_t col = ghost_data_in[i + 1];
+    const int owner = ghost_data_in[i + 2];
     if (col >= local_range1[0] and col < local_range1[1])
     {
       // Convert to local column index
@@ -366,10 +357,10 @@ void SparsityPattern::assemble()
   }
 
   // Sort and remove duplicate column indices in each row
-  std::vector<std::int32_t> adj_counts(local_size0 + ghost_owners0.size(), 0);
+  std::vector<std::int32_t> adj_counts(local_size0 + owners0.size(), 0);
   std::vector<std::int32_t> adj_data;
-  _off_diagonal_offset.resize(local_size0 + ghost_owners0.size());
-  for (std::size_t i = 0; i < local_size0 + ghost_owners0.size(); ++i)
+  _off_diagonal_offset.resize(local_size0 + owners0.size());
+  for (std::size_t i = 0; i < local_size0 + owners0.size(); ++i)
   {
     std::vector<std::int32_t>& row = _row_cache[i];
     std::sort(row.begin(), row.end());
@@ -386,7 +377,7 @@ void SparsityPattern::assemble()
   std::vector<std::vector<std::int32_t>>().swap(_row_cache);
 
   // Compute offsets for adjacency list
-  std::vector<std::int32_t> adj_offsets(local_size0 + ghost_owners0.size() + 1);
+  std::vector<std::int32_t> adj_offsets(local_size0 + owners0.size() + 1);
   std::partial_sum(adj_counts.begin(), adj_counts.end(),
                    adj_offsets.begin() + 1);
 
@@ -397,7 +388,7 @@ void SparsityPattern::assemble()
   // Column count increased due to received rows from other processes
   LOG(INFO) << "Column ghost size increased from "
             << _index_maps[1]->ghosts().size() << " to " << _col_ghosts.size()
-            << "\n";
+            << std::endl;
 }
 //-----------------------------------------------------------------------------
 std::int64_t SparsityPattern::num_nonzeros() const
@@ -428,7 +419,7 @@ const graph::AdjacencyList<std::int32_t>& SparsityPattern::graph() const
   return *_graph;
 }
 //-----------------------------------------------------------------------------
-xtl::span<const int> SparsityPattern::off_diagonal_offset() const
+std::span<const int> SparsityPattern::off_diagonal_offset() const
 {
   return _off_diagonal_offset;
 }
