@@ -8,9 +8,9 @@
 
 #include <dolfinx/common/MPI.h>
 #include <memory>
+#include <span>
 #include <utility>
 #include <vector>
-#include <xtl/xspan.hpp>
 
 namespace dolfinx::graph
 {
@@ -75,13 +75,13 @@ public:
   SparsityPattern& operator=(SparsityPattern&& pattern) = default;
 
   /// Insert non-zero locations using local (process-wise) indices
-  void insert(const xtl::span<const std::int32_t>& rows,
-              const xtl::span<const std::int32_t>& cols);
+  void insert(const std::span<const std::int32_t>& rows,
+              const std::span<const std::int32_t>& cols);
 
   /// Insert non-zero locations on the diagonal
   /// @param[in] rows The rows in local (process-wise) indices. The
   /// indices must exist in the row IndexMap.
-  void insert_diagonal(const xtl::span<const std::int32_t>& rows);
+  void insert_diagonal(const std::span<const std::int32_t>& rows);
 
   /// Finalize sparsity pattern and communicate off-process entries
   void assemble();
@@ -133,7 +133,7 @@ public:
 
   /// Row-wise start of off-diagonal (unowned columns) on each row
   /// @note Includes ghost rows
-  xtl::span<const int> off_diagonal_offset() const;
+  std::span<const int> off_diagonal_offset() const;
 
   /// Return MPI communicator
   MPI_Comm comm() const;
