@@ -12,9 +12,9 @@
 #include <dolfinx/common/log.h>
 #include <dolfinx/common/sort.h>
 #include <dolfinx/graph/AdjacencyList.h>
+#include <span>
 #include <utility>
 #include <vector>
-#include <xtl/xspan.hpp>
 
 using namespace dolfinx;
 
@@ -82,8 +82,8 @@ constexpr mesh::CellType get_cell_type(int num_vertices, int tdim)
 /// @return (0) Extended dual graph to include ghost edges (edges to
 /// off-procss cells) and (1) the number of ghost edges
 graph::AdjacencyList<std::int64_t> compute_nonlocal_dual_graph(
-    const MPI_Comm comm, const xtl::span<const std::int64_t>& facets,
-    std::size_t shape1, const xtl::span<const std::int32_t>& cells,
+    const MPI_Comm comm, const std::span<const std::int64_t>& facets,
+    std::size_t shape1, const std::span<const std::int32_t>& cells,
     const graph::AdjacencyList<std::int32_t>& local_graph)
 {
   LOG(INFO) << "Build nonlocal part of mesh dual graph";
@@ -364,7 +364,7 @@ graph::AdjacencyList<std::int64_t> compute_nonlocal_dual_graph(
     {
       auto e = local_graph.links(i);
       disp[i] += e.size();
-      std::transform(e.cbegin(), e.cend(), std::next(data.begin(), offsets[i]),
+      std::transform(e.begin(), e.end(), std::next(data.begin(), offsets[i]),
                      [cell_offset](auto x) { return x + cell_offset; });
     }
 
@@ -389,8 +389,8 @@ graph::AdjacencyList<std::int64_t> compute_nonlocal_dual_graph(
 //-----------------------------------------------------------------------------
 std::tuple<graph::AdjacencyList<std::int32_t>, std::vector<std::int64_t>,
            std::size_t, std::vector<std::int32_t>>
-mesh::build_local_dual_graph(const xtl::span<const std::int64_t>& cell_vertices,
-                             const xtl::span<const std::int32_t>& cell_offsets,
+mesh::build_local_dual_graph(const std::span<const std::int64_t>& cell_vertices,
+                             const std::span<const std::int32_t>& cell_offsets,
                              int tdim)
 {
   LOG(INFO) << "Build local part of mesh dual graph";
@@ -465,7 +465,7 @@ mesh::build_local_dual_graph(const xtl::span<const std::int64_t>& cell_vertices,
     auto it = perm.begin();
     while (it != perm.end())
     {
-      auto f0 = xtl::span(facets.data() + (*it) * shape1, shape1);
+      auto f0 = std::span(facets.data() + (*it) * shape1, shape1);
 
       // Find iterator to next facet different from f0
       auto it1 = std::find_if_not(
@@ -481,7 +481,7 @@ mesh::build_local_dual_graph(const xtl::span<const std::int64_t>& cell_vertices,
       std::int32_t cell0 = f0.back();
       for (auto itx = std::next(it); itx != it1; ++itx)
       {
-        auto f1 = xtl::span(facets.data() + *itx * shape1, shape1);
+        auto f1 = std::span(facets.data() + *itx * shape1, shape1);
         std::int32_t cell1 = f1.back();
         edges.push_back({cell0, cell1});
       }
