@@ -6,9 +6,10 @@
 
 #include <cstdint>
 #include <dolfinx/graph/AdjacencyList.h>
+#include <span>
+#include <tuple>
 #include <utility>
 #include <vector>
-#include <xtl/xspan.hpp>
 
 #pragma once
 
@@ -67,7 +68,7 @@ refine(const mesh::Mesh& mesh, bool redistribute, RefinementOptions options);
 /// returned.
 /// @return New Mesh and optional parent cell index, parent facet indices
 std::tuple<mesh::Mesh, std::vector<std::int32_t>, std::vector<std::int8_t>>
-refine(const mesh::Mesh& mesh, const xtl::span<const std::int32_t>& edges,
+refine(const mesh::Mesh& mesh, const std::span<const std::int32_t>& edges,
        bool redistribute, RefinementOptions options);
 
 /// Refine mesh returning new mesh data.
@@ -76,10 +77,12 @@ refine(const mesh::Mesh& mesh, const xtl::span<const std::int32_t>& edges,
 /// @param[in] options RefinementOptions enum to choose the computation of
 /// parent facets, parent cells. If an option is unselected, an empty list is
 /// returned.
-/// @return New mesh data: cell topology, vertex coordinates, and optional
-/// parent cell index, and parent facet indices.
-std::tuple<graph::AdjacencyList<std::int64_t>, xt::xtensor<double, 2>,
-           std::vector<std::int32_t>, std::vector<std::int8_t>>
+/// @return New mesh data: cell topology, vertex coordinates, vertex
+/// coordinates shape,  and optional parent cell index, and parent facet
+/// indices.
+std::tuple<graph::AdjacencyList<std::int64_t>, std::vector<double>,
+           std::array<std::size_t, 2>, std::vector<std::int32_t>,
+           std::vector<std::int8_t>>
 compute_refinement_data(const mesh::Mesh& mesh, RefinementOptions options);
 
 /// Refine with markers returning new mesh data.
@@ -92,10 +95,11 @@ compute_refinement_data(const mesh::Mesh& mesh, RefinementOptions options);
 /// returned.
 /// @return New mesh data: cell topology, vertex coordinates and parent
 /// cell index, and stored parent facet indices (if requested).
-std::tuple<graph::AdjacencyList<std::int64_t>, xt::xtensor<double, 2>,
-           std::vector<std::int32_t>, std::vector<std::int8_t>>
+std::tuple<graph::AdjacencyList<std::int64_t>, std::vector<double>,
+           std::array<std::size_t, 2>, std::vector<std::int32_t>,
+           std::vector<std::int8_t>>
 compute_refinement_data(const mesh::Mesh& mesh,
-                        const xtl::span<const std::int32_t>& edges,
+                        const std::span<const std::int32_t>& edges,
                         RefinementOptions options);
 
 } // namespace plaza
