@@ -5,7 +5,9 @@
 // SPDX-License-Identifier:    LGPL-3.0-or-later
 
 #include "utils.h"
+#include <dolfinx/common/IndexMap.h>
 #include <dolfinx/common/MPI.h>
+#include <dolfinx/common/sort.h>
 #include <dolfinx/fem/ElementDofLayout.h>
 #include <dolfinx/mesh/Geometry.h>
 #include <dolfinx/mesh/Mesh.h>
@@ -18,10 +20,8 @@
 #include <memory>
 #include <mpi.h>
 #include <vector>
-#include <xtensor/xview.hpp>
 
 using namespace dolfinx;
-using namespace xt::placeholders;
 
 namespace
 {
@@ -52,7 +52,7 @@ std::int64_t local_to_global(std::int32_t local_index,
 /// @param Mesh
 /// @param local_edge_to_new_vertex
 /// @return array of points
-xt::xtensor<double, 2> create_new_geometry(
+std::pair<std::vector<double>, std::array<std::size_t, 2>> create_new_geometry(
     const mesh::Mesh& mesh,
     const std::map<std::int32_t, std::int64_t>& local_edge_to_new_vertex)
 {
@@ -82,18 +82,18 @@ xt::xtensor<double, 2> create_new_geometry(
   }
 
   // Copy over existing mesh vertices
-  xtl::span<const double> x_g = mesh.geometry().x();
-
+  std::span<const double> x_g = mesh.geometry().x();
+  const std::size_t gdim = mesh.geometry().dim();
   const std::size_t num_vertices = map_v->size_local();
   const std::size_t num_new_vertices = local_edge_to_new_vertex.size();
-  xt::xtensor<double, 2> new_vertex_coordinates(
-      {num_vertices + num_new_vertices, 3});
 
+  std::array<std::size_t, 2> shape = {num_vertices + num_new_vertices, gdim};
+  std::vector<double> new_vertex_coords(shape[0] * shape[1]);
   for (std::size_t v = 0; v < num_vertices; ++v)
   {
-    const int pos = 3 * vertex_to_x[v];
-    for (std::size_t j = 0; j < 3; ++j)
-      new_vertex_coordinates(v, j) = x_g[pos + j];
+    std::size_t pos = 3 * vertex_to_x[v];
+    for (std::size_t j = 0; j < gdim; ++j)
+      new_vertex_coords[gdim * v + j] = x_g[pos + j];
   }
 
   // Compute new vertices
@@ -104,98 +104,68 @@ xt::xtensor<double, 2> create_new_geometry(
     for (auto& e : local_edge_to_new_vertex)
       edges[i++] = e.first;
 
-    const xt::xtensor<double, 2> midpoints
+    // Compute midpoint of each edge (padded to 3D)
+    const std::vector<double> midpoints
         = mesh::compute_midpoints(mesh, 1, edges);
-
-    // The below should work, but misbehaves with the Intel icpx compiler
-    // xt::view(new_vertex_coordinates, xt::range(-num_new_vertices, _),
-    // xt::all())
-    //     = midpoints;
-    auto _vertex = xt::view(new_vertex_coordinates,
-                            xt::range(-num_new_vertices, _), xt::all());
-    _vertex.assign(midpoints);
+    for (std::size_t i = 0; i < num_new_vertices; ++i)
+      for (std::size_t j = 0; j < gdim; ++j)
+        new_vertex_coords[gdim * (num_vertices + i) + j] = midpoints[3 * i + j];
   }
 
-  return xt::view(new_vertex_coordinates, xt::all(),
-                  xt::range(0, mesh.geometry().dim()));
+  return {std::move(new_vertex_coords), shape};
 }
 } // namespace
 
 //---------------------------------------------------------------------------------
-std::pair<MPI_Comm, std::map<std::int32_t, std::vector<int>>>
-refinement::compute_edge_sharing(const mesh::Mesh& mesh)
-{
-  if (!mesh.topology().connectivity(1, 0))
-    throw std::runtime_error("Edges must be initialised");
-
-  auto map_e = mesh.topology().index_map(1);
-  assert(map_e);
-
-  // Create shared edges, for both owned and ghost indices
-  // returning edge -> set(global process numbers)
-  std::map<std::int32_t, std::set<int>> shared_edges_by_proc
-      = map_e->compute_shared_indices();
-
-  // Compute a slightly wider neighborhood for direct communication of shared
-  // edges
-  std::set<int> all_neighbor_set;
-  for (const auto& q : shared_edges_by_proc)
-    all_neighbor_set.insert(q.second.begin(), q.second.end());
-  std::vector<int> neighbors(all_neighbor_set.begin(), all_neighbor_set.end());
-
-  MPI_Comm neighbor_comm;
-  MPI_Dist_graph_create_adjacent(
-      mesh.comm(), neighbors.size(), neighbors.data(), MPI_UNWEIGHTED,
-      neighbors.size(), neighbors.data(), MPI_UNWEIGHTED, MPI_INFO_NULL, false,
-      &neighbor_comm);
-
-  // Create a "shared_edge to neighbor map"
-  std::map<int, int> proc_to_neighbor;
-  for (std::size_t i = 0; i < neighbors.size(); ++i)
-    proc_to_neighbor.insert({neighbors[i], i});
-
-  std::map<std::int32_t, std::vector<int>> shared_edges;
-  for (auto& q : shared_edges_by_proc)
-  {
-    std::vector<int> neighbor_set;
-    for (int r : q.second)
-      neighbor_set.push_back(proc_to_neighbor[r]);
-    std::sort(neighbor_set.begin(), neighbor_set.end());
-    neighbor_set.erase(std::unique(neighbor_set.begin(), neighbor_set.end()),
-                       neighbor_set.end());
-    shared_edges.insert({q.first, neighbor_set});
-  }
-
-  return {neighbor_comm, std::move(shared_edges)};
-}
-//-----------------------------------------------------------------------------
 void refinement::update_logical_edgefunction(
     MPI_Comm neighbor_comm,
     const std::vector<std::vector<std::int32_t>>& marked_for_update,
-    std::vector<std::int8_t>& marked_edges, const common::IndexMap& map_e)
+    std::vector<std::int8_t>& marked_edges, const common::IndexMap& map)
 {
-  std::vector<std::int32_t> send_offsets = {0};
+  std::vector<int> send_sizes;
   std::vector<std::int64_t> data_to_send;
-  int num_neighbors = marked_for_update.size();
-  for (int i = 0; i < num_neighbors; ++i)
+  for (std::size_t i = 0; i < marked_for_update.size(); ++i)
   {
     for (std::int32_t q : marked_for_update[i])
-      data_to_send.push_back(local_to_global(q, map_e));
+      data_to_send.push_back(local_to_global(q, map));
 
-    send_offsets.push_back(data_to_send.size());
+    send_sizes.push_back(marked_for_update[i].size());
   }
 
   // Send all shared edges marked for update and receive from other
   // processes
-  const std::vector<std::int64_t> data_to_recv
-      = dolfinx::MPI::neighbor_all_to_all(
-            neighbor_comm,
-            graph::AdjacencyList<std::int64_t>(data_to_send, send_offsets))
-            .array();
+  std::vector<std::int64_t> data_to_recv;
+  {
+    int indegree(-1), outdegree(-2), weighted(-1);
+    MPI_Dist_graph_neighbors_count(neighbor_comm, &indegree, &outdegree,
+                                   &weighted);
+    assert(indegree == (int)marked_for_update.size());
+    assert(indegree == outdegree);
+
+    std::vector<int> recv_sizes(outdegree);
+    send_sizes.reserve(1);
+    recv_sizes.reserve(1);
+    MPI_Neighbor_alltoall(send_sizes.data(), 1, MPI_INT, recv_sizes.data(), 1,
+                          MPI_INT, neighbor_comm);
+
+    // Build displacements
+    std::vector<int> send_disp = {0};
+    std::partial_sum(send_sizes.begin(), send_sizes.end(),
+                     std::back_inserter(send_disp));
+    std::vector<int> recv_disp = {0};
+    std::partial_sum(recv_sizes.begin(), recv_sizes.end(),
+                     std::back_inserter(recv_disp));
+
+    data_to_recv.resize(recv_disp.back());
+    MPI_Neighbor_alltoallv(data_to_send.data(), send_sizes.data(),
+                           send_disp.data(), MPI_INT64_T, data_to_recv.data(),
+                           recv_sizes.data(), recv_disp.data(), MPI_INT64_T,
+                           neighbor_comm);
+  }
 
   // Flatten received values and set marked_edges at each index received
   std::vector<std::int32_t> local_indices(data_to_recv.size());
-  map_e.global_to_local(data_to_recv, local_indices);
+  map.global_to_local(data_to_recv, local_indices);
   for (std::int32_t local_index : local_indices)
   {
     assert(local_index != -1);
@@ -203,14 +173,15 @@ void refinement::update_logical_edgefunction(
   }
 }
 //-----------------------------------------------------------------------------
-std::pair<std::map<std::int32_t, std::int64_t>, xt::xtensor<double, 2>>
-refinement::create_new_vertices(
-    MPI_Comm neighbor_comm,
-    const std::map<std::int32_t, std::vector<std::int32_t>>& shared_edges,
-    const mesh::Mesh& mesh, const std::vector<std::int8_t>& marked_edges)
+std::tuple<std::map<std::int32_t, std::int64_t>, std::vector<double>,
+           std::array<std::size_t, 2>>
+refinement::create_new_vertices(MPI_Comm neighbor_comm,
+                                const graph::AdjacencyList<int>& shared_edges,
+                                const mesh::Mesh& mesh,
+                                const std::vector<std::int8_t>& marked_edges)
 {
   // Take marked_edges and use to create new vertices
-  const std::shared_ptr<const common::IndexMap> edge_index_map
+  std::shared_ptr<const common::IndexMap> edge_index_map
       = mesh.topology().index_map(1);
 
   // Add new edge midpoints to list of vertices
@@ -235,7 +206,7 @@ refinement::create_new_vertices(
                 [global_offset](auto& e) { e.second += global_offset; });
 
   // Create actual points
-  xt::xtensor<double, 2> new_vertex_coordinates
+  auto [new_vertex_coords, xshape]
       = create_new_geometry(mesh, local_edge_to_new_vertex);
 
   // If they are shared, then the new global vertex index needs to be
@@ -254,23 +225,53 @@ refinement::create_new_vertices(
     const std::size_t local_i = local_edge.first;
     // shared, but locally owned : remote owned are not in list.
 
-    if (auto shared_edge_i = shared_edges.find(local_i);
-        shared_edge_i != shared_edges.end())
+    for (int remote_process : shared_edges.links(local_i))
     {
-      for (int remote_process : shared_edge_i->second)
-      {
-        // send map from global edge index to new global vertex index
-        values_to_send[remote_process].push_back(
-            local_to_global(local_edge.first, *edge_index_map));
-        values_to_send[remote_process].push_back(local_edge.second);
-      }
+      // Send (global edge index) -> (new global vertex index) map
+      values_to_send[remote_process].push_back(
+          local_to_global(local_i, *edge_index_map));
+      values_to_send[remote_process].push_back(local_edge.second);
     }
   }
 
-  const std::vector<std::int64_t> received_values
-      = dolfinx::MPI::neighbor_all_to_all(
-            neighbor_comm, graph::AdjacencyList<std::int64_t>(values_to_send))
-            .array();
+  // Send all shared edges marked for update and receive from other
+  // processes
+  std::vector<std::int64_t> received_values;
+  {
+    int indegree(-1), outdegree(-2), weighted(-1);
+    MPI_Dist_graph_neighbors_count(neighbor_comm, &indegree, &outdegree,
+                                   &weighted);
+    assert(indegree == outdegree);
+
+    std::vector<std::int64_t> send_buffer;
+    std::vector<int> send_sizes;
+    for (auto& x : values_to_send)
+    {
+      send_sizes.push_back(x.size());
+      send_buffer.insert(send_buffer.end(), x.begin(), x.end());
+    }
+    assert((int)send_sizes.size() == outdegree);
+
+    std::vector<int> recv_sizes(outdegree);
+    send_sizes.reserve(1);
+    recv_sizes.reserve(1);
+    MPI_Neighbor_alltoall(send_sizes.data(), 1, MPI_INT, recv_sizes.data(), 1,
+                          MPI_INT, neighbor_comm);
+
+    // Build displacements
+    std::vector<int> send_disp = {0};
+    std::partial_sum(send_sizes.begin(), send_sizes.end(),
+                     std::back_inserter(send_disp));
+    std::vector<int> recv_disp = {0};
+    std::partial_sum(recv_sizes.begin(), recv_sizes.end(),
+                     std::back_inserter(recv_disp));
+
+    received_values.resize(recv_disp.back());
+    MPI_Neighbor_alltoallv(send_buffer.data(), send_sizes.data(),
+                           send_disp.data(), MPI_INT64_T,
+                           received_values.data(), recv_sizes.data(),
+                           recv_disp.data(), MPI_INT64_T, neighbor_comm);
+  }
 
   // Add received remote global vertex indices to map
   std::vector<std::int64_t> recv_global_edge;
@@ -288,21 +289,22 @@ refinement::create_new_vertices(
     assert(it.second);
   }
 
-  return {std::move(local_edge_to_new_vertex),
-          std::move(new_vertex_coordinates)};
+  return {std::move(local_edge_to_new_vertex), std::move(new_vertex_coords),
+          xshape};
 }
 //-----------------------------------------------------------------------------
 mesh::Mesh
 refinement::partition(const mesh::Mesh& old_mesh,
                       const graph::AdjacencyList<std::int64_t>& cell_topology,
-                      const xt::xtensor<double, 2>& new_vertex_coordinates,
-                      bool redistribute, mesh::GhostMode gm)
+                      std::span<const double> new_coords,
+                      std::array<std::size_t, 2> xshape, bool redistribute,
+                      mesh::GhostMode gm)
 {
   if (redistribute)
   {
-    xt::xtensor<double, 2> new_coords(new_vertex_coordinates);
     return mesh::create_mesh(old_mesh.comm(), cell_topology,
-                             old_mesh.geometry().cmap(), new_coords, gm);
+                             old_mesh.geometry().cmap(), new_coords, xshape,
+                             gm);
   }
 
   auto partitioner = [](MPI_Comm comm, int, int tdim,
@@ -364,82 +366,96 @@ refinement::partition(const mesh::Mesh& old_mesh,
   };
 
   return mesh::create_mesh(old_mesh.comm(), cell_topology,
-                           old_mesh.geometry().cmap(), new_vertex_coordinates,
-                           gm, partitioner);
+                           old_mesh.geometry().cmap(), new_coords, xshape, gm,
+                           partitioner);
 }
 //-----------------------------------------------------------------------------
+
 std::vector<std::int64_t>
-refinement::adjust_indices(const common::IndexMap& index_map, std::int32_t n)
+refinement::adjust_indices(const common::IndexMap& map, std::int32_t n)
 {
+  // NOTE: Is this effectively concatenating index maps?
+
   // Add in an extra "n" indices at the end of the current local_range
   // of "index_map", and adjust existing indices to match.
 
-  // Get offset for n for this process
-  MPI_Comm comm = index_map.comm();
+  // Get offset for 'n' for this process
   const std::int64_t num_local = n;
   std::int64_t global_offset = 0;
-  MPI_Exscan(&num_local, &global_offset, 1, MPI_INT64_T, MPI_SUM, comm);
+  MPI_Exscan(&num_local, &global_offset, 1, MPI_INT64_T, MPI_SUM, map.comm());
 
-  // Use MPI neighbors to get offsets for ghosts. Use the source of the
-  // forward comm to get ghost entry neighbors and create a dictionary
-  // to lookup from the global rank
-  MPI_Comm comm_fwd = index_map.comm(common::IndexMap::Direction::forward);
-  int num_neighbors_fwd, num_neighbors_rev, weighted;
-  MPI_Dist_graph_neighbors_count(comm_fwd, &num_neighbors_fwd,
-                                 &num_neighbors_rev, &weighted);
+  const std::vector<int>& owners = map.owners();
+  const std::vector<int>& src = map.src();
+  const std::vector<int>& dest = map.dest();
+
+  MPI_Comm comm;
+  MPI_Dist_graph_create_adjacent(map.comm(), src.size(), src.data(),
+                                 MPI_UNWEIGHTED, dest.size(), dest.data(),
+                                 MPI_UNWEIGHTED, MPI_INFO_NULL, false, &comm);
 
   // Communicate offset to neighbors
-  std::vector<std::int64_t> neighbor_offsets(num_neighbors_fwd, 0);
-  // Ensure allocation, in case where num_neighbors_fwd == 0, needed for some
-  // MPI
-  neighbor_offsets.reserve(1);
-  MPI_Neighbor_allgather(&global_offset, 1, MPI_INT64_T,
-                         neighbor_offsets.data(), 1, MPI_INT64_T, comm_fwd);
+  std::vector<std::int64_t> offsets(src.size(), 0);
+  offsets.reserve(1);
+  MPI_Neighbor_allgather(&global_offset, 1, MPI_INT64_T, offsets.data(), 1,
+                         MPI_INT64_T, comm);
 
-  const std::vector<int>& ghost_owners = index_map.ghost_owners();
-  int local_size = index_map.size_local();
-  std::vector<std::int64_t> global_indices = index_map.global_indices();
+  MPI_Comm_free(&comm);
+
+  int local_size = map.size_local();
+  std::vector<std::int64_t> global_indices = map.global_indices();
+
+  // Add new offset to owned indices
   std::transform(global_indices.begin(),
                  std::next(global_indices.begin(), local_size),
                  global_indices.begin(),
                  [global_offset](auto x) { return x + global_offset; });
 
-  for (std::size_t i = 0; i < ghost_owners.size(); ++i)
-    global_indices[local_size + i] += neighbor_offsets[ghost_owners[i]];
+  // Add offsets to ghost indices
+  std::transform(std::next(global_indices.begin(), local_size),
+                 global_indices.end(), owners.begin(),
+                 std::next(global_indices.begin(), local_size),
+                 [&src, &offsets](auto idx, auto r)
+                 {
+                   auto it = std::lower_bound(src.begin(), src.end(), r);
+                   assert(it != src.end() and *it == r);
+                   int rank = std::distance(src.begin(), it);
+                   return idx + offsets[rank];
+                 });
 
   return global_indices;
 }
 //-----------------------------------------------------------------------------
 mesh::MeshTags<std::int32_t> refinement::transfer_facet_meshtag(
-    const mesh::MeshTags<std::int32_t>& parent_meshtag,
-    const mesh::Mesh& refined_mesh,
-    const std::vector<std::int32_t>& parent_cell,
-    const std::vector<std::int8_t>& parent_facet)
+    const mesh::MeshTags<std::int32_t>& meshtag,
+    std::shared_ptr<const mesh::Mesh> refined_mesh,
+    const std::vector<std::int32_t>& cell,
+    const std::vector<std::int8_t>& facet)
 {
-  const int tdim = parent_meshtag.mesh()->topology().dim();
-  if (parent_meshtag.dim() != tdim - 1)
-    throw std::runtime_error("Input meshtag is not facet-based");
+  const int tdim = meshtag.mesh()->topology().dim();
 
-  if (parent_meshtag.mesh()->topology().index_map(tdim)->num_ghosts() > 0)
+  if (meshtag.dim() != tdim - 1)
+    throw std::runtime_error("Input meshtag is not facet-based");
+  if (meshtag.mesh()->topology().index_map(tdim)->num_ghosts() > 0)
     throw std::runtime_error("Ghosted meshes are not supported");
 
-  auto parent_c_to_f
-      = parent_meshtag.mesh()->topology().connectivity(tdim, tdim - 1);
-  auto c_to_f = refined_mesh.topology().connectivity(tdim, tdim - 1);
+  auto c_to_f = meshtag.mesh()->topology().connectivity(tdim, tdim - 1);
+  if (!c_to_f)
+    throw std::runtime_error("Parent mesh is missing cell-facet connectivity.");
 
   // Create map parent->child facets
   const std::int32_t num_input_facets
-      = parent_meshtag.mesh()->topology().index_map(tdim - 1)->size_local()
-        + parent_meshtag.mesh()->topology().index_map(tdim - 1)->num_ghosts();
-  std::vector<int> count_child(num_input_facets, 0);
+      = meshtag.mesh()->topology().index_map(tdim - 1)->size_local()
+        + meshtag.mesh()->topology().index_map(tdim - 1)->num_ghosts();
 
   // Get global index for each refined cell, before reordering in Mesh
   // construction
+  assert(refined_mesh);
   const std::vector<std::int64_t>& original_cell_index
-      = refined_mesh.topology().original_cell_index;
-  assert(original_cell_index.size() == parent_cell.size());
+      = refined_mesh->topology().original_cell_index;
+  assert(original_cell_index.size() == cell.size());
   std::int64_t global_offset
-      = refined_mesh.topology().index_map(tdim)->local_range()[0];
+      = refined_mesh->topology().index_map(tdim)->local_range()[0];
+
   // Map cells back to original index
   std::vector<std::int32_t> local_cell_index(original_cell_index.size());
   for (std::size_t i = 0; i < local_cell_index.size(); ++i)
@@ -451,78 +467,76 @@ mesh::MeshTags<std::int32_t> refinement::transfer_facet_meshtag(
   }
 
   // Count number of child facets for each parent facet
-  for (std::size_t c = 0; c < parent_cell.size(); ++c)
+  std::vector<int> count_child(num_input_facets, 0);
+  for (std::size_t c = 0; c < cell.size(); ++c)
   {
-    auto parent_cf = parent_c_to_f->links(parent_cell[c]);
-
-    for (int j = 0; j < (tdim + 1); ++j)
+    auto facets = c_to_f->links(cell[c]);
+    for (int j = 0; j <= tdim; ++j)
     {
-      std::int8_t fidx = parent_facet[c * (tdim + 1) + j];
-      if (fidx != -1)
-        ++count_child[parent_cf[fidx]];
+      if (std::int8_t fidx = facet[c * (tdim + 1) + j]; fidx != -1)
+        ++count_child[facets[fidx]];
     }
   }
 
+  auto c_to_f_refined = refined_mesh->topology().connectivity(tdim, tdim - 1);
+  if (!c_to_f_refined)
+  {
+    throw std::runtime_error(
+        "Refined mesh is missing cell-facet connectivity.");
+  }
+
+  // Fill in data for each child facet
   std::vector<int> offset_child(num_input_facets + 1, 0);
   std::partial_sum(count_child.begin(), count_child.end(),
                    std::next(offset_child.begin()));
   std::vector<std::int32_t> child_facet(offset_child.back());
-
-  // Fill in data for each child facet
-  for (std::size_t c = 0; c < parent_cell.size(); ++c)
+  for (std::size_t c = 0; c < cell.size(); ++c)
   {
-    std::int32_t pc = parent_cell[c];
-    auto parent_cf = parent_c_to_f->links(pc);
+    auto facets = c_to_f->links(cell[c]);
 
     // Use original indexing for child cell
-    const std::int32_t lc = local_cell_index[c];
-    auto refined_cf = c_to_f->links(lc);
+    auto refined_facets = c_to_f_refined->links(local_cell_index[c]);
 
     // Get child facets for each cell
-    for (int j = 0; j < (tdim + 1); ++j)
+    for (int j = 0; j <= tdim; ++j)
     {
-      std::int8_t fidx = parent_facet[c * (tdim + 1) + j];
-      if (fidx != -1)
+      if (std::int8_t fidx = facet[c * (tdim + 1) + j]; fidx != -1)
       {
-        int offset = offset_child[parent_cf[fidx]];
-        child_facet[offset] = refined_cf[j];
-        ++offset_child[parent_cf[fidx]];
+        int offset = offset_child[facets[fidx]];
+        child_facet[offset] = refined_facets[j];
+        ++offset_child[facets[fidx]];
       }
     }
   }
 
   // Rebuild offset
-  offset_child[0] = 0;
+  offset_child.front() = 0;
   std::partial_sum(count_child.begin(), count_child.end(),
                    std::next(offset_child.begin()));
   graph::AdjacencyList<std::int32_t> p_to_c_facet(std::move(child_facet),
                                                   std::move(offset_child));
 
   // Copy facet meshtag from parent to child
-  std::vector<std::int32_t> facet_indices;
-  std::vector<std::int32_t> tag_values;
-  const std::vector<std::int32_t>& in_index = parent_meshtag.indices();
-  const std::vector<std::int32_t>& in_value = parent_meshtag.values();
+  std::vector<std::int32_t> facet_indices, tag_values;
+  const std::vector<std::int32_t>& in_index = meshtag.indices();
+  const std::vector<std::int32_t>& in_value = meshtag.values();
   for (std::size_t i = 0; i < in_index.size(); ++i)
   {
     std::int32_t parent_index = in_index[i];
     auto pclinks = p_to_c_facet.links(parent_index);
-    // eliminate duplicates
+
+    // Eliminate duplicates
     std::sort(pclinks.begin(), pclinks.end());
     auto it_end = std::unique(pclinks.begin(), pclinks.end());
-    for (auto child_it = pclinks.begin(); child_it != it_end; ++child_it)
-    {
-      facet_indices.push_back(*child_it);
-      tag_values.push_back(in_value[i]);
-    }
+    facet_indices.insert(facet_indices.end(), pclinks.begin(), it_end);
+    tag_values.insert(tag_values.end(), std::distance(pclinks.begin(), it_end),
+                      in_value[i]);
   }
 
   // Sort values into order, based on facet indices
   std::vector<std::int32_t> sort_order(tag_values.size());
   std::iota(sort_order.begin(), sort_order.end(), 0);
-  std::sort(sort_order.begin(), sort_order.end(),
-            [&facet_indices](auto a, auto b)
-            { return facet_indices[a] < facet_indices[b]; });
+  dolfinx::argsort_radix<std::int32_t>(facet_indices, sort_order);
   std::vector<std::int32_t> sorted_facet_indices(facet_indices.size());
   std::vector<std::int32_t> sorted_tag_values(tag_values.size());
   for (std::size_t i = 0; i < sort_order.size(); ++i)
@@ -531,36 +545,38 @@ mesh::MeshTags<std::int32_t> refinement::transfer_facet_meshtag(
     sorted_facet_indices[i] = facet_indices[sort_order[i]];
   }
 
-  return mesh::MeshTags<std::int32_t>(
-      std::make_shared<mesh::Mesh>(refined_mesh), tdim - 1,
-      std::move(sorted_facet_indices), std::move(sorted_tag_values));
+  return mesh::MeshTags<std::int32_t>(refined_mesh, tdim - 1,
+                                      std::move(sorted_facet_indices),
+                                      std::move(sorted_tag_values));
 }
 //----------------------------------------------------------------------------
 mesh::MeshTags<std::int32_t> refinement::transfer_cell_meshtag(
-    const mesh::MeshTags<std::int32_t>& parent_meshtag,
-    const mesh::Mesh& refined_mesh,
-    const std::vector<std::int32_t>& parent_cell)
+    const mesh::MeshTags<std::int32_t>& meshtag,
+    std::shared_ptr<const mesh::Mesh> refined_mesh,
+    const std::vector<std::int32_t>& cell)
 {
-  const int tdim = parent_meshtag.mesh()->topology().dim();
-  if (parent_meshtag.dim() != tdim)
+  const int tdim = meshtag.mesh()->topology().dim();
+  if (meshtag.dim() != tdim)
     throw std::runtime_error("Input meshtag is not cell-based");
 
-  if (parent_meshtag.mesh()->topology().index_map(tdim)->num_ghosts() > 0)
+  if (meshtag.mesh()->topology().index_map(tdim)->num_ghosts() > 0)
     throw std::runtime_error("Ghosted meshes are not supported");
 
   // Create map parent->child facets
   const std::int32_t num_input_cells
-      = parent_meshtag.mesh()->topology().index_map(tdim)->size_local()
-        + parent_meshtag.mesh()->topology().index_map(tdim)->num_ghosts();
+      = meshtag.mesh()->topology().index_map(tdim)->size_local()
+        + meshtag.mesh()->topology().index_map(tdim)->num_ghosts();
   std::vector<int> count_child(num_input_cells, 0);
 
   // Get global index for each refined cell, before reordering in Mesh
   // construction
+  assert(refined_mesh);
   const std::vector<std::int64_t>& original_cell_index
-      = refined_mesh.topology().original_cell_index;
-  assert(original_cell_index.size() == parent_cell.size());
+      = refined_mesh->topology().original_cell_index;
+  assert(original_cell_index.size() == cell.size());
   std::int64_t global_offset
-      = refined_mesh.topology().index_map(tdim)->local_range()[0];
+      = refined_mesh->topology().index_map(tdim)->local_range()[0];
+
   // Map back to original index
   std::vector<std::int32_t> local_cell_index(original_cell_index.size());
   for (std::size_t i = 0; i < local_cell_index.size(); ++i)
@@ -572,8 +588,8 @@ mesh::MeshTags<std::int32_t> refinement::transfer_cell_meshtag(
   }
 
   // Count number of child cells for each parent cell
-  for (std::int32_t pcell : parent_cell)
-    ++count_child[pcell];
+  for (std::int32_t c : cell)
+    ++count_child[c];
 
   std::vector<int> offset_child(num_input_cells + 1, 0);
   std::partial_sum(count_child.begin(), count_child.end(),
@@ -581,10 +597,11 @@ mesh::MeshTags<std::int32_t> refinement::transfer_cell_meshtag(
   std::vector<std::int32_t> child_cell(offset_child.back());
 
   // Fill in data for each child cell
-  for (std::size_t c = 0; c < parent_cell.size(); ++c)
+  for (std::size_t c = 0; c < cell.size(); ++c)
   {
-    std::int32_t pc = parent_cell[c];
+    std::int32_t pc = cell[c];
     int offset = offset_child[pc];
+
     // Use original indexing for child cell
     const std::int32_t lc = local_cell_index[c];
     child_cell[offset] = lc;
@@ -592,7 +609,7 @@ mesh::MeshTags<std::int32_t> refinement::transfer_cell_meshtag(
   }
 
   // Rebuild offset
-  offset_child[0] = 0;
+  offset_child.front() = 0;
   std::partial_sum(count_child.begin(), count_child.end(),
                    std::next(offset_child.begin()));
   graph::AdjacencyList<std::int32_t> p_to_c_cell(std::move(child_cell),
@@ -601,25 +618,19 @@ mesh::MeshTags<std::int32_t> refinement::transfer_cell_meshtag(
   // Copy cell meshtag from parent to child
   std::vector<std::int32_t> cell_indices;
   std::vector<std::int32_t> tag_values;
-  const std::vector<std::int32_t>& in_index = parent_meshtag.indices();
-  const std::vector<std::int32_t>& in_value = parent_meshtag.values();
+  const std::vector<std::int32_t>& in_index = meshtag.indices();
+  const std::vector<std::int32_t>& in_value = meshtag.values();
   for (std::size_t i = 0; i < in_index.size(); ++i)
   {
-    std::int32_t parent_index = in_index[i];
-    auto pclinks = p_to_c_cell.links(parent_index);
-    for (std::int32_t child : pclinks)
-    {
-      cell_indices.push_back(child);
-      tag_values.push_back(in_value[i]);
-    }
+    auto pclinks = p_to_c_cell.links(in_index[i]);
+    cell_indices.insert(cell_indices.end(), pclinks.begin(), pclinks.end());
+    tag_values.insert(tag_values.end(), pclinks.size(), in_value[i]);
   }
 
   // Sort values into order, based on cell indices
   std::vector<std::int32_t> sort_order(tag_values.size());
   std::iota(sort_order.begin(), sort_order.end(), 0);
-  std::sort(sort_order.begin(), sort_order.end(),
-            [&cell_indices](auto a, auto b)
-            { return cell_indices[a] < cell_indices[b]; });
+  dolfinx::argsort_radix<std::int32_t>(cell_indices, sort_order);
   std::vector<std::int32_t> sorted_tag_values(tag_values.size());
   std::vector<std::int32_t> sorted_cell_indices(cell_indices.size());
   for (std::size_t i = 0; i < sort_order.size(); ++i)
@@ -628,7 +639,8 @@ mesh::MeshTags<std::int32_t> refinement::transfer_cell_meshtag(
     sorted_cell_indices[i] = cell_indices[sort_order[i]];
   }
 
-  return mesh::MeshTags<std::int32_t>(
-      std::make_shared<mesh::Mesh>(refined_mesh), tdim,
-      std::move(sorted_cell_indices), std::move(sorted_tag_values));
+  return mesh::MeshTags<std::int32_t>(refined_mesh, tdim,
+                                      std::move(sorted_cell_indices),
+                                      std::move(sorted_tag_values));
 }
+//-----------------------------------------------------------------------------
