@@ -6,13 +6,12 @@
 
 #ifdef HAS_ADIOS2
 
+#include <algorithm>
 #include <catch2/catch.hpp>
 #include <dolfinx/io/ADIOS2Writers.h>
 #include <dolfinx/mesh/Mesh.h>
 #include <dolfinx/mesh/generation.h>
 #include <mpi.h>
-#include <xtensor/xadapt.hpp>
-#include <xtensor/xview.hpp>
 
 using namespace dolfinx;
 
@@ -26,17 +25,17 @@ void test_fides_mesh()
       mesh::CellType::triangle, mesh::GhostMode::shared_facet));
   io::FidesWriter writer(mesh->comm(), "test_mesh.bp", mesh);
   writer.write(0.0);
-  auto points
-      = xt::adapt(mesh->geometry().x().data(), mesh->geometry().x().size(),
-                  xt::no_ownership(),
-                  std::vector{mesh->geometry().x().size() / 3, std::size_t(3)});
+
+  auto x = mesh->geometry().x();
+
   // Move all coordinates of the mesh geometry
-  points += 1;
+  std::transform(x.begin(), x.end(), x.begin(), [](auto x) { return x + 1; });
   writer.write(0.2);
 
   // Only move x coordinate
-  auto x_coords = xt::view(points, xt::all(), 0);
-  x_coords -= 0.5;
+  for (std::size_t i = 0; i < x.size(); i += 3)
+    x[i] -= 0.5;
+
   writer.write(0.4);
 }
 
