@@ -1,4 +1,4 @@
-# Copyright (C) 2009-2019 Chris N. Richardson, Garth N. Wells and Michal Habera
+# Copyright (C) 2009-2022 Chris N. Richardson, Garth N. Wells and Michal Habera
 #
 # This file is part of DOLFINx (https://www.fenicsproject.org)
 #
@@ -14,8 +14,8 @@ if typing.TYPE_CHECKING:
 
 from functools import singledispatch
 
-import cffi
 import numpy as np
+import numpy.typing as npt
 
 import ufl
 import ufl.algorithms
@@ -36,18 +36,22 @@ class Constant(ufl.Constant):
             c: Value of the constant.
 
         """
-        c_np = np.asarray(c)
-        super().__init__(domain, c_np.shape)
-        if c_np.dtype == np.complex64:
-            self._cpp_object = _cpp.fem.Constant_complex64(c_np)
-        elif c_np.dtype == np.complex128:
-            self._cpp_object = _cpp.fem.Constant_complex128(c_np)
-        elif c_np.dtype == np.float32:
-            self._cpp_object = _cpp.fem.Constant_float32(c_np)
-        elif c_np.dtype == np.float64:
-            self._cpp_object = _cpp.fem.Constant_float64(c_np)
-        else:
-            raise RuntimeError("Unsupported dtype")
+        c = np.asarray(c)
+        super().__init__(domain, c.shape)
+
+        try:
+            if c.dtype == np.complex64:
+                self._cpp_object = _cpp.fem.Constant_complex64(c)
+            elif c.dtype == np.complex128:
+                self._cpp_object = _cpp.fem.Constant_complex128(c)
+            elif c.dtype == np.float32:
+                self._cpp_object = _cpp.fem.Constant_float32(c)
+            elif c.dtype == np.float64:
+                self._cpp_object = _cpp.fem.Constant_float64(c)
+            else:
+                raise RuntimeError("Unsupported dtype")
+        except AttributeError:
+            raise AttributeError("Constant value must have a dtype attribute.")
 
     @property
     def value(self):
@@ -110,14 +114,11 @@ class Expression:
         else:
             raise RuntimeError(f"Unsupported scalar type {dtype} for Expression.")
 
-        self._ufcx_expression, _, self._code = jit.ffcx_jit(mesh.comm, (ufl_expression, _X),
-                                                            form_compiler_params=form_compiler_params,
-                                                            jit_params=jit_params)
+        self._ufcx_expression, module, self._code = jit.ffcx_jit(mesh.comm, (ufl_expression, _X),
+                                                                 form_compiler_params=form_compiler_params,
+                                                                 jit_params=jit_params)
         self._ufl_expression = ufl_expression
 
-        # Tabulation function.
-        ffi = cffi.FFI()
-
         # Prepare coefficients data. For every coefficient in form take
         # its C++ object.
         original_coefficients = ufl.algorithms.extract_coefficients(ufl_expression)
@@ -144,6 +145,7 @@ class Expression:
             else:
                 raise NotImplementedError(f"Type {dtype} not supported.")
 
+        ffi = module.ffi
         self._cpp_object = create_expression(dtype)(ffi.cast("uintptr_t", ffi.addressof(self._ufcx_expression)),
                                                     coeffs, constants, mesh, self.argument_function_space)
 
@@ -158,7 +160,7 @@ class Expression:
             argument_space_dimension = 1
         else:
             argument_space_dimension = self.argument_function_space.element.space_dimension
-        values_shape = (_cells.shape[0], self.X.shape[0] * self.value_size * argument_space_dimension)
+        values_shape = (_cells.shape[0], self.X().shape[0] * self.value_size * argument_space_dimension)
 
         # Allocate memory for result if u was not provided
         if values is None:
@@ -173,16 +175,15 @@ class Expression:
 
         return values
 
+    def X(self) -> np.ndarray:
+        """Evaluation points on the reference cell"""
+        return self._cpp_object.X()
+
     @property
     def ufl_expression(self):
         """Original UFL Expression"""
         return self._ufl_expression
 
-    @property
-    def X(self) -> np.ndarray:
-        """Evaluation points on the reference cell"""
-        return self._cpp_object.X
-
     @property
     def value_size(self) -> int:
         """Value size of the expression"""
@@ -265,29 +266,29 @@ class Function(ufl.Coefficient):
         """The FunctionSpace that the Function is defined on"""
         return self._V
 
-    def eval(self, x: np.ndarray, cells: np.ndarray, u=None) -> np.ndarray:
+    def eval(self, x: npt.ArrayLike, cells: npt.ArrayLike, u=None) -> np.ndarray:
         """Evaluate Function at points x, where x has shape (num_points, 3),
         and cells has shape (num_points,) and cell[i] is the index of the
         cell containing point x[i]. If the cell index is negative the
         point is ignored."""
 
         # Make sure input coordinates are a NumPy array
-        x = np.asarray(x, dtype=np.float64)
-        assert x.ndim < 3
-        if len(x) == 0:
-            x = np.zeros((0, 3))
+        _x = np.asarray(x, dtype=np.float64)
+        assert _x.ndim < 3
+        if len(_x) == 0:
+            _x = np.zeros((0, 3))
         else:
-            shape0 = x.shape[0] if x.ndim == 2 else 1
-            x = np.reshape(x, (shape0, -1))
-        num_points = x.shape[0]
-        if x.shape[1] != 3:
+            shape0 = _x.shape[0] if _x.ndim == 2 else 1
+            _x = np.reshape(_x, (shape0, -1))
+        num_points = _x.shape[0]
+        if _x.shape[1] != 3:
             raise ValueError("Coordinate(s) for Function evaluation must have length 3.")
 
         # Make sure cells are a NumPy array
-        cells = np.asarray(cells, dtype=np.int32)
-        assert cells.ndim < 2
-        num_points_c = cells.shape[0] if cells.ndim == 1 else 1
-        cells = np.reshape(cells, num_points_c)
+        _cells = np.asarray(cells, dtype=np.int32)
+        assert _cells.ndim < 2
+        num_points_c = _cells.shape[0] if _cells.ndim == 1 else 1
+        _cells = np.reshape(_cells, num_points_c)
 
         # Allocate memory for return value if not provided
         if u is None:
@@ -297,7 +298,7 @@ class Function(ufl.Coefficient):
             else:
                 u = np.empty((num_points, value_size))
 
-        self._cpp_object.eval(x, cells, u)
+        self._cpp_object.eval(_x, _cells, u)
         if num_points == 1:
             u = np.reshape(u, (-1, ))
         return u
@@ -307,20 +308,24 @@ class Function(ufl.Coefficient):
         """Interpolate an expression
 
         Args:
-            u: The function, Expression or Function to interpolate
+            u: The function, Expression or Function to interpolate.
             cells: The cells to interpolate over. If `None` then all
-                cells are interpolated over
+                cells are interpolated over.
 
         """
         @singledispatch
-        def _interpolate(u, cells):
-            try:
-                self._cpp_object.interpolate(u._cpp_object, cells)
-            except AttributeError:
-                self._cpp_object.interpolate(u, cells)
+        def _interpolate(u, cells: typing.Optional[np.ndarray] = None):
+            """Interpolate a cpp.fem.Function"""
+            self._cpp_object.interpolate(u, cells)
+
+        @_interpolate.register(Function)
+        def _(u: Function, cells: typing.Optional[np.ndarray] = None):
+            """Interpolate a fem.Function"""
+            self._cpp_object.interpolate(u._cpp_object, cells)
 
         @_interpolate.register(int)
-        def _(u_ptr, cells):
+        def _(u_ptr: int, cells: typing.Optional[np.ndarray] = None):
+            """Interpolate using a pointer to a function f(x)"""
             self._cpp_object.interpolate_ptr(u_ptr, cells)
 
         @_interpolate.register(Expression)
@@ -333,7 +338,14 @@ class Function(ufl.Coefficient):
             map = mesh.topology.index_map(mesh.topology.dim)
             cells = np.arange(map.size_local + map.num_ghosts, dtype=np.int32)
 
-        _interpolate(u, cells)
+        try:
+            # u is a Function or Expression (or pointer to one)
+            _interpolate(u, cells)
+        except TypeError:
+            # u is callable
+            assert callable(u)
+            x = _cpp.fem.interpolation_coords(self._V.element, self._V.mesh, cells)
+            self.interpolate(u(x), cells)
 
     def copy(self) -> Function:
         """Return a copy of the Function. The FunctionSpace is shared and the
@@ -378,8 +390,8 @@ class Function(ufl.Coefficient):
             i: The index of the sub-function to extract.
 
         Note:
-            The sub functions are numbered from i = 0..N-1, where N is
-            the total number of sub spaces.
+            The sub functions are numbered i = 0..N-1, where N is the
+            total number of sub spaces.
 
         """
         return Function(self._V.sub(i), self.x, name=f"{str(self)}_{i}")
@@ -411,15 +423,15 @@ class ElementMetaData(typing.NamedTuple):
     """Data for representing a finite element"""
     family: str
     degree: int
-    form_degree: typing.Optional[int] = None  # noqa
 
 
 class FunctionSpace(ufl.FunctionSpace):
     """A space on which Functions (fields) can be defined."""
 
-    def __init__(self, mesh: Mesh, element: typing.Union[ufl.FiniteElementBase, ElementMetaData],
+    def __init__(self, mesh: typing.Union[None, Mesh],
+                 element: typing.Union[ufl.FiniteElementBase, ElementMetaData, typing.Tuple[str, int]],
                  cppV: typing.Optional[_cpp.fem.FunctionSpace] = None,
-                 form_compiler_params: dict = {}, jit_params: dict = {}):
+                 form_compiler_params: dict[str, typing.Any] = {}, jit_params: dict[str, typing.Any] = {}):
         """Create a finite element function space."""
 
         # Create function space from a UFL element and existing cpp
@@ -431,26 +443,28 @@ class FunctionSpace(ufl.FunctionSpace):
             self._cpp_object = cppV
             return
 
-        # Initialise the ufl.FunctionSpace
-        if isinstance(element, ufl.FiniteElementBase):
-            super().__init__(mesh.ufl_domain(), element)
-        else:
-            e = ElementMetaData(*element)
-            ufl_element = ufl.FiniteElement(e.family, mesh.ufl_cell(), e.degree, form_degree=e.form_degree)
-            super().__init__(mesh.ufl_domain(), ufl_element)
+        if mesh is not None:
+            assert cppV is None
+            # Initialise the ufl.FunctionSpace
+            if isinstance(element, ufl.FiniteElementBase):
+                super().__init__(mesh.ufl_domain(), element)
+            else:
+                e = ElementMetaData(*element)
+                ufl_element = ufl.FiniteElement(e.family, mesh.ufl_cell(), e.degree)
+                super().__init__(mesh.ufl_domain(), ufl_element)
 
-        # Compile dofmap and element and create DOLFIN objects
-        (self._ufcx_element, self._ufcx_dofmap), module, code = jit.ffcx_jit(
-            mesh.comm, self.ufl_element(), form_compiler_params=form_compiler_params,
-            jit_params=jit_params)
+            # Compile dofmap and element and create DOLFIN objects
+            (self._ufcx_element, self._ufcx_dofmap), module, code = jit.ffcx_jit(
+                mesh.comm, self.ufl_element(), form_compiler_params=form_compiler_params,
+                jit_params=jit_params)
 
-        ffi = cffi.FFI()
-        cpp_element = _cpp.fem.FiniteElement(ffi.cast("uintptr_t", ffi.addressof(self._ufcx_element)))
-        cpp_dofmap = _cpp.fem.create_dofmap(mesh.comm, ffi.cast(
-            "uintptr_t", ffi.addressof(self._ufcx_dofmap)), mesh.topology, cpp_element)
+            ffi = module.ffi
+            cpp_element = _cpp.fem.FiniteElement(ffi.cast("uintptr_t", ffi.addressof(self._ufcx_element)))
+            cpp_dofmap = _cpp.fem.create_dofmap(mesh.comm, ffi.cast(
+                "uintptr_t", ffi.addressof(self._ufcx_dofmap)), mesh.topology, cpp_element)
 
-        # Initialize the cpp.FunctionSpace
-        self._cpp_object = _cpp.fem.FunctionSpace(mesh, cpp_element, cpp_dofmap)
+            # Initialize the cpp.FunctionSpace
+            self._cpp_object = _cpp.fem.FunctionSpace(mesh, cpp_element, cpp_dofmap)
 
     def clone(self) -> FunctionSpace:
         """Return a new FunctionSpace :math:`W` which shares data with this
@@ -539,7 +553,7 @@ class FunctionSpace(ufl.FunctionSpace):
         new to old dofs.
 
         Returns:
-            The new function space and the map from new to old dofs.
+            The new function space and the map from new to old degrees-of-freedom.
 
         """
         cpp_space, dofs = self._cpp_object.collapse()
@@ -550,12 +564,12 @@ class FunctionSpace(ufl.FunctionSpace):
         return self._cpp_object.tabulate_dof_coordinates()
 
 
-def VectorFunctionSpace(mesh: Mesh, element: ElementMetaData, dim=None,
+def VectorFunctionSpace(mesh: Mesh, element: typing.Union[ElementMetaData, typing.Tuple[str, int]], dim=None,
                         restriction=None) -> FunctionSpace:
     """Create vector finite element (composition of scalar elements) function space."""
 
     e = ElementMetaData(*element)
-    ufl_element = ufl.VectorElement(e.family, mesh.ufl_cell(), e.degree, form_degree=e.form_degree, dim=dim)
+    ufl_element = ufl.VectorElement(e.family, mesh.ufl_cell(), e.degree, dim=dim)
     return FunctionSpace(mesh, ufl_element)
 
 
