@@ -12,14 +12,9 @@ preparation."""
 
 from __future__ import annotations
 
-import typing
-
-if typing.TYPE_CHECKING:
-    from dolfinx.fem.forms import FormMetaClass
-    from dolfinx.fem.bcs import DirichletBCMetaClass
-
 import contextlib
 import functools
+import typing
 
 import ufl
 from dolfinx import cpp as _cpp
@@ -27,9 +22,12 @@ from dolfinx import la
 from dolfinx.cpp.fem import pack_coefficients as _pack_coefficients
 from dolfinx.cpp.fem import pack_constants as _pack_constants
 from dolfinx.fem import assemble
+from dolfinx.fem.bcs import DirichletBCMetaClass
 from dolfinx.fem.bcs import bcs_by_block as _bcs_by_block
+from dolfinx.fem.forms import FormMetaClass
 from dolfinx.fem.forms import extract_function_spaces as _extract_spaces
 from dolfinx.fem.forms import form as _create_form
+from dolfinx.fem.forms import form_types
 from dolfinx.fem.function import Function as _Function
 
 from petsc4py import PETSc
@@ -47,11 +45,11 @@ def _extract_function_spaces(a: typing.List[typing.List[FormMetaClass]]):
     def fn(form):
         return form.function_spaces if form is not None else None
     from functools import partial
-    Vblock = map(partial(map, fn), a)
+    Vblock: typing.Iterable = map(partial(map, fn), a)
 
     # Compute spaces for each row/column block
-    rows = [set() for i in range(len(a))]
-    cols = [set() for i in range(len(a[0]))]
+    rows: typing.List[typing.Set] = [set() for i in range(len(a))]
+    cols: typing.List[typing.Set] = [set() for i in range(len(a[0]))]
     for i, Vrow in enumerate(Vblock):
         for j, V in enumerate(Vrow):
             if V is not None:
@@ -68,7 +66,7 @@ def _extract_function_spaces(a: typing.List[typing.List[FormMetaClass]]):
 # -- Vector instantiation ----------------------------------------------------
 
 
-def create_vector(L: FormMetaClass) -> PETSc.Vec:
+def create_vector(L: form_types) -> PETSc.Vec:
     """Create a PETSc vector that is compaible with a linear form.
 
     Args:
@@ -82,7 +80,7 @@ def create_vector(L: FormMetaClass) -> PETSc.Vec:
     return la.create_petsc_vector(dofmap.index_map, dofmap.index_map_bs)
 
 
-def create_vector_block(L: typing.List[FormMetaClass]) -> PETSc.Vec:
+def create_vector_block(L: typing.List[form_types]) -> PETSc.Vec:
     """Create a PETSc vector (blocked) that is compaible with a list of linear forms.
 
     Args:
@@ -97,7 +95,7 @@ def create_vector_block(L: typing.List[FormMetaClass]) -> PETSc.Vec:
     return _cpp.fem.petsc.create_vector_block(maps)
 
 
-def create_vector_nest(L: typing.List[FormMetaClass]) -> PETSc.Vec:
+def create_vector_nest(L: typing.List[form_types]) -> PETSc.Vec:
     """Create a PETSc netsted vector (``VecNest``) that is compaible with a list of linear forms.
 
     Args:
@@ -115,7 +113,7 @@ def create_vector_nest(L: typing.List[FormMetaClass]) -> PETSc.Vec:
 
 # -- Matrix instantiation ----------------------------------------------------
 
-def create_matrix(a: FormMetaClass, mat_type=None) -> PETSc.Mat:
+def create_matrix(a: form_types, mat_type=None) -> PETSc.Mat:
     """Create a PETSc matrix that is compaible with a bilinear form.
 
     Args:
@@ -132,7 +130,7 @@ def create_matrix(a: FormMetaClass, mat_type=None) -> PETSc.Mat:
         return _cpp.fem.petsc.create_matrix(a, mat_type)
 
 
-def create_matrix_block(a: typing.List[typing.List[FormMetaClass]]) -> PETSc.Mat:
+def create_matrix_block(a: typing.List[typing.List[form_types]]) -> PETSc.Mat:
     """Create a PETSc matrix that is compaible with a rectangular array of bilinear forms.
 
     Args:
@@ -145,7 +143,7 @@ def create_matrix_block(a: typing.List[typing.List[FormMetaClass]]) -> PETSc.Mat
     return _cpp.fem.petsc.create_matrix_block(a)
 
 
-def create_matrix_nest(a: typing.List[typing.List[FormMetaClass]]) -> PETSc.Mat:
+def create_matrix_nest(a: typing.List[typing.List[form_types]]) -> PETSc.Mat:
     """Create a PETSc matrix (``MatNest``) that is compaible with a rectangular array of bilinear forms.
 
     Args:
@@ -161,7 +159,12 @@ def create_matrix_nest(a: typing.List[typing.List[FormMetaClass]]) -> PETSc.Mat:
 # -- Vector assembly ---------------------------------------------------------
 
 @functools.singledispatch
-def assemble_vector(L: FormMetaClass, constants=None, coeffs=None) -> PETSc.Vec:
+def assemble_vector(L: typing.Any, constants=None, coeffs=None) -> PETSc.Vec:
+    return _assemble_vector_form(L, constants, coeffs)
+
+
+@assemble_vector.register(FormMetaClass)
+def _assemble_vector_form(L: form_types, constants=None, coeffs=None) -> PETSc.Vec:
     """Assemble linear form into a new PETSc vector.
 
     Note:
@@ -178,12 +181,12 @@ def assemble_vector(L: FormMetaClass, constants=None, coeffs=None) -> PETSc.Vec:
     b = la.create_petsc_vector(L.function_spaces[0].dofmap.index_map,
                                L.function_spaces[0].dofmap.index_map_bs)
     with b.localForm() as b_local:
-        assemble.assemble_vector(b_local.array_w, L, constants, coeffs)
+        assemble._assemble_vector_array(b_local.array_w, L, constants, coeffs)
     return b
 
 
 @assemble_vector.register(PETSc.Vec)
-def _(b: PETSc.Vec, L: FormMetaClass, constants=None, coeffs=None) -> PETSc.Vec:
+def _assemble_vector_vec(b: PETSc.Vec, L: form_types, constants=None, coeffs=None) -> PETSc.Vec:
     """Assemble linear form into an existing PETSc vector.
 
     Note:
@@ -200,12 +203,17 @@ def _(b: PETSc.Vec, L: FormMetaClass, constants=None, coeffs=None) -> PETSc.Vec:
 
     """
     with b.localForm() as b_local:
-        assemble.assemble_vector(b_local.array_w, L, constants, coeffs)
+        assemble._assemble_vector_array(b_local.array_w, L, constants, coeffs)
     return b
 
 
 @functools.singledispatch
-def assemble_vector_nest(L: typing.List[FormMetaClass], constants=None, coeffs=None) -> PETSc.Vec:
+def assemble_vector_nest(L: typing.Any, constants=None, coeffs=None) -> PETSc.Vec:
+    return _assemble_vector_nest_forms(L, constants, coeffs)
+
+
+@assemble_vector_nest.register(list)
+def _assemble_vector_nest_forms(L: typing.List[form_types], constants=None, coeffs=None) -> PETSc.Vec:
     """Assemble linear forms into a new nested PETSc (VecNest) vector.
     The returned vector is not finalised, i.e. ghost values are not
     accumulated on the owning processes.
@@ -217,11 +225,11 @@ def assemble_vector_nest(L: typing.List[FormMetaClass], constants=None, coeffs=N
     for b_sub in b.getNestSubVecs():
         with b_sub.localForm() as b_local:
             b_local.set(0.0)
-    return assemble_vector_nest(b, L, constants, coeffs)
+    return _assemble_vector_nest_vec(b, L, constants, coeffs)
 
 
-@assemble_vector_nest.register(PETSc.Vec)
-def _(b: PETSc.Vec, L: typing.List[FormMetaClass], constants=None, coeffs=None) -> PETSc.Vec:
+@assemble_vector_nest.register
+def _assemble_vector_nest_vec(b: PETSc.Vec, L: typing.List[form_types], constants=None, coeffs=None) -> PETSc.Vec:
     """Assemble linear forms into a nested PETSc (VecNest) vector. The
     vector is not zeroed before assembly and it is not finalised, i.e.
     ghost values are not accumulated on the owning processes.
@@ -231,19 +239,30 @@ def _(b: PETSc.Vec, L: typing.List[FormMetaClass], constants=None, coeffs=None)
     coeffs = [None] * len(L) if coeffs is None else coeffs
     for b_sub, L_sub, const, coeff in zip(b.getNestSubVecs(), L, constants, coeffs):
         with b_sub.localForm() as b_local:
-            assemble.assemble_vector(b_local.array_w, L_sub, const, coeff)
+            assemble._assemble_vector_array(b_local.array_w, L_sub, const, coeff)
     return b
 
 
 # FIXME: Revise this interface
 @functools.singledispatch
-def assemble_vector_block(L: typing.List[FormMetaClass],
-                          a: typing.List[typing.List[FormMetaClass]],
+def assemble_vector_block(L: typing.Any,
+                          a: typing.List[typing.List[form_types]],
                           bcs: typing.List[DirichletBCMetaClass] = [],
                           x0: typing.Optional[PETSc.Vec] = None,
                           scale: float = 1.0,
                           constants_L=None, coeffs_L=None,
                           constants_a=None, coeffs_a=None) -> PETSc.Vec:
+    return _assemble_vector_block_form(L, a, bcs, x0, scale, constants_L, coeffs_L, constants_a, coeffs_a)
+
+
+@assemble_vector_block.register(list)
+def _assemble_vector_block_form(L: typing.List[form_types],
+                                a: typing.List[typing.List[form_types]],
+                                bcs: typing.List[DirichletBCMetaClass] = [],
+                                x0: typing.Optional[PETSc.Vec] = None,
+                                scale: float = 1.0,
+                                constants_L=None, coeffs_L=None,
+                                constants_a=None, coeffs_a=None) -> PETSc.Vec:
     """Assemble linear forms into a monolithic vector. The vector is not
     finalised, i.e. ghost values are not accumulated.
 
@@ -253,19 +272,19 @@ def assemble_vector_block(L: typing.List[FormMetaClass],
     b = _cpp.fem.petsc.create_vector_block(maps)
     with b.localForm() as b_local:
         b_local.set(0.0)
-    return assemble_vector_block(b, L, a, bcs, x0, scale, constants_L, coeffs_L,
-                                 constants_a, coeffs_a)
+    return _assemble_vector_block_vec(b, L, a, bcs, x0, scale, constants_L, coeffs_L,
+                                      constants_a, coeffs_a)
 
 
-@assemble_vector_block.register(PETSc.Vec)
-def _(b: PETSc.Vec,
-      L: typing.List[FormMetaClass],
-      a,
-      bcs: typing.List[DirichletBCMetaClass] = [],
-      x0: typing.Optional[PETSc.Vec] = None,
-      scale: float = 1.0,
-      constants_L=None, coeffs_L=None,
-      constants_a=None, coeffs_a=None) -> PETSc.Vec:
+@assemble_vector_block.register
+def _assemble_vector_block_vec(b: PETSc.Vec,
+                               L: typing.List[form_types],
+                               a: typing.List[typing.List[form_types]],
+                               bcs: typing.List[DirichletBCMetaClass] = [],
+                               x0: typing.Optional[PETSc.Vec] = None,
+                               scale: float = 1.0,
+                               constants_L=None, coeffs_L=None,
+                               constants_a=None, coeffs_a=None) -> PETSc.Vec:
     """Assemble linear forms into a monolithic vector. The vector is not
     zeroed and it is not finalised, i.e. ghost values are not
     accumulated.
@@ -311,24 +330,29 @@ def _(b: PETSc.Vec,
 
 
 # -- Matrix assembly ---------------------------------------------------------
-
-
 @functools.singledispatch
-def assemble_matrix(a: FormMetaClass, bcs: typing.List[DirichletBCMetaClass] = [],
+def assemble_matrix(a: typing.Any, bcs: typing.List[DirichletBCMetaClass] = [],
                     diagonal: float = 1.0,
                     constants=None, coeffs=None) -> PETSc.Mat:
+    return _assemble_matrix_form(a, bcs, diagonal, constants, coeffs)
+
+
+@assemble_matrix.register(FormMetaClass)
+def _assemble_matrix_form(a: form_types, bcs: typing.List[DirichletBCMetaClass] = [],
+                          diagonal: float = 1.0,
+                          constants=None, coeffs=None) -> PETSc.Mat:
     """Assemble bilinear form into a matrix. The returned matrix is not
     finalised, i.e. ghost values are not accumulated.
 
     """
     A = _cpp.fem.petsc.create_matrix(a)
-    assemble_matrix(A, a, bcs, diagonal, constants, coeffs)
+    _assemble_matrix_mat(A, a, bcs, diagonal, constants, coeffs)
     return A
 
 
-@assemble_matrix.register(PETSc.Mat)
-def _(A: PETSc.Mat, a: FormMetaClass, bcs: typing.List[DirichletBCMetaClass] = [],
-      diagonal: float = 1.0, constants=None, coeffs=None) -> PETSc.Mat:
+@assemble_matrix.register
+def _assemble_matrix_mat(A: PETSc.Mat, a: form_types, bcs: typing.List[DirichletBCMetaClass] = [],
+                         diagonal: float = 1.0, constants=None, coeffs=None) -> PETSc.Mat:
     """Assemble bilinear form into a matrix. The returned matrix is not
     finalised, i.e. ghost values are not accumulated.
 
@@ -345,20 +369,28 @@ def _(A: PETSc.Mat, a: FormMetaClass, bcs: typing.List[DirichletBCMetaClass] = [
 
 # FIXME: Revise this interface
 @functools.singledispatch
-def assemble_matrix_nest(a: typing.List[typing.List[FormMetaClass]],
+def assemble_matrix_nest(a: typing.Any,
                          bcs: typing.List[DirichletBCMetaClass] = [], mat_types=[],
                          diagonal: float = 1.0,
                          constants=None, coeffs=None) -> PETSc.Mat:
+    return _assemble_matrix_nest_form(a, bcs, diagonal, constants, coeffs)
+
+
+@assemble_matrix_nest.register(list)
+def _assemble_matrix_nest_form(a: typing.List[typing.List[form_types]],
+                               bcs: typing.List[DirichletBCMetaClass] = [], mat_types=[],
+                               diagonal: float = 1.0,
+                               constants=None, coeffs=None) -> PETSc.Mat:
     """Assemble bilinear forms into matrix"""
     A = _cpp.fem.petsc.create_matrix_nest(a, mat_types)
-    assemble_matrix_nest(A, a, bcs, diagonal, constants, coeffs)
+    _assemble_matrix_nest_mat(A, a, bcs, diagonal, constants, coeffs)
     return A
 
 
-@assemble_matrix_nest.register(PETSc.Mat)
-def _(A: PETSc.Mat, a: typing.List[typing.List[FormMetaClass]],
-      bcs: typing.List[DirichletBCMetaClass] = [], diagonal: float = 1.0,
-      constants=None, coeffs=None) -> PETSc.Mat:
+@assemble_matrix_nest.register
+def _assemble_matrix_nest_mat(A: PETSc.Mat, a: typing.List[typing.List[form_types]],
+                              bcs: typing.List[DirichletBCMetaClass] = [], diagonal: float = 1.0,
+                              constants=None, coeffs=None) -> PETSc.Mat:
     """Assemble bilinear forms into matrix"""
     constants = [[form and _pack_constants(form) for form in forms]
                  for forms in a] if constants is None else constants
@@ -368,10 +400,12 @@ def _(A: PETSc.Mat, a: typing.List[typing.List[FormMetaClass]],
         for j, (a_block, const, coeff) in enumerate(zip(a_row, const_row, coeff_row)):
             if a_block is not None:
                 Asub = A.getNestSubMatrix(i, j)
-                assemble_matrix(Asub, a_block, bcs, diagonal, const, coeff)
+                _assemble_matrix_mat(Asub, a_block, bcs, diagonal, const, coeff)
             elif i == j:
                 for bc in bcs:
-                    if a_row[0].function_spaces[0].contains(bc.function_space):
+                    row_forms = [row_form for row_form in a_row if row_form is not None]
+                    assert len(row_forms) > 0
+                    if row_forms[0].function_spaces[0].contains(bc.function_space):
                         raise RuntimeError(
                             f"Diagonal sub-block ({i}, {j}) cannot be 'None' and have DirichletBC applied."
                             " Consider assembling a zero block.")
@@ -380,19 +414,27 @@ def _(A: PETSc.Mat, a: typing.List[typing.List[FormMetaClass]],
 
 # FIXME: Revise this interface
 @functools.singledispatch
-def assemble_matrix_block(a: typing.List[typing.List[FormMetaClass]],
+def assemble_matrix_block(a: typing.Any,
                           bcs: typing.List[DirichletBCMetaClass] = [],
                           diagonal: float = 1.0,
                           constants=None, coeffs=None) -> PETSc.Mat:
+    return _assemble_matrix_block_form(a, bcs, diagonal, constants, coeffs)
+
+
+@assemble_matrix_block.register(list)
+def _assemble_matrix_block_form(a: typing.List[typing.List[form_types]],
+                                bcs: typing.List[DirichletBCMetaClass] = [],
+                                diagonal: float = 1.0,
+                                constants=None, coeffs=None) -> PETSc.Mat:
     """Assemble bilinear forms into matrix"""
     A = _cpp.fem.petsc.create_matrix_block(a)
-    return assemble_matrix_block(A, a, bcs, diagonal, constants, coeffs)
+    return _assemble_matrix_block_mat(A, a, bcs, diagonal, constants, coeffs)
 
 
-@assemble_matrix_block.register(PETSc.Mat)
-def _(A: PETSc.Mat, a: typing.List[typing.List[FormMetaClass]],
-      bcs: typing.List[DirichletBCMetaClass] = [], diagonal: float = 1.0,
-      constants=None, coeffs=None) -> PETSc.Mat:
+@assemble_matrix_block.register
+def _assemble_matrix_block_mat(A: PETSc.Mat, a: typing.List[typing.List[form_types]],
+                               bcs: typing.List[DirichletBCMetaClass] = [], diagonal: float = 1.0,
+                               constants=None, coeffs=None) -> PETSc.Mat:
     """Assemble bilinear forms into matrix"""
 
     constants = [[form and _pack_constants(form) for form in forms]
@@ -413,7 +455,9 @@ def _(A: PETSc.Mat, a: typing.List[typing.List[FormMetaClass]],
                 A.restoreLocalSubMatrix(is_rows[i], is_cols[j], Asub)
             elif i == j:
                 for bc in bcs:
-                    if a_row[0].function_spaces[0].contains(bc.function_space):
+                    row_forms = [row_form for row_form in a_row if row_form is not None]
+                    assert len(row_forms) > 0
+                    if row_forms[0].function_spaces[0].contains(bc.function_space):
                         raise RuntimeError(
                             f"Diagonal sub-block ({i}, {j}) cannot be 'None' and have DirichletBC applied."
                             " Consider assembling a zero block.")
@@ -435,9 +479,9 @@ def _(A: PETSc.Mat, a: typing.List[typing.List[FormMetaClass]],
 
 # -- Modifiers for Dirichlet conditions ---------------------------------------
 
-def apply_lifting(b: PETSc.Vec, a: typing.List[FormMetaClass],
+def apply_lifting(b: PETSc.Vec, a: typing.List[form_types],
                   bcs: typing.List[typing.List[DirichletBCMetaClass]],
-                  x0: typing.Optional[typing.List[PETSc.Vec]] = [],
+                  x0: typing.List[PETSc.Vec] = [],
                   scale: float = 1.0, constants=None, coeffs=None) -> None:
     """Apply the function :func:`dolfinx.fem.apply_lifting` to a PETSc Vector."""
     with contextlib.ExitStack() as stack:
@@ -447,7 +491,7 @@ def apply_lifting(b: PETSc.Vec, a: typing.List[FormMetaClass],
         assemble.apply_lifting(b_local.array_w, a, bcs, x0_r, scale, constants, coeffs)
 
 
-def apply_lifting_nest(b: PETSc.Vec, a: typing.List[typing.List[FormMetaClass]],
+def apply_lifting_nest(b: PETSc.Vec, a: typing.List[typing.List[form_types]],
                        bcs: typing.List[DirichletBCMetaClass],
                        x0: typing.Optional[PETSc.Vec] = None,
                        scale: float = 1.0, constants=None, coeffs=None) -> PETSc.Vec:
@@ -482,7 +526,7 @@ def set_bc_nest(b: PETSc.Vec, bcs: typing.List[typing.List[DirichletBCMetaClass]
         set_bc(b_sub, bc, x_sub, scale)
 
 
-class LinearProblem():
+class LinearProblem:
     """Class for solving a linear variational problem of the form :math:`a(u, v) = L(v) \\,  \\forall v \\in V`
     using PETSc as a linear algebra backend.
 
@@ -557,7 +601,7 @@ class LinearProblem():
 
         # Assemble lhs
         self._A.zeroEntries()
-        assemble_matrix(self._A, self._a, bcs=self.bcs)
+        _assemble_matrix_mat(self._A, self._a, bcs=self.bcs)
         self._A.assemble()
 
         # Assemble rhs
@@ -690,5 +734,5 @@ class NonlinearProblem:
 
         """
         A.zeroEntries()
-        assemble_matrix(A, self._a, self.bcs)
+        _assemble_matrix_mat(A, self._a, self.bcs)
         A.assemble()
