@@ -9,6 +9,7 @@
 #include "caster_mpi.h"
 #include "caster_petsc.h"
 #include <cfloat>
+#include <dolfinx/common/IndexMap.h>
 #include <dolfinx/fem/CoordinateElement.h>
 #include <dolfinx/fem/ElementDofLayout.h>
 #include <dolfinx/mesh/Geometry.h>
@@ -27,8 +28,8 @@
 #include <pybind11/numpy.h>
 #include <pybind11/pybind11.h>
 #include <pybind11/stl.h>
+#include <span>
 #include <xtensor/xadapt.hpp>
-#include <xtl/xspan.hpp>
 
 namespace py = pybind11;
 
@@ -122,7 +123,9 @@ void declare_meshtags(py::module& m, std::string type)
                                return py::array_t<std::int32_t>(
                                    self.indices().size(), self.indices().data(),
                                    py::cast(self));
-                             });
+                             })
+      .def("find", [](dolfinx::mesh::MeshTags<T>& self, T value)
+           { return as_pyarray(self.find(value)); });
 
   m.def("create_meshtags",
         [](const std::shared_ptr<const dolfinx::mesh::Mesh>& mesh, int dim,
@@ -130,7 +133,7 @@ void declare_meshtags(py::module& m, std::string type)
            const py::array_t<T, py::array::c_style>& values)
         {
           return dolfinx::mesh::create_meshtags(
-              mesh, dim, entities, xtl::span(values.data(), values.size()));
+              mesh, dim, entities, std::span(values.data(), values.size()));
         });
 }
 
@@ -147,23 +150,32 @@ void mesh(py::module& m)
       .value("prism", dolfinx::mesh::CellType::prism)
       .value("hexahedron", dolfinx::mesh::CellType::hexahedron);
 
-  m.def("to_type", &dolfinx::mesh::to_type);
-  m.def("to_string", &dolfinx::mesh::to_string);
-  m.def("is_simplex", &dolfinx::mesh::is_simplex);
+  m.def("to_type", &dolfinx::mesh::to_type, py::arg("cell"));
+  m.def("to_string", &dolfinx::mesh::to_string, py::arg("type"));
+  m.def("is_simplex", &dolfinx::mesh::is_simplex, py::arg("type"));
 
-  m.def("cell_entity_type", &dolfinx::mesh::cell_entity_type);
-  m.def("cell_dim", &dolfinx::mesh::cell_dim);
-  m.def("cell_num_entities", &dolfinx::mesh::cell_num_entities);
-  m.def("cell_num_vertices", &dolfinx::mesh::num_cell_vertices);
-  m.def("cell_normals",
-        [](const dolfinx::mesh::Mesh& mesh, int dim,
-           const py::array_t<std::int32_t, py::array::c_style>& entities)
-        {
-          return xt_as_pyarray(dolfinx::mesh::cell_normals(
-              mesh, dim, xtl::span(entities.data(), entities.size())));
-        });
-  m.def("get_entity_vertices", &dolfinx::mesh::get_entity_vertices);
-  m.def("extract_topology", &dolfinx::mesh::extract_topology);
+  m.def("cell_entity_type", &dolfinx::mesh::cell_entity_type, py::arg("type"),
+        py::arg("dim"), py::arg("index"));
+  m.def("cell_dim", &dolfinx::mesh::cell_dim, py::arg("type"));
+  m.def("cell_num_entities", &dolfinx::mesh::cell_num_entities, py::arg("type"),
+        py::arg("dim"));
+  m.def("cell_num_vertices", &dolfinx::mesh::num_cell_vertices,
+        py::arg("type"));
+  m.def(
+      "cell_normals",
+      [](const dolfinx::mesh::Mesh& mesh, int dim,
+         const py::array_t<std::int32_t, py::array::c_style>& entities)
+      {
+        std::vector<double> n = dolfinx::mesh::cell_normals(
+            mesh, dim, std::span(entities.data(), entities.size()));
+        return as_pyarray(std::move(n),
+                          std::array<std::size_t, 2>{n.size() / 3, 3});
+      },
+      py::arg("mesh"), py::arg("dim"), py::arg("entities"));
+  m.def("get_entity_vertices", &dolfinx::mesh::get_entity_vertices,
+        py::arg("type"), py::arg("dim"));
+  m.def("extract_topology", &dolfinx::mesh::extract_topology,
+        py::arg("cell_type"), py::arg("layout"), py::arg("cells"));
 
   m.def(
       "h",
@@ -171,18 +183,22 @@ void mesh(py::module& m)
          const py::array_t<std::int32_t, py::array::c_style>& entities)
       {
         return as_pyarray(dolfinx::mesh::h(
-            mesh, xtl::span(entities.data(), entities.size()), dim));
+            mesh, std::span(entities.data(), entities.size()), dim));
       },
+      py::arg("mesh"), py::arg("dim"), py::arg("entities"),
       "Compute maximum distance between any two vertices.");
 
-  m.def("compute_midpoints",
-        [](const dolfinx::mesh::Mesh& mesh, int dim,
-           py::array_t<std::int32_t, py::array::c_style> entity_list)
-        {
-          return xt_as_pyarray(dolfinx::mesh::compute_midpoints(
-              mesh, dim, xtl::span(entity_list.data(), entity_list.size())));
-        });
-  m.def("compute_boundary_facets", &dolfinx::mesh::compute_boundary_facets);
+  m.def(
+      "compute_midpoints",
+      [](const dolfinx::mesh::Mesh& mesh, int dim,
+         py::array_t<std::int32_t, py::array::c_style> entities)
+      {
+        std::vector<double> x = dolfinx::mesh::compute_midpoints(
+            mesh, dim, std::span(entities.data(), entities.size()));
+        std::array<std::size_t, 2> shape = {(std::size_t)entities.size(), 3};
+        return as_pyarray(std::move(x), shape);
+      },
+      py::arg("mesh"), py::arg("dim"), py::arg("entities"));
 
   using PythonPartitioningFunction
       = std::function<dolfinx::graph::AdjacencyList<std::int32_t>(
@@ -195,6 +211,7 @@ void mesh(py::module& m)
       [](const MPICommWrapper comm,
          const dolfinx::graph::AdjacencyList<std::int64_t>& cells, int tdim)
       { return dolfinx::mesh::build_dual_graph(comm.get(), cells, tdim); },
+      py::arg("comm"), py::arg("cells"), py::arg("tdim"),
       "Build dual graph for cells");
 
   m.def(
@@ -217,19 +234,25 @@ void mesh(py::module& m)
 
         std::size_t shape1 = x.ndim() == 1 ? 1 : x.shape()[1];
         std::vector shape{std::size_t(x.shape(0)), shape1};
-        auto _x = xt::adapt(x.data(), x.size(), xt::no_ownership(), shape);
-        return dolfinx::mesh::create_mesh(comm.get(), cells, element, _x,
-                                          ghost_mode, partitioner_wrapper);
+        return dolfinx::mesh::create_mesh(
+            comm.get(), cells, element, std::span(x.data(), x.size()),
+            {static_cast<std::size_t>(x.shape(0)),
+             static_cast<std::size_t>(x.shape(1))},
+            ghost_mode, partitioner_wrapper);
       },
+      py::arg("comm"), py::arg("cells"), py::arg("element"), py::arg("x"),
+      py::arg("ghost_mode"), py::arg("partitioner"),
       "Helper function for creating meshes.");
 
-  m.def("create_submesh",
-        [](const dolfinx::mesh::Mesh& mesh, int dim,
-           const py::array_t<std::int32_t, py::array::c_style>& entities)
-        {
-          return dolfinx::mesh::create_submesh(
-              mesh, dim, xtl::span(entities.data(), entities.size()));
-        });
+  m.def(
+      "create_submesh",
+      [](const dolfinx::mesh::Mesh& mesh, int dim,
+         const py::array_t<std::int32_t, py::array::c_style>& entities)
+      {
+        return dolfinx::mesh::create_submesh(
+            mesh, dim, std::span(entities.data(), entities.size()));
+      },
+      py::arg("mesh"), py::arg("dim"), py::arg("entities"));
 
   // dolfinx::mesh::GhostMode enums
   py::enum_<dolfinx::mesh::GhostMode>(m, "GhostMode")
@@ -259,23 +282,32 @@ void mesh(py::module& m)
                              &dolfinx::mesh::Geometry::input_global_indices);
 
   // dolfinx::mesh::TopologyComputation
-  m.def("compute_entities", [](const MPICommWrapper comm,
-                               const dolfinx::mesh::Topology& topology, int dim)
-        { return dolfinx::mesh::compute_entities(comm.get(), topology, dim); });
-  m.def("compute_connectivity", &dolfinx::mesh::compute_connectivity);
+  m.def(
+      "compute_entities",
+      [](const MPICommWrapper comm, const dolfinx::mesh::Topology& topology,
+         int dim)
+      { return dolfinx::mesh::compute_entities(comm.get(), topology, dim); },
+      py::arg("comm"), py::arg("topology"), py::arg("dim"));
+  m.def("compute_connectivity", &dolfinx::mesh::compute_connectivity,
+        py::arg("topology"), py::arg("d0"), py::arg("d1"));
 
   // dolfinx::mesh::Topology class
   py::class_<dolfinx::mesh::Topology, std::shared_ptr<dolfinx::mesh::Topology>>(
       m, "Topology", py::dynamic_attr(), "Topology object")
-      .def(py::init(
-          [](const MPICommWrapper comm, const dolfinx::mesh::CellType cell_type)
-          { return dolfinx::mesh::Topology(comm.get(), cell_type); }))
-      .def("set_connectivity", &dolfinx::mesh::Topology::set_connectivity)
-      .def("set_index_map", &dolfinx::mesh::Topology::set_index_map)
-      .def("create_entities", &dolfinx::mesh::Topology::create_entities)
+      .def(py::init([](const MPICommWrapper comm,
+                       const dolfinx::mesh::CellType cell_type)
+                    { return dolfinx::mesh::Topology(comm.get(), cell_type); }),
+           py::arg("comm"), py::arg("cell_type"))
+      .def("set_connectivity", &dolfinx::mesh::Topology::set_connectivity,
+           py::arg("c"), py::arg("d0"), py::arg("d1"))
+      .def("set_index_map", &dolfinx::mesh::Topology::set_index_map,
+           py::arg("dim"), py::arg("map"))
+      .def("create_entities", &dolfinx::mesh::Topology::create_entities,
+           py::arg("dim"))
       .def("create_entity_permutations",
            &dolfinx::mesh::Topology::create_entity_permutations)
-      .def("create_connectivity", &dolfinx::mesh::Topology::create_connectivity)
+      .def("create_connectivity", &dolfinx::mesh::Topology::create_connectivity,
+           py::arg("d0"), py::arg("d1"))
       .def("get_facet_permutations",
            [](const dolfinx::mesh::Topology& self)
            {
@@ -303,8 +335,9 @@ void mesh(py::module& m)
                              })
       .def("connectivity",
            py::overload_cast<int, int>(&dolfinx::mesh::Topology::connectivity,
-                                       py::const_))
-      .def("index_map", &dolfinx::mesh::Topology::index_map)
+                                       py::const_),
+           py::arg("d0"), py::arg("d1"))
+      .def("index_map", &dolfinx::mesh::Topology::index_map, py::arg("dim"))
       .def_property_readonly("cell_type", &dolfinx::mesh::Topology::cell_type)
       .def("cell_name", [](const dolfinx::mesh::Topology& self)
            { return dolfinx::mesh::to_string(self.cell_type()); })
@@ -315,9 +348,11 @@ void mesh(py::module& m)
   py::class_<dolfinx::mesh::Mesh, std::shared_ptr<dolfinx::mesh::Mesh>>(
       m, "Mesh", py::dynamic_attr(), "Mesh object")
       .def(py::init(
-          [](const MPICommWrapper comm, const dolfinx::mesh::Topology& topology,
-             dolfinx::mesh::Geometry& geometry)
-          { return dolfinx::mesh::Mesh(comm.get(), topology, geometry); }))
+               [](const MPICommWrapper comm,
+                  const dolfinx::mesh::Topology& topology,
+                  dolfinx::mesh::Geometry& geometry)
+               { return dolfinx::mesh::Mesh(comm.get(), topology, geometry); }),
+           py::arg("comm"), py::arg("topology"), py::arg("geometry"))
       .def_property_readonly(
           "geometry", py::overload_cast<>(&dolfinx::mesh::Mesh::geometry),
           "Mesh geometry")
@@ -347,71 +382,88 @@ void mesh(py::module& m)
           return create_cell_partitioner_py(
               dolfinx::mesh::create_cell_partitioner());
         });
-  m.def("create_cell_partitioner",
-        [](const std::function<dolfinx::graph::AdjacencyList<std::int32_t>(
-               MPICommWrapper comm, int nparts,
-               const dolfinx::graph::AdjacencyList<std::int64_t>& local_graph,
-               bool ghosting)>& part) -> PythonCellPartitionFunction
-        {
-          return create_cell_partitioner_py(
-              dolfinx::mesh::create_cell_partitioner(
-                  create_cell_partitioner_cpp(part)));
-        });
+  m.def(
+      "create_cell_partitioner",
+      [](const std::function<dolfinx::graph::AdjacencyList<std::int32_t>(
+             MPICommWrapper comm, int nparts,
+             const dolfinx::graph::AdjacencyList<std::int64_t>& local_graph,
+             bool ghosting)>& part) -> PythonCellPartitionFunction
+      {
+        return create_cell_partitioner_py(
+            dolfinx::mesh::create_cell_partitioner(
+                create_cell_partitioner_cpp(part)));
+      },
+      py::arg("part"));
 
-  m.def("locate_entities",
-        [](const dolfinx::mesh::Mesh& mesh, int dim,
-           const std::function<py::array_t<bool>(
-               const py::array_t<double, py::array::c_style>&)>& marker)
+  m.def(
+      "locate_entities",
+      [](const dolfinx::mesh::Mesh& mesh, int dim,
+         const std::function<py::array_t<bool>(
+             const py::array_t<double, py::array::c_style>&)>& marker)
+      {
+        auto cpp_marker
+            = [&marker](const xt::xtensor<double, 2>& x) -> xt::xtensor<bool, 1>
         {
-          auto cpp_marker =
-              [&marker](const xt::xtensor<double, 2>& x) -> xt::xtensor<bool, 1>
-          {
-            py::array_t<double> x_view(x.shape(), x.data(), py::none());
-            py::array_t<bool> marked = marker(x_view);
-            std::array shape = {static_cast<std::size_t>(marked.size())};
-            return xt::adapt(marked.data(), marked.size(), xt::no_ownership(),
-                             shape);
-          };
-          return as_pyarray(
-              dolfinx::mesh::locate_entities(mesh, dim, cpp_marker));
-        });
+          py::array_t<double> x_view(x.shape(), x.data(), py::none());
+          py::array_t<bool> marked = marker(x_view);
+          std::array shape = {static_cast<std::size_t>(marked.size())};
+          return xt::adapt(marked.data(), marked.size(), xt::no_ownership(),
+                           shape);
+        };
+        return as_pyarray(
+            dolfinx::mesh::locate_entities(mesh, dim, cpp_marker));
+      },
+      py::arg("mesh"), py::arg("dim"), py::arg("marker"));
 
-  m.def("locate_entities_boundary",
-        [](const dolfinx::mesh::Mesh& mesh, int dim,
-           const std::function<py::array_t<bool>(
-               const py::array_t<double, py::array::c_style>&)>& marker)
+  m.def(
+      "locate_entities_boundary",
+      [](const dolfinx::mesh::Mesh& mesh, int dim,
+         const std::function<py::array_t<bool>(
+             const py::array_t<double, py::array::c_style>&)>& marker)
+      {
+        auto cpp_marker
+            = [&marker](const xt::xtensor<double, 2>& x) -> xt::xtensor<bool, 1>
         {
-          auto cpp_marker =
-              [&marker](const xt::xtensor<double, 2>& x) -> xt::xtensor<bool, 1>
-          {
-            py::array_t<double> x_view(x.shape(), x.data(), py::none());
-            py::array_t<bool> marked = marker(x_view);
-            std::array shape = {static_cast<std::size_t>(marked.size())};
-            return xt::adapt(marked.data(), marked.size(), xt::no_ownership(),
-                             shape);
-          };
-          return as_pyarray(
-              dolfinx::mesh::locate_entities_boundary(mesh, dim, cpp_marker));
-        });
+          py::array_t<double> x_view(x.shape(), x.data(), py::none());
+          py::array_t<bool> marked = marker(x_view);
+          std::array shape = {static_cast<std::size_t>(marked.size())};
+          return xt::adapt(marked.data(), marked.size(), xt::no_ownership(),
+                           shape);
+        };
+        return as_pyarray(
+            dolfinx::mesh::locate_entities_boundary(mesh, dim, cpp_marker));
+      },
+      py::arg("mesh"), py::arg("dim"), py::arg("marker"));
 
-  m.def("entities_to_geometry",
-        [](const dolfinx::mesh::Mesh& mesh, int dim,
-           py::array_t<std::int32_t, py::array::c_style> entity_list,
-           bool orient)
-        {
-          return xt_as_pyarray(dolfinx::mesh::entities_to_geometry(
-              mesh, dim, xtl::span(entity_list.data(), entity_list.size()),
-              orient));
-        });
-  m.def("exterior_facet_indices", &dolfinx::mesh::exterior_facet_indices);
-  m.def("compute_incident_entities",
-        [](const dolfinx::mesh::Mesh& mesh,
-           py::array_t<std::int32_t, py::array::c_style> entity_list, int d0,
-           int d1)
-        {
-          return as_pyarray(dolfinx::mesh::compute_incident_entities(
-              mesh, xtl::span(entity_list.data(), entity_list.size()), d0, d1));
-        });
+  m.def(
+      "entities_to_geometry",
+      [](const dolfinx::mesh::Mesh& mesh, int dim,
+         py::array_t<std::int32_t, py::array::c_style> entities, bool orient)
+      {
+        std::vector<std::int32_t> idx = dolfinx::mesh::entities_to_geometry(
+            mesh, dim, std::span(entities.data(), entities.size()), orient);
+        dolfinx::mesh::CellType cell_type = mesh.topology().cell_type();
+        std::size_t num_vertices = dolfinx::mesh::num_cell_vertices(
+            cell_entity_type(cell_type, dim, 0));
+        std::array<std::size_t, 2> shape
+            = {(std::size_t)entities.size(), num_vertices};
+        return as_pyarray(std::move(idx), shape);
+      },
+      py::arg("mesh"), py::arg("dim"), py::arg("entities"), py::arg("orient"));
+  m.def(
+      "exterior_facet_indices",
+      [](const dolfinx::mesh::Topology& t)
+      { return as_pyarray(dolfinx::mesh::exterior_facet_indices(t)); },
+      py::arg("topology"));
+  m.def(
+      "compute_incident_entities",
+      [](const dolfinx::mesh::Mesh& mesh,
+         py::array_t<std::int32_t, py::array::c_style> entities, int d0, int d1)
+      {
+        return as_pyarray(dolfinx::mesh::compute_incident_entities(
+            mesh, std::span(entities.data(), entities.size()), d0, d1));
+      },
+      py::arg("mesh"), py::arg("entities"), py::arg("d0"), py::arg("d1"));
 
   // Mesh generation
   py::enum_<dolfinx::mesh::DiagonalType>(m, "DiagonalType")
