@@ -17,9 +17,9 @@ from mpi4py import MPI
 
 # Supported XDMF file encoding
 if MPI.COMM_WORLD.size > 1:
-    encodings = (XDMFFile.Encoding.HDF5, )
+    encodings = [XDMFFile.Encoding.HDF5]
 else:
-    encodings = (XDMFFile.Encoding.ASCII, XDMFFile.Encoding.HDF5)
+    encodings = [XDMFFile.Encoding.ASCII, XDMFFile.Encoding.HDF5]
 
 celltypes_3D = [CellType.tetrahedron, CellType.hexahedron]
 
