@@ -7,7 +7,7 @@
 
 from dolfinx.cpp.la import SparsityPattern
 from dolfinx.fem import VectorFunctionSpace, locate_dofs_topological
-from dolfinx.mesh import compute_boundary_facets, create_unit_square
+from dolfinx.mesh import create_unit_square, exterior_facet_indices
 
 from mpi4py import MPI
 
@@ -19,7 +19,7 @@ def test_add_diagonal():
     pattern = SparsityPattern(mesh.comm, [V.dofmap.index_map, V.dofmap.index_map],
                               [V.dofmap.index_map_bs, V.dofmap.index_map_bs])
     mesh.topology.create_connectivity(mesh.topology.dim - 1, mesh.topology.dim)
-    facets = compute_boundary_facets(mesh.topology)
+    facets = exterior_facet_indices(mesh.topology)
     blocks = locate_dofs_topological(V, mesh.topology.dim - 1, facets)
     pattern.insert_diagonal(blocks)
     pattern.assemble()
