@@ -12,9 +12,10 @@ import numpy as np
 import pytest
 
 import ufl
-from dolfinx.cpp.io import perm_gmsh, perm_vtk
+from dolfinx.cpp.io import perm_vtk
 from dolfinx.fem import assemble_scalar, form
-from dolfinx.io import XDMFFile, ufl_mesh_from_gmsh
+from dolfinx.io import XDMFFile
+from dolfinx.io.gmshio import cell_perm_array, ufl_mesh
 from dolfinx.mesh import CellType, create_mesh
 from ufl import dx
 
@@ -162,9 +163,7 @@ def test_quadrilateral_mesh(order):
             for i in range(1, order):
                 cell.append(coord_to_vertex(i, j))
 
-    domain = ufl.Mesh(ufl.VectorElement(
-        "Q", ufl.Cell("quadrilateral", geometric_dimension=2), order))
-
+    domain = ufl.Mesh(ufl.VectorElement("Q", ufl.Cell("quadrilateral", geometric_dimension=2), order))
     check_cell_volume(points, cell, domain, 1)
 
 
@@ -240,9 +239,7 @@ def test_hexahedron_mesh(order):
                 for i in range(1, order):
                     cell.append(coord_to_vertex(i, j, k))
 
-    domain = ufl.Mesh(ufl.VectorElement(
-        "Q", ufl.Cell("hexahedron", geometric_dimension=3), order))
-
+    domain = ufl.Mesh(ufl.VectorElement("Q", ufl.Cell("hexahedron", geometric_dimension=3), order))
     check_cell_volume(points, cell, domain, 1)
 
 
@@ -279,10 +276,7 @@ def test_triangle_mesh_vtk(order):
             raise NotImplementedError
 
     cell = np.array(cell)[perm_vtk(CellType.triangle, len(cell))]
-
-    domain = ufl.Mesh(ufl.VectorElement(
-        "Lagrange", ufl.Cell("triangle", geometric_dimension=2), order))
-
+    domain = ufl.Mesh(ufl.VectorElement("Lagrange", ufl.Cell("triangle", geometric_dimension=2), order))
     check_cell_volume(points, cell, domain, 0.5)
 
 
@@ -374,10 +368,7 @@ def test_tetrahedron_mesh_vtk(order):
                         cell.append(coord_to_vertex(i, j, k))
 
     cell = np.array(cell)[perm_vtk(CellType.tetrahedron, len(cell))]
-
-    domain = ufl.Mesh(ufl.VectorElement(
-        "Lagrange", ufl.Cell("tetrahedron", geometric_dimension=3), order))
-
+    domain = ufl.Mesh(ufl.VectorElement("Lagrange", ufl.Cell("tetrahedron", geometric_dimension=3), order))
     check_cell_volume(points, cell, domain, 1 / 6)
 
 
@@ -414,10 +405,7 @@ def test_quadrilateral_mesh_vtk(order):
                 cell.append(coord_to_vertex(i, j))
 
     cell = np.array(cell)[perm_vtk(CellType.quadrilateral, len(cell))]
-
-    domain = ufl.Mesh(ufl.VectorElement(
-        "Q", ufl.Cell("quadrilateral", geometric_dimension=2), order))
-
+    domain = ufl.Mesh(ufl.VectorElement("Q", ufl.Cell("quadrilateral", geometric_dimension=2), order))
     check_cell_volume(points, cell, domain, 1)
 
 
@@ -501,10 +489,7 @@ def test_hexahedron_mesh_vtk(order):
                     cell.append(coord_to_vertex(i, j, k))
 
     cell = np.array(cell)[perm_vtk(CellType.hexahedron, len(cell))]
-
-    domain = ufl.Mesh(ufl.VectorElement(
-        "Q", ufl.Cell("hexahedron", geometric_dimension=3), order))
-
+    domain = ufl.Mesh(ufl.VectorElement("Q", ufl.Cell("hexahedron", geometric_dimension=3), order))
     check_cell_volume(points, cell, domain, 1)
 
 
@@ -573,27 +558,12 @@ def test_gmsh_input_2d(order, cell_type):
         gmsh_cell_id = gmsh.model.mesh.getElementType("quadrangle", order)
     gmsh.finalize()
 
-    cells = cells[:, perm_gmsh(cell_type, cells.shape[1])]
-    mesh = create_mesh(MPI.COMM_WORLD, cells, x, ufl_mesh_from_gmsh(gmsh_cell_id, x.shape[1]))
+    cells = cells[:, cell_perm_array(cell_type, cells.shape[1])]
+    mesh = create_mesh(MPI.COMM_WORLD, cells, x, ufl_mesh(gmsh_cell_id, x.shape[1]))
     surface = assemble_scalar(form(1 * dx(mesh)))
 
     assert mesh.comm.allreduce(surface, op=MPI.SUM) == pytest.approx(4 * np.pi, rel=10 ** (-1 - order))
 
-    # Bug related to VTK output writing
-    # def e2(x):
-    #     values = np.empty((x.shape[0], 1))
-    #     values[:, 0] = x[:, 0]
-    #     return values
-    # cmap = fem.create_coordinate_map(mesh.comm, mesh.ufl_domain())
-    # mesh.geometry.coord_mapping = cmap
-    # V = FunctionSpace(mesh, ("Lagrange", order))
-    # u = Function(V)
-    # u.interpolate(e2)
-    # from dolfinx.io import VTKFile
-    # VTKFile("u{0:d}.pvd".format(order)).write(u)
-    # print(min(u.vector.array),max(u.vector.array))
-    # print(assemble_scalar(u*dx(mesh)))
-
 
 @pytest.mark.skip_in_parallel
 @pytest.mark.parametrize('order', range(1, 4))
@@ -604,7 +574,7 @@ def test_gmsh_input_3d(order, cell_type):
     except ImportError:
         pytest.skip()
     if cell_type == CellType.hexahedron and order > 2:
-        pytest.xfail("GMSH permutation for order > 2 hexahedra not implemented in DOLFINx.")
+        pytest.xfail("Gmsh permutation for order > 2 hexahedra not implemented in DOLFINx.")
 
     res = 0.2
 
@@ -644,14 +614,12 @@ def test_gmsh_input_3d(order, cell_type):
         gmsh_cell_id = MPI.COMM_WORLD.bcast(gmsh.model.mesh.getElementType("hexahedron", order), root=0)
     gmsh.finalize()
 
-    # Permute the mesh topology from GMSH ordering to DOLFINx ordering
-    domain = ufl_mesh_from_gmsh(gmsh_cell_id, 3)
-    cells = cells[:, perm_gmsh(cell_type, cells.shape[1])]
+    # Permute the mesh topology from Gmsh ordering to DOLFINx ordering
+    domain = ufl_mesh(gmsh_cell_id, 3)
+    cells = cells[:, cell_perm_array(cell_type, cells.shape[1])]
 
     mesh = create_mesh(MPI.COMM_WORLD, cells, x, domain)
-
     volume = assemble_scalar(form(1 * dx(mesh)))
-
     assert mesh.comm.allreduce(volume, op=MPI.SUM) == pytest.approx(np.pi, rel=10 ** (-1 - order))
 
 
@@ -668,8 +636,5 @@ def test_quadrilateral_cell_order_3():
     ]
 
     cell = list(range(16))
-
-    domain = ufl.Mesh(ufl.VectorElement(
-        "Lagrange", ufl.Cell("quadrilateral", geometric_dimension=2), 3))
-
+    domain = ufl.Mesh(ufl.VectorElement("Lagrange", ufl.Cell("quadrilateral", geometric_dimension=2), 3))
     check_cell_volume(points, cell, domain, 5 / 6)
